<?php

namespace App\Http\Controllers\Accounting;

use App\Http\Controllers\Controller;
use App\Http\Requests\Accounting\AccountRequest;
use App\Http\Requests\Accounting\LedgerRequest;
use Illuminate\Http\Request;
use App\Http\Resources\Accounting\Ledger as LedgerResource;
use App\Models\Accounting\Ledger;
use App\Repositories\Accounting\LedgerRepository;

class LedgerController extends Controller
{
    private $request;
    private $repo;

    /**
     * Instantiate a new controller instance
     * @return void
     */
    public function __construct(
        Request $request,
        LedgerRepository $repo
    ) {
        $this->request  = $request;
        $this->repo     = $repo;
    }

    /**
     * Get pre requisites
     * @get ("/api/accounting/ledgers/pre-requisite")
     * @return array
     */
    public function preRequisite()
    {
        return $this->ok($this->repo->getPreRequisite());
    }

    /**
     * Get all ledgers
     * @get ("/api/accounting/ledgers")
     * @return array
     */
    public function index()
    {
        $this->authorize('viewAny', Ledger::class);

        return $this->repo->paginate();
    }

    /**
     * Search the resource.
     * @get ("/api/accounting/ledgers/search")
     * @return array
     */
    public function search()
    {
        return $this->ok($this->repo->search());
    }

    /**
     * Store ledger
     * @post ("/api/accounting/ledgers")
     * @param ({
     *      @Parameter("name", type="string", required="true", description="Ledger name"),
     *      @Parameter("alias", type="string", required="optional", description="Ledger alias"),
     *      @Parameter("ledger_type_id", type="integer", required="true", description="Ledger type"),
     *      @Parameter("description", type="text", required="optional", description="Ledger description"),
     * })
     * @return array
     */
    public function store(LedgerRequest $request)
    {
        $this->authorize('create', Ledger::class);

        $ledger = new LedgerResource($this->repo->create());

        return $this->success(['message' => __('global.added', ['attribute' => __('accounting.ledger.ledger')]), 'ledger' => $ledger]);
    }

    /**
     * Get ledger detail
     * @get ("/api/accounting/ledgers/{ledger}")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     * })
     * @return LedgerResource
     */
    public function show(Ledger $ledger)
    {
        $this->authorize('view', $ledger);

        return new LedgerResource($ledger);
    }

    /**
     * Add contact to ledger
     * @post ("/api/accounting/ledgers/{ledger}/contacts")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     * })
     * @return LedgerResource
     */
    public function addContact(Ledger $ledger)
    {
        $this->authorize('update', $ledger);

        $ledger->addContact();

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger.ledger')])]);
    }

    /**
     * Remove contact from ledger
     * @delete ("/api/accounting/ledgers/{ledger}/contacts")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     * })
     * @return LedgerResource
     */
    public function removeContact(Ledger $ledger)
    {
        $this->authorize('update', $ledger);

        $ledger->removeContact();

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger.ledger')])]);
    }

    /**
     * Add account to ledger
     * @post ("/api/accounting/ledgers/{ledger}/accounts")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     * })
     * @return LedgerResource
     */
    public function addAccount(Ledger $ledger, AccountRequest $request)
    {
        $this->authorize('update', $ledger);

        $this->repo->addAccount($ledger);

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger.ledger')])]);
    }

    /**
     * Update ledger account
     * @patch ("/api/accounting/ledgers/{ledger}/accounts/{account}")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     *      @Parameter("account", type="uuid", required="true", description="Ledger account unique id"),
     * })
     * @return LedgerResource
     */
    public function updateAccount(Ledger $ledger, $uuid)
    {
        $this->authorize('update', $ledger);

        $this->repo->updateAccount($ledger, $uuid);

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger.ledger')])]);
    }

    /**
     * Remove account from ledger
     * @delete ("/api/accounting/ledgers/{ledger}/accounts")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     * })
     * @return LedgerResource
     */
    public function removeAccount(Ledger $ledger)
    {
        $this->authorize('update', $ledger);

        $this->repo->removeAccount($ledger);

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger.ledger')])]);
    }

    /**
     * Update ledger
     * @patch ("/api/accounting/ledgers/{ledger}")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     *      @Parameter("name", type="string", required="true", description="Ledger name"),
     *      @Parameter("alias", type="string", required="optional", description="Ledger alias"),
     *      @Parameter("ledger_type_id", type="integer", required="true", description="Ledger type"),
     *      @Parameter("description", type="text", required="optional", description="Ledger description"),
     * })
     * @return array
     */
    public function update(LedgerRequest $request, Ledger $ledger)
    {
        $this->authorize('update', $ledger);

        $this->repo->update($ledger);

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger.ledger')])]);
    }

    /**
     * Delete ledger
     * @delete ("/api/accounting/ledgers/{ledger}")
     * @param ({
     *      @Parameter("ledger", type="uuid", required="true", description="Ledger unique id"),
     * })
     * @return array
     */
    public function destroy(Ledger $ledger)
    {
        $this->authorize('delete', $ledger);

        $this->repo->delete($ledger);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('accounting.ledger.ledger')])]);
    }
}
