<?php

namespace App\Http\Controllers\Accounting;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests\Accounting\LedgerTypeRequest;
use App\Models\Accounting\LedgerType;
use App\Repositories\Accounting\LedgerTypeRepository;
use App\Http\Resources\Accounting\LedgerType as LedgerTypeResource;

class LedgerTypeController extends Controller
{
    private $request;
    private $repo;

    /**
     * Instantiate a new controller instance
     * @return void
     */
    public function __construct(
        Request $request,
        LedgerTypeRepository $repo
    ) {
        $this->request  = $request;
        $this->repo     = $repo;

        $this->middleware('permission:manage-ledger-type')->except(['search']);
    }

    /**
     * Get pre requisites
     * @get ("/api/accounting/ledger-types/pre-requisite")
     * @return array
     */
    public function preRequisite()
    {
        return $this->ok($this->repo->getPreRequisite());
    }

    /**
     * Get all ledger types
     * @get ("/api/accounting/ledger-types")
     * @return array
     */
    public function index()
    {
        return $this->repo->paginate();
    }

    /**
     * Search the resource.
     * @get ("/api/accounting/ledger-types/search")
     * @return array
     */
    public function search()
    {
        return $this->ok($this->repo->search());
    }

    /**
     * Store ledger type
     * @post ("/api/accounting/ledger-types")
     * @param ({
     *      @Parameter("name", type="string", required="true", description="Ledger type name"),
     *      @Parameter("alias", type="string", required="optional", description="Ledger type alias"),
     *      @Parameter("parent_id", type="integer", required="true", description="Ledger type parent"),
     *      @Parameter("description", type="text", required="optional", description="Ledger type description"),
     * })
     * @return array
     */
    public function store(LedgerTypeRequest $request)
    {
        $ledger_type = new LedgerTypeResource($this->repo->create());

        return $this->success(['message' => __('global.added', ['attribute' => __('accounting.ledger_type.ledger_type')]), 'ledger_type' => $ledger_type]);
    }

    /**
     * Get ledger type detail
     * @get ("/api/accounting/ledger-types/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Ledger type unique id"),
     * })
     * @return LedgerTypeResource
     */
    public function show(LedgerType $ledger_type)
    {
        return new LedgerTypeResource($ledger_type);
    }

    /**
     * Update ledger type
     * @patch ("/api/accounting/ledger-types/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Ledger type unique id"),
     *      @Parameter("name", type="string", required="true", description="Ledger type name"),
     *      @Parameter("alias", type="string", required="optional", description="Ledger type alias"),
     *      @Parameter("parent_id", type="integer", required="true", description="Ledger type parent"),
     *      @Parameter("description", type="text", required="optional", description="Ledger type description"),
     * })
     * @return array
     */
    public function update(LedgerTypeRequest $request, LedgerType $ledger_type)
    {
        $this->repo->update($ledger_type);

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger_type.ledger_type')])]);
    }

    /**
     * Update ledger type role
     * @post ("/api/accounting/ledger-types/{uuid}/role")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Ledger type unique id"),
     *      @Parameter("roles", type="array", required="true", description="Array of roles"),
     * })
     * @return array
     */
    public function updateRole(LedgerType $ledger_type)
    {
        $this->repo->updateRole($ledger_type);

        return $this->success(['message' => __('global.updated', ['attribute' => __('accounting.ledger_type.ledger_type')])]);
    }

    /**
     * Delete ledger type
     * @delete ("/api/accounting/ledger-types/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Ledger type unique id"),
     * })
     * @return array
     */
    public function destroy(LedgerType $ledger_type)
    {
        $this->repo->delete($ledger_type);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('accounting.ledger_type.ledger_type')])]);
    }
}