<?php

namespace App\Http\Controllers\Accounting;

use Illuminate\Http\Request;
use App\Models\Accounting\Voucher;
use App\Http\Controllers\Controller;
use App\Http\Requests\Accounting\VoucherRequest;
use App\Repositories\Accounting\VoucherRepository;
use App\Http\Resources\Accounting\Voucher as VoucherResource;
use App\Http\Resources\Accounting\VoucherSummary as VoucherSummaryResource;

class VoucherController extends Controller
{
    private $request;
    private $repo;

    /**
     * Instantiate a new controller instance
     * @return void
     */
    public function __construct(
        Request $request,
        VoucherRepository $repo
    ) {
        $this->request  = $request;
        $this->repo     = $repo;
    }

    /**
     * Get pre requisites
     * @get ("/api/accounting/voucher/{voucher}/pre-requisite")
     * @return array
     */
    public function preRequisite()
    {
        return $this->ok($this->repo->getPreRequisite());
    }

    /**
     * Get all vouchers
     * @get ("/api/accounting/voucher/{voucher}")
     * @return array
     */
    public function index()
    {
        $this->authorize('viewAny', Voucher::class);

        return $this->repo->paginate();
    }

    /**
     * Store voucher
     * @post ("/api/accounting/voucher/{voucher}")
     * @param ({
     *      @Parameter("voucher", type="array", required="true", description="Voucher"),
     * })
     * @return array
     */
    public function store(VoucherRequest $request)
    {
        $this->authorize('create', Voucher::class);

        $voucher = new VoucherSummaryResource($this->repo->create());

        return $this->success(['message' => __('global.added', ['attribute' => __('voucher.voucher')]), 'voucher' => $voucher]);
    }

    /**
     * Get voucher detail
     * @get ("/api/accounting/voucher/{voucher}/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Voucher unique id"),
     * })
     * @return VoucherResource
     */
    public function show($type, $uuid)
    {
        $voucher = $this->repo->findByUuidOrFail($uuid);

        $this->authorize('view', $voucher);

        $voucher->load('details', 'details.item', 'referral', 'transport', 'followUps');

        return new VoucherResource($voucher);
    }

    /**
     * Update voucher
     * @patch ("/api/accounting/voucher/{voucher}/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Voucher unique id"),
     *      @Parameter("voucher", type="array", required="true", description="Voucher"),
     * })
     * @return array
     */
    public function update(VoucherRequest $request, $type, $uuid)
    {
        $voucher = $this->repo->findByUuidOrFail($uuid);

        $this->authorize('update', $voucher);

        $this->repo->update($voucher);

        return $this->success(['message' => __('global.updated', ['attribute' => __('voucher.voucher')])]);
    }

    /**
     * Delete voucher
     * @delete ("/api/accounting/voucher/{voucher}/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Voucher unique id"),
     * })
     * @return array
     */
    public function destroy($type, $uuid)
    {
        $voucher = $this->repo->findByUuidOrFail($uuid);

        $this->authorize('delete', $voucher);

        $this->repo->delete($voucher);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('voucher.voucher')])]);
    }
}
