<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Requests\ContactRequest;
use Illuminate\Http\Request;
use App\Http\Resources\Contact as ContactResource;
use App\Models\Contact;
use App\Repositories\ContactRepository;

class ContactController extends Controller
{
    private $request;
    private $repo;

    /**
     * Instantiate a new controller instance
     * @return void
     */
    public function __construct(
        Request $request,
        ContactRepository $repo
    ) {
        $this->request  = $request;
        $this->repo     = $repo;
    }

    /**
     * Get pre requisites
     * @get ("/api/contacts/pre-requisite")
     * @return array
     */
    public function preRequisite()
    {
        return $this->ok($this->repo->getPreRequisite());
    }

    /**
     * Get all contacts
     * @get ("/api/contacts")
     * @return array
     */
    public function index()
    {
        $this->authorize('viewAny', Contact::class);

        return $this->repo->paginate();
    }

    /**
     * Search the resource.
     * @get ("/api/contacts/search")
     * @return array
     */
    public function search()
    {
        return $this->ok($this->repo->search());
    }

    /**
     * Store contact
     * @post ("/api/contacts")
     * @param ({
     *      @Parameter("name", type="string", required="true", description="Contact name"),
     *      @Parameter("alias", type="string", required="optional", description="Contact alias"),
     *      @Parameter("mobile", type="string", required="optional", description="Contact mobile"),
     * })
     * @return array
     */
    public function store(ContactRequest $request)
    {
        $this->authorize('create', Contact::class);

        $contact = new ContactResource($this->repo->create());

        return $this->success(['message' => __('global.added', ['attribute' => __('contact.contact')]), 'contact' => $contact]);
    }

    /**
     * Get contact detail
     * @get ("/api/contacts/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Contact unique id"),
     * })
     * @return ContactResource
     */
    public function show(Contact $contact)
    {
        $this->authorize('view', $contact);

        $contact->load('ledgers', 'brands');

        return new ContactResource($contact);
    }

    /**
     * Update contact
     * @patch ("/api/contacts/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Contact unique id"),
     *      @Parameter("name", type="string", required="true", description="Contact name"),
     *      @Parameter("alias", type="string", required="optional", description="Contact alias"),
     *      @Parameter("mobile", type="string", required="optional", description="Contact mobile"),
     * })
     * @return array
     */
    public function update(ContactRequest $request, Contact $contact)
    {
        $this->authorize('update', $contact);

        $this->repo->update($contact);

        return $this->success(['message' => __('global.updated', ['attribute' => __('contact.contact')])]);
    }

    /**
     * Delete contact
     * @delete ("/api/contacts/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Contact unique id"),
     * })
     * @return array
     */
    public function destroy(Contact $contact)
    {
        $this->authorize('delete', $contact);

        $this->repo->delete($contact);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('contact.contact')])]);
    }
}