<?php

namespace App\Http\Controllers\Inventory;

use App\Models\Inventory\Brand;
use App\Http\Controllers\Controller;
use App\Http\Requests\Inventory\BrandRequest;
use App\Repositories\Inventory\BrandRepository;
use App\Http\Resources\Inventory\Brand as BrandResource;
use Illuminate\Http\Request;

class BrandController extends Controller
{
    protected $repo;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        BrandRepository $repo
    ) {
        $this->repo = $repo;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $this->authorize('viewAny', Brand::class);

        return $this->repo->paginate();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(BrandRequest $request)
    {
        $this->authorize('create', Brand::class);

        $brand = $this->repo->create();

        $brand = new BrandResource($brand);

        return $this->success(['message' => __('global.added', ['attribute' => __('inventory.brand.brand')]), 'brand' => $brand]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Brand  $brand
     * @return \Illuminate\Http\Response
     */
    public function show(Brand $brand)
    {
        $this->authorize('view', Brand::class);

        $brand->load('contacts');

        return new BrandResource($brand);
    }

    /**
     * Add contact to the specified resource.
     *
     * @param  \App\Models\Brand  $brand
     * @return \Illuminate\Http\Response
     */
    public function addContact(Brand $brand)
    {
        $this->authorize('update', $brand);

        $brand->addContact();

        return $this->success(['message' => __('global.updated', ['attribute' => __('inventory.brand.brand')])]);
    }

    /**
     * Remove contact from the specified resource.
     *
     * @param  \App\Models\Brand  $brand
     * @return \Illuminate\Http\Response
     */
    public function removeContact(Brand $brand)
    {
        $this->authorize('update', $brand);

        $brand->removeContact();

        return $this->success(['message' => __('global.updated', ['attribute' => __('inventory.brand.brand')])]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Brand  $brand
     * @return \Illuminate\Http\Response
     */
    public function update(BrandRequest $request, Brand $brand)
    {
        $this->authorize('update', $brand);

        $brand = $this->repo->update($brand);

        return $this->success(['message' => __('global.updated', ['attribute' => __('inventory.brand.brand')])]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Brand  $brand
     * @return \Illuminate\Http\Response
     */
    public function destroy(Brand $brand)
    {
        $this->authorize('delete', $brand);

        $this->repo->delete($brand);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('inventory.brand.brand')])]);
    }
}