<?php

namespace App\Http\Controllers\Inventory;

use App\Models\Inventory\ItemCategory;
use App\Http\Controllers\Controller;
use App\Http\Requests\Inventory\ItemCategoryRequest;
use App\Repositories\Inventory\ItemCategoryRepository;
use App\Http\Resources\Inventory\ItemCategory as ItemCategoryResource;
use Illuminate\Http\Request;

class ItemCategoryController extends Controller
{
    protected $repo;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        ItemCategoryRepository $repo
    ) {
        $this->repo = $repo;
    }

    /**
     * Get pre requisites
     * @get ("/api/inventory/item-categories/pre-requisite")
     * @return array
     */
    public function preRequisite()
    {
        return $this->ok($this->repo->getPreRequisite());
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $this->authorize('viewAny', ItemCategory::class);

        return $this->repo->paginate();
    }

    /**
     * Search the resource.
     * @get ("/api/inventory/item-categories/search")
     * @return array
     */
    public function search()
    {
        $this->authorize('viewAny', ItemCategory::class);

        return $this->ok($this->repo->search());
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(ItemCategoryRequest $request)
    {
        $this->authorize('create', ItemCategory::class);

        $item_category = $this->repo->create();

        $item_category = new ItemCategoryResource($item_category);

        return $this->success(['message' => __('global.added', ['attribute' => __('inventory.item_category.item_category')]), 'item_category' => $item_category]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\ItemCategory  $item_category
     * @return \Illuminate\Http\Response
     */
    public function show(ItemCategory $item_category)
    {
        $this->authorize('view', ItemCategory::class);

        $item_category = $this->repo->findByUuidOrFail($item_category->uuid);

        return new ItemCategoryResource($item_category);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\ItemCategory  $item_category
     * @return \Illuminate\Http\Response
     */
    public function update(ItemCategoryRequest $request, ItemCategory $item_category)
    {
        $this->authorize('update', $item_category);

        $item_category = $this->repo->update($item_category);

        return $this->success(['message' => __('global.updated', ['attribute' => __('inventory.item_category.item_category')])]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\ItemCategory  $item_category
     * @return \Illuminate\Http\Response
     */
    public function destroy(ItemCategory $item_category)
    {
        $this->authorize('delete', $item_category);

        $this->repo->delete($item_category);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('inventory.item_category.item_category')])]);
    }
}
