<?php

namespace App\Http\Controllers\Inventory;

use App\Models\Inventory\Item;
use App\Http\Controllers\Controller;
use App\Http\Requests\Inventory\ItemRequest;
use App\Repositories\Inventory\ItemRepository;
use App\Http\Resources\Inventory\Item as ItemResource;
use Illuminate\Http\Request;

class ItemController extends Controller
{
    protected $repo;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        ItemRepository $repo
    ) {
        $this->repo = $repo;
    }

    /**
     * Get pre requisites
     * @get ("/api/inventory/items/pre-requisite")
     * @return array
     */
    public function preRequisite()
    {
        return $this->ok($this->repo->getPreRequisite());
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $this->authorize('viewAny', Item::class);

        return $this->repo->paginate();
    }

    /**
     * Search the resource.
     * @get ("/api/inventory/items/search")
     * @return array
     */
    public function search()
    {
        $this->authorize('viewAny', Item::class);
        
        return $this->ok($this->repo->search());
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(ItemRequest $request)
    {
        $this->authorize('create', Item::class);

        $item = $this->repo->create();

        $item = new ItemResource($item);

        return $this->success(['message' => __('global.added', ['attribute' => __('inventory.item.item')]), 'item' => $item]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Item  $item
     * @return \Illuminate\Http\Response
     */
    public function show(Item $item)
    {
        $this->authorize('view', Item::class);

        return new ItemResource($item);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Item  $item
     * @return \Illuminate\Http\Response
     */
    public function update(ItemRequest $request, Item $item)
    {
        $this->authorize('update', $item);

        $item = $this->repo->update($item);

        return $this->success(['message' => __('global.updated', ['attribute' => __('inventory.item.item')])]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Item  $item
     * @return \Illuminate\Http\Response
     */
    public function destroy(Item $item)
    {
        $this->authorize('delete', $item);

        $this->repo->delete($item);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('inventory.item.item')])]);
    }
}