<?php

namespace App\Http\Controllers\Inventory;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests\Inventory\StockMatchRequest;
use App\Http\Resources\Inventory\StockMatch as StockMatchResource;
use App\Http\Resources\Inventory\StockMatchSummary;
use App\Models\Inventory\StockMatch;
use App\Repositories\Inventory\StockMatchRepository;

class StockMatchController extends Controller
{
    private $request;
    private $repo;

    /**
     * Instantiate a new controller instance
     * @return void
     */
    public function __construct(
        Request $request,
        StockMatchRepository $repo
    ) {
        $this->request  = $request;
        $this->repo     = $repo;
    }

    /**
     * Get all stock matches
     * @get ("/api/inventory/stock-matches/{stock_match}")
     * @return array
     */
    public function index()
    {
        $this->authorize('viewAny', StockMatch::class);

        return $this->repo->paginate();
    }

    /**
     * Store stock match
     * @post ("/api/inventory/stock-matches/{stock_match}")
     * @param ({
     *      @Parameter("date", type="array", required="true", description="Date"),
     * })
     * @return array
     */
    public function store(StockMatchRequest $request)
    {
        $this->authorize('create', StockMatch::class);

        $stock_match = new StockMatchSummary($this->repo->create());

        return $this->success(['message' => __('global.added', ['attribute' => __('inventory.stock_match.stock_match')]), 'stock_match' => $stock_match]);
    }

    /**
     * Get stock match detail
     * @get ("/api/inventory/stock-matches/{stock_match}/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Stock match unique id"),
     * })
     * @return StockMatchResource
     */
    public function show($uuid)
    {
        $stock_match = $this->repo->findByUuidOrFail($uuid);

        $this->authorize('view', $stock_match);

        $stock_match->load('details', 'details.item');

        return new StockMatchResource($stock_match);
    }

    /**
     * Update stock match
     * @patch ("/api/inventory/stock-matches/{stock_match}/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Stock match unique id"),
     *      @Parameter("date", type="array", required="true", description="date"),
     * })
     * @return array
     */
    public function update(StockMatchRequest $request, $uuid)
    {
        $stock_match = $this->repo->findByUuidOrFail($uuid);

        $this->authorize('update', $stock_match);

        $this->repo->update($stock_match);

        return $this->success(['message' => __('global.updated', ['attribute' => __('inventory.stock_match.stock_match')])]);
    }

    /**
     * Delete stock match
     * @delete ("/api/inventory/stock-matches/{stock_match}/{uuid}")
     * @param ({
     *      @Parameter("uuid", type="uuid", required="true", description="Stock match unique id"),
     * })
     * @return array
     */
    public function destroy($uuid)
    {
        $stock_match = $this->repo->findByUuidOrFail($uuid);

        $this->authorize('delete', $stock_match);

        $this->repo->delete($stock_match);

        return $this->success(['message' => __('global.deleted', ['attribute' => __('inventory.stock_match.stock_match')])]);
    }
}
