<?php

namespace App\Http\Requests\Accounting;

use App\Enums\VoucherType;
use App\Traits\Validation;
use Illuminate\Foundation\Http\FormRequest;

class VoucherRequest extends FormRequest
{
    use Validation;

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        $rules = [
            'date'                   => 'required|date',
            'due_date'               => 'date|after_or_equal:date',
            'delivery_date'          => 'date|after_or_equal:date',
            'expected_delivery_date' => 'date|after_or_equal:date',
            'referral'               => 'nullable|array',
            'transport'              => 'nullable|array',
            'enquiry_source'         => 'nullable|array',
            'enquiry_type'           => 'nullable|array',
            'items'                  => 'required_if:is_published,1|array|min:1',
            'items.*.item.uuid'      => 'required_if:is_published,1|required_unless:items.*.custom,1|distinct',
            'items.*.item.name'      => 'required_if:is_published,1,items.*.custom,1|distinct',
            'items.*.item.unit.code' => 'required_if:is_published,1,items.*.custom,1',
            'items.*.tax.value'      => 'numeric|min:0',
            'items.*.discount'       => 'numeric|min:0',
            'discounts'              => 'array',
            'discounts.*.name'       => 'required|min:2|distinct',
            'discounts.*.value'      => 'numeric|min:0',
            'taxes'                  => 'array',
            'taxes.*.tax.uuid'       => 'required|distinct',
            'taxes.*.tax.value'      => 'numeric|min:0',
            'shipping'               => 'numeric|min:0',
            'round_off'              => 'numeric'
        ];

        $rules = $this->addVoucherLevelRules($rules);

        return $rules;
    }

    private function addVoucherLevelRules($rules = array())
    {
        if (! in_array(request('type'), [VoucherType::PURCHASE_ENQUIRY, VoucherType::SALE_ENQUIRY, VoucherType::PURCHASE])) {
            $rules['code_number'] = 'required';
        }

        if (in_array(request('type'), [VoucherType::PURCHASE_ENQUIRY, VoucherType::SALE_ENQUIRY])) {
            $rules['contact'] = 'required|array';
        } else {
            $rules['billing_info.billing_name'] = 'required_if:is_published,1|min:3';
            $rules['items.*.quantity'] = 'required_if:is_published,1|numeric|min:0';
        }

        if (! in_array(request('type'), [VoucherType::PURCHASE_ENQUIRY, VoucherType::SALE_ENQUIRY, VoucherType::PROCUREMENT])) {
            $rules['ledger'] = 'required_if:is_published,1|array';
        }

        if (! in_array(request('type'), [VoucherType::PROCUREMENT])) {
            $rules['items.*.rate'] = 'required_if:is_published,1|numeric|min:0';
        }

        if (in_array(request('type'), [VoucherType::QUOTATION, VoucherType::PROCUREMENT])) {
            $rules['subject'] = 'required';
        }

        return $rules;
    }

    /**
     * Translate fields with user friendly name.
     *
     * @return array
     */
    public function attributes()
    {
        return [
            'items.*.item.uuid' => trans('inventory.item.item'),
            'taxes.*.tax.uuid' => trans('setting.tax.tax'),
            'items.*.item.unit.code' => trans('setting.unit.unit'),
            'items.*.custom' => trans('inventory.item.props.custom_item')
        ];
    }

    public function withValidator($validator)
    {
        $validator->after(function ($validator) {
            $this->change($validator, 'items', 'message');
            $this->simplify($validator, 'items.*.item.uuid', 'items.*.item');
            $this->simplify($validator, 'items.*.item.name', 'items.*');
            $this->simplify($validator, 'items.*.item.unit.code', 'items.*');
            $this->simplify($validator, 'items.*.tax.value', 'items.*.tax');
            $this->simplify($validator, 'taxes.*.tax.uuid', 'taxes.*.tax');
            $this->simplify($validator, 'taxes.*.tax.value', 'taxes.*.tax');
        });
    }

    /**
     * Translate rules with user friendly name.
     *
     * @return array
     */
    public function messages()
    {
        return [
            'code_number.required_if'       => trans('validation.required', ['attribute' => trans('purchase.props.code_number')]),
            'due_date.required_if'          => trans('validation.required', ['attribute' => trans('purchase.props.due_date')]),
            'delivery_date.required_if'     => trans('validation.required', ['attribute' => trans('purchase.props.delivery_date')]),
            'ledger.required_if'            => trans('validation.required', ['attribute' => trans('accounting.ledger.ledger')]),
            'items.required_if'             => trans('validation.required', ['attribute' => trans('inventory.item.item')]),
            'items.*.item.uuid.required_if' => trans('validation.required', ['attribute' => trans('inventory.item.item')]),
            'items.*.item.uuid.required_unless' => trans('validation.required', ['attribute' => trans('inventory.item.item')]),
            'items.*.item.name.required_if' => trans('validation.required', ['attribute' => trans('inventory.item.props.name')]),
            'items.*.item.unit.code.required_if' => trans('validation.required', ['attribute' => trans('setting.unit.unit')]),
            'items.*.quantity.required_if'  => trans('validation.required', ['attribute' => trans('inventory.item.props.quantity')]),
            'items.*.rate.required_if'      => trans('validation.required', ['attribute' => trans('purchase.props.rate')]),
            'items.*.discount.required_if'  => trans('validation.required', ['attribute' => trans('setting.discount.discount')]),
            'items.*.tax.value.required_if'  => trans('validation.required', ['attribute' => trans('setting.tax.tax')]),
            'billing_info.billing_name.required_if' => trans('validation.required', ['attribute' => trans('accounting.ledger.props.billing_name')])
        ];
    }
}
