<?php

namespace App\Http\Resources\Inventory;

use App\Helpers\CalHelper;
use App\Helpers\SysHelper;
use App\Http\Resources\Setting\Margin;
use App\Http\Resources\Setting\Tax;
use App\Http\Resources\Setting\Unit;
use App\Http\Resources\TagCollection;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Str;
use Illuminate\Support\Arr;

class Item extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function toArray($request)
    {
        $variants = $this->variants ? : [];

        $item_category_variants = $this->category->variants;

        $variants = collect($variants)->transform(function($variant) use ($item_category_variants) {
            $type = Arr::get($variant, 'type', 'text');

            $options = Arr::get($variant, 'options', []);
            $value = null;
            if (in_array(Arr::get($type, 'uuid'), ['single_select', 'multi_select'])) {
                $options = collect($options)->transform(function($option) {
                    $option_name = Arr::get($option, 'name');
                    return array('uuid' => Str::slug($option_name), 'name' => $option_name);
                })->all();

            } else {
                $value = Arr::get($variant, 'value');
            }

            $variant_name = Arr::get($variant, 'name');
            return array('uuid' => Str::slug($variant_name), 'name' => $variant_name, 'options' => $options, 'value' => $value);
        });

        $margin = $this->margin_id ? $this->category->margin : $this->category->margin;
        $current_rate_taxed = $this->current_rate_taxed ? :  $this->opening_rate_taxed;
        $current_rate_non_taxed = $this->current_rate_non_taxed ? :  $this->opening_rate_non_taxed;

        if ($this->is_service) {
            $current_rate_taxed = $this->opening_rate_taxed;
            $current_rate_non_taxed = $this->opening_rate_non_taxed;
        }

        $taxed_price = $this->getPrice($margin, $current_rate_taxed);
        $non_taxed_price = $this->getPrice($margin, $current_rate_non_taxed, true);

        $taxed_rate = (float) Arr::get($taxed_price, 'rate_without_tax', 0);
        $non_taxed_rate = (float) Arr::get($non_taxed_price, 'rate_without_tax', 0);

        return [
            'uuid'                          => $this->uuid,
            'name'                          => $this->name,
            'alias'                         => $this->alias,
            'brand'                         => $this->when($this->is_product, new Brand($this->whenLoaded('brand'))),
            'item_category'                 => new ItemCategory($this->whenLoaded('category')),
            'type'                          => array('uuid' => $this->type, 'name' => trans('list.inventory.types.' . $this->type)),
            'unit'                          => $this->unit_id ? new Unit($this->unit) : new Unit($this->category->unit),
            'tax'                           => $this->tax_id ? new Tax($this->tax) : new Tax($this->category->tax),
            'margin'                        => $this->margin_id ? new Margin($this->margin) : new Margin($this->category->margin),
            'price_calculation_method'      => $this->when($this->price_calculation_method, array('uuid' => $this->price_calculation_method, 'name' => trans('list.inventory.price_calculation_methods.' . $this->price_calculation_method))),
            'opening_quantity_taxed'        => $this->when($this->is_product, $this->opening_quantity_taxed),
            'opening_rate_taxed'            => $this->opening_rate_taxed,
            'opening_quantity_non_taxed'    => $this->when($this->is_product, $this->opening_quantity_non_taxed),
            'opening_rate_non_taxed'        => $this->when($this->is_product, $this->opening_rate_non_taxed),
            'current_quantity_taxed'        => $this->when($this->is_product, $this->current_quantity_taxed),
            'current_quantity_non_taxed'    => $this->when($this->is_product, $this->current_quantity_non_taxed),
            'current_rate_taxed'            => $this->when($this->showCurrentRate(), $current_rate_taxed),
            'current_rate_non_taxed'        => $this->when($this->showCurrentRate(), $current_rate_non_taxed),
            'rate_taxed'                    => $taxed_rate,
            'rate_non_taxed'                => $non_taxed_rate,
            'price_taxed'                   => (float) Arr::get($taxed_price, 'price_with_tax'),
            'discounted_price_taxed'        => (float) Arr::get($taxed_price, 'discounted_price_with_tax'),
            'margin_percentage_taxed'       => (float) Arr::get($taxed_price, 'margin_percentage'),
            'discount_percentage_taxed'     => (float) Arr::get($taxed_price, 'discount_percentage'),
            'price_non_taxed'               => (float) Arr::get($non_taxed_price, 'price_with_tax'),
            'discounted_price_non_taxed'    => (float) Arr::get($non_taxed_price, 'discounted_price_with_tax'),
            'margin_percentage_non_taxed'   => (float) Arr::get($non_taxed_price, 'margin_percentage'),
            'discount_percentage_non_taxed' => (float) Arr::get($non_taxed_price, 'discount_percentage'),
            'net_quantity_taxed'            => $this->when($this->is_product, $this->opening_quantity_taxed + $this->current_quantity_taxed),
            'net_quantity_non_taxed'        => $this->when($this->is_product, $this->opening_quantity_non_taxed + $this->current_quantity_non_taxed),
            'mrp'                           => SysHelper::formatCurrency($this->mrp),
            'warranty'                      => $this->warranty,
            'reorder_level'                 => $this->is_product ? $this->reorder_level : null,
            'requires_reorder'              => $this->when($this->is_product, ($this->opening_quantity_taxed + $this->current_quantity_taxed) > $this->reorder_level ? false : true),
            'identifier'                    => $this->identifier,
            'variants'                      => $variants,
            'description'                   => $this->description,
            'tags'                          => new TagCollection($this->tags),
            'created_at'                    => CalHelper::toDateTime($this->created_at),
            'updated_at'                    => CalHelper::toDateTime($this->updated_at)
        ];
    }

    private function showCurrentRate() : bool
    {
        if (\Auth::user()->hasAnyPermission(['create-purchase', 'create-purchase-enquiry', 'edit-purchase', 'edit-purchase-enqiury'])) {
            return true;
        }

        return false;
    }

    private function getTaxValue()
    {
        $tax = $this->tax_id ? $this->tax : ($this->category->tax_id ? $this->category->tax : null);
        return optional($tax)->value ?? 0;
    }

    private function getPrice($margin, $rate, $non_taxed = false)
    {
        $tax_value = $this->getTaxValue();

        if ($non_taxed) {
            $tax_value = 0;
        }

        $margin_percentage = 15;
        $discount_percentage = 0;

        if ($margin) {
            $margin_detail = $margin->details->where('min_amount', '<=', $rate)->where('max_amount', '>=', $rate)->first();

            if ($margin_detail) {
                $margin_percentage = (float) $margin_detail->value;
                $discount_percentage = Arr::get(Arr::first($margin_detail->discounts ?? [], function($item) {
                    return Arr::get($item, 'name') == \Auth::user()->getRole();
                }), 'value', 0);
            }
        }

        $price = SysHelper::formatCurrency($rate + (($rate * $margin_percentage) / 100));
        $price_with_tax = SysHelper::formatCurrency($price + (($price * $tax_value) / 100));

        if ($price_with_tax > $this->mrp) {
            $price_with_tax = $this->mrp;
        }

        $margin_with_discount_percentage = $margin_percentage - SysHelper::formatCurrency(($margin_percentage * $discount_percentage) / 100);

        $discounted_price = SysHelper::formatCurrency($rate + (($rate * ($margin_with_discount_percentage)) / 100));
        $discounted_price_with_tax = SysHelper::formatCurrency($discounted_price + (($discounted_price * $tax_value) / 100));

        if ($discounted_price_with_tax > $this->mrp) {
            $discounted_price_with_tax = $this->mrp;
        }

        if (config('config.item.round_off_price_' . ($non_taxed ? 'non_taxed' : 'taxed'))) {
            $price_with_tax = round($price_with_tax, 0);
        }

        if (config('config.item.round_off_discounted_price_' . ($non_taxed ? 'non_taxed' : 'taxed'))) {
            $discounted_price_with_tax = round($discounted_price_with_tax, 0);
        }

        $rate_without_tax = SysHelper::formatCurrency($price_with_tax / (1 + ($tax_value/100)));

        return compact('margin_percentage', 'discount_percentage', 'price', 'price_with_tax', 'discounted_price', 'discounted_price_with_tax', 'rate_without_tax');
    }

    private function getRate($margin, $rate)
    {
        if ($this->is_service) {
            return SysHelper::formatCurrency($this->mrp);
        }

        $margin_percentage = 15;

        if ($margin) {
            $margin_detail = $margin->details->where('min_amount', '<=', $rate)->where('max_amount', '>=', $rate)->first();

            if ($margin_detail) {
                $margin_percentage = (float) $margin_detail->value;
            }
        }

        $price = SysHelper::formatCurrency($rate + (($rate * $margin_percentage) / 100));

        if ($price > $this->mrp) {
            $price = $this->mrp;
        }

        return SysHelper::formatCurrency($price);
    }
}
