<?php

namespace App\Models\Accounting;

use App\Enums\LedgerGroup;
use App\Models\Contact;
use App\Models\Tag;
use App\Traits\HasContact;
use App\Traits\HasMeta;
use App\Traits\HasUuid;
use App\Traits\ModelOption;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Arr;
use Spatie\Activitylog\Traits\LogsActivity;

class Ledger extends Model
{
    use HasUuid, HasMeta, ModelOption, HasContact, LogsActivity;

    protected $guarded = [];
    protected $casts = [
        'meta' => 'array',
        'opening' => 'array',
        'current' => 'array',
        'on_account' => 'array',
    ];
    protected $table = 'ledgers';
    protected static $logName = 'ledger';
    protected static $logUnguarded = true;
    protected static $logOnlyDirty = true;
    protected static $logAttributesToIgnore = ['updated_at'];
    protected static $sortOptions = ['created_at', 'name', 'balance_taxed', 'balance_non_taxed', 'updated_at'];
    protected static $defaultSortBy = 'name';
    protected $with = ['type', 'contacts', 'accounts', 'tags'];

    // Meta
    public function getMeta(string $option)
    {
        return Arr::get($this->meta, $option);
    }

    public function accounts()
    {
        return $this->morphMany(Account::class, 'accountable');
    }

    public function contacts()
    {
        return $this->morphToMany(Contact::class, 'contactable');
    }

    public function type() : BelongsTo
    {
        return $this->belongsTo(LedgerType::class, 'ledger_type_id');
    }

    public function vouchers() : HasMany
    {
        return $this->hasMany(Voucher::class);
    }

    public function transactions() : HasMany
    {
        return $this->hasMany(Transaction::class);
    }

    public function transactionDetails() : HasMany
    {
        return $this->hasMany(TransactionDetail::class);
    }

    public function tags()
    {
        return $this->morphToMany(Tag::class, 'taggable');
    }

    public function getCurrentBalanceTaxedAttribute()
    {
        return (float) Arr::get($this->current, 'balance_taxed', 0);
    }

    public function getCurrentBalanceNonTaxedAttribute()
    {
        return (float) Arr::get($this->current, 'balance_non_taxed', 0);
    }

    public function getOpeningBalanceTaxedAttribute()
    {
        return (float) Arr::get($this->opening, 'balance_taxed', 0);
    }

    public function getOpeningBalanceNonTaxedAttribute()
    {
        return (float) Arr::get($this->opening, 'balance_non_taxed', 0);
    }

    // Booted
    public static function booted()
    {
    }

    protected static function ensureUpdatable() : void
    {

    }

    public function scopeSearch(Builder $query, $search = null)
    {
        $fuzzy = implode("%", str_split($search));
        $fuzzy = "%$fuzzy%";

        $query->when($search, function ($q) use ($fuzzy) {
            return $q->where(function($q1) use ($fuzzy) {
                $q1->where('name', 'like', $fuzzy)
                ->orWhere('alias', 'like', $fuzzy)
                ->orWhere(\DB::raw('lower(meta->"$.billing_name")'), "LIKE", "%".strtolower($fuzzy)."%");
            });
        });
    }

    // Filters
    public function scopeFilterById(Builder $query, $id) : void
    {
        $query->when($id, function ($q, $id) {
            return $q->where('id', '=', $id);
        });
    }

    public function scopeFilterByUuid(Builder $query, $uuid) : void
    {
        $query->when($uuid, function ($q, $uuid) {
            return $q->where('uuid', '=', $uuid);
        });
    }

    public function scopeFilterByName(Builder $query, $name = null) : void
    {
        $query->when($name, function ($q, $name) {
            return $q->where('name', 'like', '%'.$name.'%');
        });
    }

    public function scopeFilterByAlias(Builder $query, $alias = null) : void
    {
        $query->when($alias, function ($q, $alias) {
            return $q->where('alias', 'like', '%'.$alias.'%');
        });
    }

    public function scopeFilterByNameOrAlias(Builder $query, $keyword = null) : void
    {
        $query->when($keyword, function ($q, $keyword) {
            return $q->where(function($q1) use ($keyword) {
                $q1->where('name', 'like', '%'.$keyword.'%')->orWhere('alias', 'like', '%'.$keyword.'%');
            });
        });
    }

    public function scopeForVoucher(Builder $query, $group = null) : void
    {
        $query->whereHas('type', function($q) use ($group) {
            if ($group) {
                $group = is_array($group) ? $group : [$group];
                $q->whereIn('code', [$group]);
            } else {
                $q->whereIn('code', [LedgerGroup::SUNDRY_CREDITOR, LedgerGroup::SUNDRY_DEBTOR, LedgerGroup::CASH]);
            }
        });
    }

    public function scopeForTransaction(Builder $query, $group = null) : void
    {
        $query->whereHas('type', function($q) use ($group) {
            if ($group) {
                $group = is_array($group) ? $group : [$group];
                $q->whereIn('code', [$group]);
            } else {
                $q->whereIn('code', [LedgerGroup::CASH, LedgerGroup::BANK_ACCOUNT]);
            }
        });
    }

    public function scopeForPayment(Builder $query, $group = null) : void
    {
        $query->whereHas('type', function($q) use ($group) {
            if ($group) {
                $group = is_array($group) ? $group : [$group];
                $q->whereIn('code', [$group]);
            } else {
                $q->whereIn('code', [LedgerGroup::SUNDRY_CREDITOR, LedgerGroup::SUNDRY_DEBTOR, LedgerGroup::INDIRECT_EXPENSE]);
            }
        });
    }

    public function scopeForReceipt(Builder $query, $group = null) : void
    {
        $query->whereHas('type', function($q) use ($group) {
            if ($group) {
                $group = is_array($group) ? $group : [$group];
                $q->whereIn('code', [$group]);
            } else {
                $q->whereIn('code', [LedgerGroup::SUNDRY_CREDITOR, LedgerGroup::SUNDRY_DEBTOR, LedgerGroup::INDIRECT_INCOME]);
            }
        });
    }
}
