<?php

namespace App\Models\Accounting;

use App\Models\Tag;
use App\Models\User;
use App\Traits\HasMeta;
use App\Traits\HasUuid;
use App\Traits\ModelOption;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Arr;
use Spatie\Activitylog\Contracts\Activity;
use Spatie\Activitylog\Traits\LogsActivity;

class Transaction extends Model
{
    use HasUuid, HasMeta, ModelOption, LogsActivity;

    protected $guarded = [];
    protected $casts = [
        'meta'         => 'array',
        'is_non_taxed' => 'boolean',
    ];
    protected $table = 'transactions';
    protected static $logName = 'transaction';
    protected static $logUnguarded = true;
    protected static $logOnlyDirty = true;
    protected static $logAttributesToIgnore = ['updated_at'];
    protected static $sortOptions = ['created_at', 'date', 'type', 'updated_at'];
    protected static $defaultSortBy = 'date';
    protected $with = ['ledger', 'ledger.type', 'details', 'details.ledger', 'tags'];

    // Meta
    public function getMeta(string $option)
    {
        return Arr::get($this->meta, $option);
    }

    public function details() : HasMany
    {
        return $this->hasMany(TransactionDetail::class);
    }

    public function paymentDetails() : HasMany
    {
        return $this->hasMany(TransactionPaymentDetail::class);
    }

    public function ledger() : BelongsTo
    {
        return $this->belongsTo(Ledger::class);
    }

    public function tags()
    {
        return $this->morphToMany(Tag::class, 'taggable');
    }

    public function user() : BelongsTo
    {
        return $this->belongsTo(User::class);
    }

    // Booted
    public static function booted()
    {
    }

    protected static function ensureUpdatable() : void
    {

    }

    public function tapActivity(Activity $activity, string $eventName)
    {
        $properties = $activity->properties;
        $properties['info'] = array(
            'name' => $this->code_number,
            'type' => 'transaction',
            'sub_type' => $this->type,
            'uuid' => $this->uuid
        );
        $activity->properties = $properties;
    }

    // Filters
    public function scopeFilterById(Builder $query, $id) : void
    {
        $query->when($id, function ($q, $id) {
            return $q->where('id', '=', $id);
        });
    }

    public function scopeFilterByUuid(Builder $query, $uuid) : void
    {
        $query->when($uuid, function ($q, $uuid) {
            return $q->where('uuid', '=', $uuid);
        });
    }

    public function scopeDateBetween(Builder $query, $dates = array()) : void
    {
        $start_date = Arr::get($dates, 'start_date');
        $end_date = Arr::get($dates, 'end_date') ? : $start_date;

        if ($start_date && $end_date && $start_date <= $end_date) {
            $query->where('date', '>=', $start_date)->where('date', '<=', $end_date);
        }
    }
}
