<?php

namespace App\Models\Accounting;

use App\Enums\VoucherType;
use App\Helpers\ArrHelper;
use App\Helpers\CalHelper;
use App\Models\Contact;
use App\Models\Option;
use App\Models\Tag;
use App\Models\User;
use App\Traits\HasMeta;
use App\Traits\HasUuid;
use App\Traits\ModelOption;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Arr;
use Spatie\Activitylog\Traits\LogsActivity;

class Voucher extends Model
{
    use HasUuid, HasMeta, ModelOption, LogsActivity;

    protected $guarded = [];
    protected $casts = [
        'meta'                   => 'array',
        'discount'               => 'array',
        'tax'                    => 'array',
        'info'                   => 'array',
        'follow_up'              => 'array',
        'date'                   => 'date',
        'due_date'               => 'date',
        'delivery_date'          => 'date',
        'expected_delivery_date' => 'date',
        'is_non_taxed'           => 'boolean',
    ];
    protected $table = 'vouchers';
    protected static $logName = 'voucher';
    protected static $logUnguarded = true;
    protected static $logOnlyDirty = true;
    protected static $logAttributesToIgnore = ['updated_at'];
    protected static $sortOptions = ['date', 'status', 'created_at', 'updated_at'];
    protected static $defaultSortBy = 'date';
    protected $with = ['ledger', 'contact', 'enquiryType', 'enquirySource', 'user', 'tags'];

    // Meta
    public function getMeta(string $option)
    {
        return Arr::get($this->meta, $option);
    }

    public function getInfo(string $option)
    {
        return Arr::get($this->info, $option);
    }

    public function getStatusDetailAttribute()
    {
        if (in_array($this->type, [VoucherType::PURCHASE_ENQUIRY, VoucherType::SALE_ENQUIRY])) {
            return ArrHelper::searchByKey(ArrHelper::getTransList('statuses', 'enquiry'), 'uuid', $this->status);
        }

        return $this->status;
    }

    public function taxes() : HasMany
    {
        return $this->hasMany(VoucherTax::class);
    }

    public function details() : HasMany
    {
        return $this->hasMany(VoucherDetail::class);
    }

    public function followUps() : HasMany
    {
        return $this->hasMany(FollowUp::class);
    }

    public function ledger() : BelongsTo
    {
        return $this->belongsTo(Ledger::class);
    }

    public function contact() : BelongsTo
    {
        return $this->belongsTo(Contact::class);
    }

    public function referral() : BelongsTo
    {
        return $this->belongsTo(Contact::class, 'referral_id');
    }

    public function transport() : BelongsTo
    {
        return $this->belongsTo(Option::class, 'transport_id');
    }

    public function enquiryType() : BelongsTo
    {
        return $this->belongsTo(Option::class, 'enquiry_type_id');
    }

    public function enquirySource() : BelongsTo
    {
        return $this->belongsTo(Option::class, 'enquiry_source_id');
    }

    public function tags()
    {
        return $this->morphToMany(Tag::class, 'taggable');
    }

    public function user() : BelongsTo
    {
        return $this->belongsTo(User::class);
    }

    // Booted
    public static function booted()
    {
    }

    protected static function ensureUpdatable() : void
    {

    }

    // Filters
    public function scopeFilterById(Builder $query, $id) : void
    {
        $query->when($id, function ($q, $id) {
            return $q->where('id', '=', $id);
        });
    }

    public function scopeFilterByUuid(Builder $query, $uuid) : void
    {
        $query->when($uuid, function ($q, $uuid) {
            return $q->where('uuid', '=', $uuid);
        });
    }

    public function scopeFilterByType(Builder $query, $type = null) : void
    {
        $query->when($type, function ($q, $type) {
            return $q->where('type', '=', $type);
        });
    }

    public function scopeDateBetween(Builder $query, $type = 'date', $dates = array()) : void
    {
        $start_date = Arr::get($dates, 'start_date');
        $end_date = Arr::get($dates, 'end_date') ? : $start_date;

        if ($start_date && $end_date && $start_date <= $end_date) {
            $query->where($type, '>=', $start_date)->where($type, '<=', $end_date);
        }
    }
}
