<?php

namespace App\Models;

use App\Models\Accounting\Ledger;
use App\Models\Accounting\Voucher;
use App\Models\Inventory\Brand;
use App\Traits\HasMeta;
use App\Traits\HasUuid;
use App\Traits\ModelOption;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;
use Spatie\Activitylog\Traits\LogsActivity;

class Contact extends Model
{
    use HasUuid, HasMeta, ModelOption, LogsActivity;

    protected $guarded = [];
    protected $casts = [
        'meta' => 'array',
        'address' => 'array',
        'birth_date' => 'date',
        'anniversary_date' => 'date'
    ];
    protected $table = 'contacts';
    protected static $logName = 'contact';
    protected static $logUnguarded = true;
    protected static $logOnlyDirty = true;
    protected static $logAttributesToIgnore = ['updated_at'];
    protected static $sortOptions = ['created_at', 'name', 'alias', 'email', 'updated_at'];
    protected static $defaultSortBy = 'name';
    protected $with = ['tags'];

    // Meta
    public function getMeta(string $option)
    {
        return Arr::get($this->meta, $option);
    }

    public function ledgers()
    {
        return $this->morphedByMany(Ledger::class, 'contactable');
    }

    public function brands()
    {
        return $this->morphedByMany(Brand::class, 'contactable');
    }

    public function vouchers()
    {
        return $this->hasMany(Voucher::class);
    }

    public function tags()
    {
        return $this->morphToMany(Tag::class, 'taggable');
    }

    // Booted
    public static function booted()
    {
    }

    protected static function ensureUpdatable() : void
    {

    }

    // Filters

    public function scopeSearch(Builder $query, $search = null)
    {
        $fuzzy = implode("%", str_split($search));
        $fuzzy = "%$fuzzy%";

        $query->when($search, function ($q) use ($fuzzy) {
            return $q->where(function($q1) use ($fuzzy) {
                $q1->where('name', 'like', $fuzzy)
                ->orWhere('alias', 'like', $fuzzy);
            });
        });
    }

    public function scopeFilterById(Builder $query, $id) : void
    {
        $query->when($id, function ($q, $id) {
            return $q->where('id', '=', $id);
        });
    }

    public function scopeFilterByUuid(Builder $query, $uuid) : void
    {
        $query->when($uuid, function ($q, $uuid) {
            return $q->where('uuid', '=', $uuid);
        });
    }

    public function scopeFilterByName(Builder $query, $name = null) : void
    {
        $query->when($name, function ($q, $name) {
            return $q->where('name', 'like', '%'.$name.'%');
        });
    }

    public function scopeFilterByAlias(Builder $query, $alias = null) : void
    {
        $query->when($alias, function ($q, $alias) {
            return $q->where('alias', 'like', '%'.$alias.'%');
        });
    }

    public function scopeFilterByNameOrAlias(Builder $query, $keyword = null) : void
    {
        $query->when($keyword, function ($q, $keyword) {
            return $q->where(function($q1) use ($keyword) {
                $q1->where('name', 'like', '%'.$keyword.'%')->orWhere('alias', 'like', '%'.$keyword.'%');
            });
        });
    }
}
