<?php

namespace App\Models\Inventory;

use App\Enums\ItemCategoryType;
use App\Models\Accounting\VoucherDetail;
use App\Models\Setting\Margin;
use App\Models\Setting\Tax;
use App\Models\Setting\Unit;
use App\Models\Tag;
use App\Traits\HasMeta;
use App\Traits\HasUuid;
use App\Traits\ModelOption;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Arr;
use Spatie\Activitylog\Traits\LogsActivity;

class Item extends Model
{
    use HasMeta, HasUuid, ModelOption, LogsActivity;

    protected $guarded = [];
    protected $casts = [
        'meta' => 'array',
        'identifier' => 'array',
        'opening' => 'array',
        'current' => 'array',
        'variants' => 'array',
    ];
    protected $table = 'items';
    protected static $logName = 'item';
    protected static $logUnguarded = true;
    protected static $logOnlyDirty = true;
    protected static $logAttributesToIgnore = ['updated_at'];
    protected static $sortOptions = ['created_at', 'name', 'alias', 'quantity_taxed', 'quantity_non_taxed', 'updated_at'];
    protected static $defaultSortBy = 'name';
    protected $with = ['category', 'brand', 'margin', 'margin.details', 'tax', 'unit', 'tags'];
    protected $appends = ['type', 'is_product', 'is_service'];

    public function getCurrentQuantityTaxedAttribute()
    {
        return (float) Arr::get($this->current, 'quantity_taxed', 0);
    }

    public function getCurrentQuantityNonTaxedAttribute()
    {
        return (float) Arr::get($this->current, 'quantity_non_taxed', 0);
    }

    public function getOpeningQuantityTaxedAttribute()
    {
        return (float) Arr::get($this->opening, 'quantity_taxed', 0);
    }

    public function getOpeningQuantityNonTaxedAttribute()
    {
        return (float) Arr::get($this->opening, 'quantity_non_taxed', 0);
    }

    public function getCurrentRateTaxedAttribute()
    {
        return (float) Arr::get($this->current, 'rate_taxed', 0);
    }

    public function getCurrentRateNonTaxedAttribute()
    {
        return (float) Arr::get($this->current, 'rate_non_taxed', 0);
    }

    public function getOpeningRateTaxedAttribute()
    {
        return (float) Arr::get($this->opening, 'rate_taxed', 0);
    }

    public function getOpeningRateNonTaxedAttribute()
    {
        return (float) Arr::get($this->opening, 'rate_non_taxed', 0);
    }

    public function getTypeAttribute()
    {
        return $this->category->type == ItemCategoryType::PRODUCT ? 'product' : 'service';
    }

    public function getIsProductAttribute()
    {
        return $this->category->type == ItemCategoryType::PRODUCT ? true : false;
    }

    public function getIsServiceAttribute()
    {
        return $this->category->type == ItemCategoryType::SERVICE ? true : false;
    }

    // Relations
    public function unit() : BelongsTo
    {
        return $this->belongsTo(Unit::class);
    }

    public function tax() : BelongsTo
    {
        return $this->belongsTo(Tax::class);
    }

    public function margin() : BelongsTo
    {
        return $this->belongsTo(Margin::class);
    }

    public function brand() : BelongsTo
    {
        return $this->belongsTo(Brand::class);
    }

    public function category() : BelongsTo
    {
        return $this->belongsTo(ItemCategory::class, 'item_category_id');
    }

    public function voucherDetails() : HasMany
    {
        return $this->hasMany(VoucherDetail::class);
    }

    public function tags()
    {
        return $this->morphToMany(Tag::class, 'taggable');
    }

    // Booted
    public static function booted()
    {
    }

    protected static function ensureUpdatable() : void
    {
    }

    public function scopeSearch(Builder $query, $search = null)
    {
        $fuzzy = implode("%", str_split($search));
        $fuzzy = "%$fuzzy%";

        $query->when($search, function ($q) use ($fuzzy) {
            return $q->where(function($q1) use ($fuzzy) {
                $q1->where('name', 'like', $fuzzy)
                ->orWhere('alias', 'like', $fuzzy);
            });
        });
    }

    // Filters

    public function scopeFilterByName(Builder $query, $name = null) : void
    {
        $query->when($name, function ($q, $name) {
            return $q->where('name', 'like', '%'.$name.'%');
        });
    }

    public function scopeFilterByAlias(Builder $query, $alias = null) : void
    {
        $query->when($alias, function ($q, $alias) {
            return $q->where('alias', 'like', '%'.$alias.'%');
        });
    }

    public function scopeFilterByNameOrAlias(Builder $query, $keyword = null) : void
    {
        $query->when($keyword, function ($q, $keyword) {
            return $q->where(function($q1) use ($keyword) {
                $q1->where('name', 'like', '%'.$keyword.'%')->orWhere('alias', 'like', '%'.$keyword.'%');
            });
        });
    }
}
