<?php

namespace App\Models\Inventory;

use App\Models\Setting\Margin;
use App\Models\Setting\Tax;
use App\Models\Setting\Unit;
use App\Models\Tag;
use App\Scopes\LastStockMatchScope;
use App\Traits\HasMeta;
use App\Traits\HasUuid;
use App\Traits\ModelOption;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Spatie\Activitylog\Traits\LogsActivity;

class ItemCategory extends Model
{
    use HasMeta, HasUuid, ModelOption, LogsActivity;

    protected $guarded = [];
    protected $casts = [
        'meta' => 'array',
        'variants' => 'array',
    ];
    protected $table = 'item_categories';
    protected static $logName = 'item_category';
    protected static $logUnguarded = true;
    protected static $logOnlyDirty = true;
    protected static $logAttributesToIgnore = ['updated_at'];
    protected static $sortOptions = ['created_at', 'name', 'alias', 'stock_match_date', 'updated'];
    protected static $defaultSortBy = 'name';
    protected $with = ['unit', 'tax', 'margin', 'margin.details', 'parent', 'tags'];

    // Relations
    public function parent() : BelongsTo
    {
        return $this->belongsTo(self::class, 'parent_id');
    }

    public function unit() : BelongsTo
    {
        return $this->belongsTo(Unit::class);
    }

    public function tax() : BelongsTo
    {
        return $this->belongsTo(Tax::class);
    }

    public function margin() : BelongsTo
    {
        return $this->belongsTo(Margin::class);
    }

    public function items() : HasMany
    {
        return $this->hasMany(Item::class);
    }

    public function stockMatches() : HasMany
    {
        return $this->hasMany(StockMatch::class);
    }

    public function tags()
    {
        return $this->morphToMany(Tag::class, 'taggable');
    }

    // Booted
    protected static function booted()
    {
        // static::addGlobalScope('lastStockMatch', function (Builder $builder) {
        //     $builder->with(['stockMatches' => function($q) {
        //         $q->orderBy('date','desc')->limit(1);
        //     }]);
        // });

        static::addGlobalScope('lastStockMatch', function (Builder $builder) {
            $builder->lastStockMatchDate();
        });
    }

    protected static function ensureUpdatable() : void
    {
    }

    // Filters

    public function scopeSearch(Builder $query, $search = null)
    {
        $fuzzy = implode("%", str_split($search));
        $fuzzy = "%$fuzzy%";

        $query->when($search, function ($q) use ($fuzzy) {
            return $q->where(function($q1) use ($fuzzy) {
                $q1->where('name', 'like', $fuzzy)
                ->orWhere('alias', 'like', $fuzzy)
                ->orWhere('description', 'like', $fuzzy);
            });
        });
    }

    public function scopeLastStockMatchDate(Builder $query)
    {
        $query->addSelect(['last_stock_match_date' => StockMatch::select('date')
            ->whereColumn('item_category_id', 'item_categories.id')
            ->orderByDesc('date')
            ->limit(1)
        ]);
    }

    public function scopeFilterByName(Builder $query, $name = null) : void
    {
        $query->when($name, function ($q, $name) {
            return $q->where('name', 'like', '%'.$name.'%');
        });
    }

    public function scopeFilterByAlias(Builder $query, $alias = null) : void
    {
        $query->when($alias, function ($q, $alias) {
            return $q->where('alias', 'like', '%'.$alias.'%');
        });
    }

    public function scopeFilterByNameOrAlias(Builder $query, $keyword = null) : void
    {
        $query->when($keyword, function ($q, $keyword) {
            return $q->where(function($q1) use ($keyword) {
                $q1->where('name', 'like', '%'.$keyword.'%')->orWhere('alias', 'like', '%'.$keyword.'%');
            });
        });
    }
}
