<?php

namespace App\Repositories\Accounting;

use App\Models\Accounting\Account;
use App\Models\Accounting\Ledger;
use Illuminate\Validation\ValidationException;

class AccountRepository
{
    protected $account;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Account $account
    ) {
        $this->account = $account;
    }

    /**
     * Find account by uuid or fail
     *
     * @param Ledger $ledger
     */
    private function findByUuidOrFail(Ledger $ledger) : Account
    {
        $account = $this->account->whereAccountableType('Ledger')->whereAccountableId($ledger->id)->whereUuid(request('account.uuid'))->first();

        if (! $account) {
            throw ValidationException::withMessages(['mesage' => trans('global.could_not_find', ['attribute' => trans('accounting.account.account')])]);
        }
        
        return $account;
    }


    /**
     * Create a new ledger
     */
    public function create(Ledger $ledger) : Account
    {
        $account = $this->account->forceCreate($this->formatParams($ledger));

        $ledger->accounts()->save($account);

        return $account;
    }

    /**
     * Prepare given params for inserting into database
     * @param Ledger $ledger
     * @param uuid $uuid
     */
    private function formatParams(Ledger $ledger, $uuid = null) : array
    {
        $existing_account = $this->account->when('uuid', function($q, $uuid) {
            $q->where('uuid', '!=', $uuid);
        })->whereAccountableType('Ledger')->whereAccountableId($ledger->id)->whereName(request('account.name'))->where('bank_details->number', request('account.account_number'))->count();

        if ($existing_account) {
            throw ValidationException::withMessages(['account.account_number' => trans('global.duplicate_found', ['attribute' => trans('accounting.account.account')])]);
        }

        $formatted = [
            'name'       => request('account.name'),
            'alias'      => request('account.alias'),
            'bank_details'    => array(
                'number'      => request('account.account_number'),
                'bank_code1'  => request('account.bank_code1'),
                'bank_code2'  => request('account.bank_code2'),
                'bank_code3'  => request('account.bank_code3'),
                'bank_name'  => request('account.bank_name'),
                'branch_name' => request('account.branch_name')
            ),
            'is_primary' => request()->has('enable_account') ? true : false
        ];

        return $formatted;
    }

    /**
     * Update account
     *
     * @param Ledger $ledger
     * @param uuid $uuid
     */
    public function update(Ledger $ledger, $uuid) : void
    {
        $account = $this->findByUuidOrFail($ledger);

        $account->forceFill($this->formatParams($ledger, $uuid))->save();
    }

    /**
     * Delete account
     *
     * @param Ledger $ledger
     */
    public function delete(Ledger $ledger) : void
    {
        $account = $this->findByUuidOrFail($ledger);

        $account->delete();
    }
}