<?php
namespace App\Repositories\Accounting;

use App\Enums\LedgerGroup;
use App\Enums\TransactionType;
use App\Enums\VoucherStatus;
use App\Enums\VoucherType;
use App\Helpers\CalHelper;
use App\Helpers\SysHelper;
use App\Http\Resources\Accounting\LedgerSummary;
use App\Models\Accounting\Ledger;
use App\Models\Accounting\Transaction;
use App\Models\Accounting\TransactionDetail;
use App\Models\Accounting\Voucher;
use App\Models\Accounting\VoucherTax;
use App\Traits\CollectionPaginator;
use App\Traits\LedgerConstraints;
use App\Traits\TransactionConstraints;
use App\Traits\VoucherConstraints;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class LedgerReportRepository
{
    use VoucherConstraints, TransactionConstraints, LedgerConstraints, CollectionPaginator;

    protected $ledger;
    protected $voucher;
    protected $transaction;
    protected $transaction_detail;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        LedgerRepository $ledger,
        Voucher $voucher,
        Transaction $transaction,
        TransactionDetail $transaction_detail
    ) {
        $this->ledger = $ledger;
        $this->voucher = $voucher;
        $this->transaction = $transaction;
        $this->transaction_detail = $transaction_detail;
    }

    private function getDuration()
    {
        $start_date = CalHelper::getDate(request('start_date', today()->subDays(7)->toDateString()));
        $end_date = CalHelper::getDate(request('end_date', today()->toDateString()));

        return compact('start_date', 'end_date');
    }

    /**
     * Get report
     *
     * @return array
     */
    public function getReport(Ledger $ledger) : array
    {
        $per_page     = $this->voucher->getPerPage();
        $current_page = $this->voucher->getCurrentPage();

        $duration = $this->getDuration();
        $ledger = $this->ledger->findByUuidOrFail($ledger->uuid, 'message');
        $is_non_taxed = (bool) request()->boolean('is_non_taxed');

        $opening_balance = $is_non_taxed ? $ledger->opening_balance_non_taxed : $ledger->opening_balance_taxed;

        $opening_balance = $this->calculateVoucherOpeningBalance($ledger, $opening_balance, Arr::get($duration, 'start_date'), $is_non_taxed);

        $opening_balance = $this->calculatePrimaryLedgerOpeningBalance($ledger, $opening_balance, Arr::get($duration, 'start_date'), $is_non_taxed);

        $opening_balance = $this->calculateSecondaryLedgerOpeningBalance($ledger, $opening_balance, Arr::get($duration, 'start_date'), $is_non_taxed);

        $data = $this->getVouchers($ledger, $duration, $is_non_taxed, []);

        $data = $this->getPrimaryTransactions($ledger, $duration, $is_non_taxed, $data);

        $data = $this->getSecondaryTransactions($ledger, $duration, $is_non_taxed, $data);

        $data = $this->reorder($data, $opening_balance, $ledger->type->code);

        return $this->collectionPaginate($data, $per_page, $current_page);
    }

    /**
     * Caulcate voucher opening balance
     *
     * @param Ledger $ledger
     * @param float $opening_balance
     * @param string $date
     * @param bool $is_non_taxed
     */
    private function calculateVoucherOpeningBalance(Ledger $ledger, $opening_balance = 0, $date = null, $is_non_taxed = false) : float
    {
        $vouchers = $this->voucher->whereLedgerId($ledger->id)->whereIn('type', [VoucherType::SALE, VoucherType::PURCHASE])->whereStatus(VoucherStatus::PUBLISHED)->where('date', '<', $date)->whereIsNonTaxed($is_non_taxed)->groupBy('type')->select(['type', \DB::raw("SUM(total) as total")])->get();

        $existing_sale_balance = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'sale'))->total : 0;
        $existing_purchase_balance = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'purchase'))->total : 0;

        return $opening_balance + ($existing_purchase_balance * $this->getLedgerMultiplier($ledger, 'purchase')) + ($existing_sale_balance * $this->getLedgerMultiplier($ledger, 'sale'));
    }

    /**
     * Caulcate primary ledger opening balance
     *
     * @param Ledger $ledger
     * @param float $opening_balance
     * @param string $date
     * @param bool $is_non_taxed
     */
    private function calculatePrimaryLedgerOpeningBalance(Ledger $ledger, $opening_balance, $date = null, $is_non_taxed = false) : float
    {
        if (! in_array($ledger->type->code, [LedgerGroup::CASH, LedgerGroup::BANK_ACCOUNT])) {
            return $opening_balance;
        }

        $transactions = $this->transaction->whereIsNonTaxed($is_non_taxed)->where('ledger_id', $ledger->id)->whereIsCancelled(0)->where('date', '<', $date)->groupBy('type')->select(['type', \DB::raw("SUM(amount) as total")])->get();

        $existing_payment = (float) $transactions->count() ? optional($transactions->firstWhere('type', 'payment'))->total : 0;
        $existing_receipt = (float) $transactions->count() ? optional($transactions->firstWhere('type', 'receipt'))->total : 0;

        return $opening_balance + ($existing_payment * $this->getPrimaryLedgerMultiplier('payment')) + ($existing_receipt * $this->getPrimaryLedgerMultiplier('receipt'));
    }

    /**
     * Caulcate secondary ledger opening balance
     *
     * @param Ledger $ledger
     * @param float $opening_balance
     * @param string $date
     * @param bool $is_non_taxed
     */
    private function calculateSecondaryLedgerOpeningBalance(Ledger $ledger, $opening_balance, $date = null, $is_non_taxed = false) : float
    {
        $transaction_details = $this->transaction_detail->with('transaction')->whereHas('transaction', function($q) use ($ledger, $date, $is_non_taxed) {
            $q->where('is_non_taxed', $is_non_taxed)->where('date', '<', $date)->where('is_cancelled', 0);
        })->whereLedgerId($ledger->id)->get();

        $existing_payment = $transaction_details->where('transaction.type', 'payment')->sum('amount');
        $existing_receipt = $transaction_details->where('transaction.type', 'receipt')->sum('amount');

        return $opening_balance + ($existing_payment * $this->getSecondaryLedgerMultiplier('payment', $ledger)) + ($existing_receipt * $this->getSecondaryLedgerMultiplier('receipt', $ledger));
    }

    /**
     * Get vouchers
     *
     * @param Ledger $ledger
     * @param array $duration
     * @param bool $is_non_taxed
     * @param array $data
     */
    private function getVouchers(Ledger $ledger, $duration, $is_non_taxed, $data = array()) : array
    {
        $vouchers = $this->voucher->whereLedgerId($ledger->id)->whereIn('type', [VoucherType::SALE, VoucherType::PURCHASE])->whereStatus(VoucherStatus::PUBLISHED)->dateBetween('date', $duration)->whereIsNonTaxed($is_non_taxed)->orderBy('date', 'asc')->get();

        foreach($vouchers as $voucher) {
            $data[] = array(
                'uuid' => $voucher->uuid,
                'date' => CalHelper::toDate($voucher->date),
                'type' => array('uuid' => $voucher->type, 'name' => trans($voucher->type.'.'.$voucher->type)),
                'code_number' => $voucher->code_number,
                'amount' => SysHelper::formatCurrency($voucher->total),
                'balance' => 0,
                'multiplier' => $this->getLedgerMultiplier($ledger, $voucher->type),
                'sub_type' => null,
                'direction' => $this->getVoucherDirection($voucher),
                'second_ledger' => null,
                'status' => array('uuid' => $voucher->payment_status, 'name' => trans($voucher->type.'.'.$voucher->payment_status)),
                'voucher_uuid' => $voucher->uuid
            );
        }

        return $data;
    }

    /**
     * Get primary transactions
     *
     * @param Ledger $ledger
     * @param array $duration
     * @param bool $is_non_taxed
     * @param array $data
     */
    private function getPrimaryTransactions(Ledger $ledger, $duration, $is_non_taxed, $data = array()) : array
    {
        if (! in_array($ledger->type->code, [LedgerGroup::CASH, LedgerGroup::BANK_ACCOUNT])) {
            return $data;
        }

        $transactions = $this->transaction->with('details', 'details.ledger')->whereIsNonTaxed($is_non_taxed)->where('ledger_id', $ledger->id)->whereIsCancelled(0)->dateBetween($duration)->orderBy('date', 'asc')->get();

        foreach($transactions as $transaction) {
            foreach ($transaction->details as $transaction_detail) {
                $data[] = array(
                    'uuid' => $transaction_detail->uuid,
                    'date' => CalHelper::toDate($transaction->date),
                    'type' => array('uuid' => 'transaction', 'name' => trans('transaction.transaction')),
                    'code_number' => $transaction->code_number,
                    'amount' => SysHelper::formatCurrency($transaction_detail->amount),
                    'balance' => 0,
                    'multiplier' => $this->getPrimaryLedgerMultiplier($transaction->type),
                    'sub_type' => array('uuid' => $transaction->type, 'name' => trans('transaction.types.'.$transaction->type)),
                    'direction' => $this->getTransactionDirection($transaction),
                    'second_ledger' => new LedgerSummary($transaction_detail->ledger),
                    'status' => null,
                    'voucher_uuid' => $transaction->uuid
                );
            }
        }

        return $data;
    }

    /**
     * Get secondary transactions
     *
     * @param Ledger $ledger
     * @param array $duration
     * @param bool $is_non_taxed
     * @param array $data
     */
    private function getSecondaryTransactions(Ledger $ledger, $duration, $is_non_taxed, $data = array()) : array
    {
        $transaction_details = $this->transaction_detail->with('transaction', 'transaction.ledger')->whereHas('transaction', function($q) use ($ledger, $duration, $is_non_taxed) {
            $q->where('is_non_taxed', $is_non_taxed)->dateBetween($duration)->where('is_cancelled', 0);
        })->whereLedgerId($ledger->id)->get();

        foreach($transaction_details as $transaction_detail) {
            $data[] = array(
                'uuid' => $transaction_detail->uuid,
                'date' => CalHelper::toDate($transaction_detail->transaction->date),
                'type' => array('uuid' => 'transaction', 'name' => trans('transaction.transaction')),
                'code_number' => $transaction_detail->transaction->code_number,
                'amount' => SysHelper::formatCurrency($transaction_detail->amount),
                'balance' => 0,
                'multiplier' => $this->getSecondaryLedgerMultiplier($transaction_detail->transaction->type, $ledger),
                'sub_type' => array('uuid' => $transaction_detail->transaction->type, 'name' => trans('transaction.types.'.$transaction_detail->transaction->type)),
                'direction' => $this->getTransactionDirection($transaction_detail->transaction),
                'second_ledger' => new LedgerSummary($transaction_detail->transaction->ledger),
                'status' => null,
                'voucher_uuid' => $transaction_detail->transaction->uuid
            );
        }

        return $data;
    }

    /**
     * Reorder
     *
     * @param array $data
     */
    private function reorder($data = array(), $opening_balance = 0, $ledger_group = null) : array
    {
        array_multisort(array_map(function($element) {
            return $element['date'];
        }, $data), SORT_ASC, $data);

        $rows = array();
        $balance = $opening_balance;
        foreach ($data as $item)
        {
            $amount = Arr::get($item, 'amount');
            $balance += ($amount * Arr::get($item, 'multiplier'));
            $rows[] = array(
                'uuid'          => Arr::get($item, 'uuid'),
                'date'          => Arr::get($item, 'date'),
                'type'          => Arr::get($item, 'type'),
                'code_number'   => Arr::get($item, 'code_number'),
                'amount'        => $amount,
                'balance'       => abs($balance),
                'class'         => $this->getClass($ledger_group, $balance),
                'sub_type'      => Arr::get($item, 'sub_type'),
                'direction'     => Arr::get($item, 'direction'),
                'second_ledger' => Arr::get($item, 'second_ledger'),
                'status'        => Arr::get($item, 'status'),
                'voucher_uuid'  => Arr::get($item, 'voucher_uuid')
            );
        }

        return $rows;
    }

    /**
     * Get direction for voucher
     *
     * @param Voucher $voucher
     */
    private function getVoucherDirection(Voucher $voucher) : ?string
    {
        return null;
    }

    /**
     * Get direction for transaction
     *
     * @param Voucher $voucher
     */
    private function getTransactionDirection(Transaction $transaction) : string
    {
        if (in_array($transaction->type, [TransactionType::PAYMENT, TransactionType::CONTRA])) {
            return 'from';
        }

        return 'to';
    }
}
