<?php
namespace App\Repositories\Accounting;

use App\Http\Resources\Accounting\LedgerCollection;
use App\Models\Accounting\Ledger;
use App\Repositories\ContactRepository;
use Illuminate\Validation\ValidationException;

class LedgerRepository
{
    protected $ledger;
    protected $ledger_type;
    protected $contact;
    protected $account;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Ledger $ledger,
        LedgerTypeRepository $ledger_type,
        ContactRepository $contact,
        AccountRepository $account
    ) {
        $this->ledger = $ledger;
        $this->ledger_type = $ledger_type;
        $this->contact = $contact;
        $this->account = $account;
    }

    /**
     * Find ledger with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->find($id);

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given id for voucher or throw an error
     * @param integer $id
     */
    public function findOrFailForVoucher($id, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->forVoucher()->filterbyId($id)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given id for transaction or throw an error
     * @param integer $id
     */
    public function findOrFailForTransaction($id, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->forTransaction()->filterbyId($id)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given uuid or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->filterByUuid($uuid)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given uuid for voucher or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFailForVoucher($uuid, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->forVoucher()->filterByUuid($uuid)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given uuid for transaction or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFailForTransaction($uuid, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->forTransaction()->filterByUuid($uuid)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given uuid for payment or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFailForPayment($uuid, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->forPayment()->filterByUuid($uuid)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Find ledger with given uuid for receipt or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFailForReceipt($uuid, $field = 'message') : Ledger
    {
        $ledger = $this->ledger->forReceipt()->filterByUuid($uuid)->first();

        if (! $ledger) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger.ledger')])]);
        }

        return $ledger;
    }

    /**
     * Get all filtered data
     */
    public function getData($params = array())
    {
        $sort_by      = $this->ledger->getSortBy();
        $order        = $this->ledger->getOrder();

        $name             = request()->query('name');
        $keywords         = request()->query('keywords');
        $ledger_type_uuid = request()->query('ledger_type');

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        return $this->ledger->search($name)->search($keywords)->when($ledger_type_uuid, function($q, $ledger_type_uuid) {
            $q->whereHas('type', function($q1) use($ledger_type_uuid) {
                $q1->whereUuid($ledger_type_uuid);
            });
        })->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        })->select('ledgers.*', \DB::raw('json_extract(opening, "$.balance_taxed") + json_extract(current, "$.balance_taxed") as balance_taxed'), \DB::raw('json_extract(opening, "$.balance_non_taxed") + json_extract(current, "$.balance_non_taxed") as balance_non_taxed'))
        ->orderBy($sort_by, $order);
    }

    /**
     * Paginate all ledgers
     */
    public function paginate() : LedgerCollection
    {
        $per_page     = $this->ledger->getPerPage();
        $current_page = $this->ledger->getCurrentPage();

        return new LedgerCollection($this->getData()->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get pre requisite.
     *
     * @return Array
     */
    public function getPreRequisite()
    {
        // $ledger_types = $this->ledger_type->list();

        return [];
    }

    /**
     * Search ledger
     */
    public function search() : LedgerCollection
    {
        $query = $this->ledger->filterByNameOrAlias(request()->query('q'))->when(request()->query('type'), function($q, $type) {
            if ($type === 'voucher' || $type === 'quotation') {
                $q->forVoucher();
            } else if ($type === 'transaction') {
                $q->forTransaction();
            } else if ($type === 'payment') {
                $q->forPayment();
            } else if ($type === 'receipt') {
                $q->forReceipt();
            } else {
                $q->whereHas('type', function($q1) {
                    $q1->whereIn('code', []);
                });
            }
        });

        return new LedgerCollection($query->orderBy('name', 'asc')->take(10)->get());
    }

    /**
     * Create a new ledger
     */
    public function create() : Ledger
    {
        \DB::beginTransaction();

        $ledger = $this->ledger->forceCreate($this->formatParams());

        $this->updateMeta($ledger);

        if ($ledger->type->has_contact) {
            $ledger->newContact();
        }

        if ($ledger->type->has_account) {
            $this->addAccount($ledger);
        }

        \DB::commit();

        return $ledger;
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $ledger_type = $this->ledger_type->findByUuidOrFail(request('ledger_type.uuid'), 'ledger_type');

        if (is_null($uuid) && $ledger_type->has_account && request('enable_account', false)) {
            request()->validate([
                'account.name' => 'required',
                'account.account_number' => 'required'
            ], [], [
                'account.name' => trans('accounting.account.props.name'),
                'account.account_number' => trans('accounting.account.props.account_number'),
            ]);
        }

        if (is_null($uuid) && $ledger_type->has_contact) {
            request()->validate([
                'contact.mobile' => 'required_without:contact.email',
                'contact.email' => 'email|required_without:contact.mobile',
            ], [], [
                'contact.mobile' => trans('contact.props.mobile'),
                'contact.email' => trans('contact.props.email'),
            ]);
        }

        $formatted = [
            'name'           => request('name'),
            'alias'          => request('alias'),
            'description'    => request('description'),
            'ledger_type_id' => $ledger_type->id,
            'opening'        => array(
                'balance_taxed'     => request('opening_balance_taxed', 0),
                'balance_non_taxed' => request('opening_balance_non_taxed', 0),
            ),
            'current' => array(
                'balance_taxed' => 0,
                'balance_non_taxed' => 0,
            )
        ];

        return $formatted;
    }

    /**
     * Update ledger meta
     *
     * @param Ledger $ledger
     */
    private function updateMeta(Ledger $ledger) : void
    {
        $is_business = (bool) request('is_business', false);

        $meta = $ledger->meta;
        $meta['is_business'] = $is_business;
        $meta['billing_name'] = request('billing_name', request('contact.billing_name', $ledger->name));
        $meta['business_code1'] = request('business_code1');
        $meta['business_code2'] = request('business_code2');
        $meta['business_code3'] = request('business_code3');

        $ledger->meta = $meta;
        $ledger->save();
    }

    /**
     * Add account to ledger
     *
     * @param Ledger $ledger
     */
    public function addAccount(Ledger $ledger) : void
    {
        if (! $ledger->type->has_account) {
            throw ValidationException::withMessages(['message' => trans('general.invalid_action')]);
        }

        if (request()->has('enable_account') && ! request('enable_account', false)) {
            return;
        }

        $this->account->create($ledger);
    }

    /**
     * Update ledger account if required
     *
     * @param Ledger $ledger
     * @param uuid $uuid
     */
    public function updateAccount(Ledger $ledger, $uuid) : void
    {
        if (! $ledger->type->has_account) {
            throw ValidationException::withMessages(['message' => trans('general.invalid_action')]);
        }

        $this->account->update($ledger, $uuid);
    }

    /**
     * Remove account from ledger
     *
     * @param Ledger $ledger
     */
    public function removeAccount(Ledger $ledger) : void
    {
        $this->account->delete($ledger);
    }

    /**
     * Update given ledger
     * @param Ledger $ledger
     */
    public function update(Ledger $ledger) : Ledger
    {
        $ledger_exists = $ledger->vouchers->count();
        $transaction_exists = $ledger->transactions->count();
        $transaction_detail_exists = $ledger->transactionDetails->count();

        if (($ledger_exists || $transaction_exists || $transaction_detail_exists) && $ledger->type->uuid != request('ledger_type.uuid')) {
            throw ValidationException::withMessages(['message' => trans('accounting.ledger.could_not_change_type_if_voucher_or_transaction_exists')]);
        }

        $ledger->forceFill($this->formatParams($ledger->uuid))->save();

        $this->updateMeta($ledger);

        return $ledger;
    }

    /**
     * Delete ledger
     * @param Ledger $ledger
     */
    public function delete(Ledger $ledger) : void
    {
        if ($ledger->vouchers->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('accounting.ledger.ledger'), 'dependency' => trans('voucher.voucher')])]);
        }

        if ($ledger->transactions->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('accounting.ledger.ledger'), 'dependency' => trans('transaction.transaction')])]);
        }

        if ($ledger->transactionDetails->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('accounting.ledger.ledger'), 'dependency' => trans('transaction.transaction')])]);
        }

        $ledger->delete();
    }
}
