<?php
namespace App\Repositories\Accounting;

use App\Http\Resources\Accounting\LedgerTypeCollection;
use App\Models\Accounting\LedgerType;
use App\Repositories\Config\RoleRepository;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class LedgerTypeRepository
{
    protected $ledger_type;
    protected $role;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        LedgerType $ledger_type,
        RoleRepository $role
    ) {
        $this->ledger_type = $ledger_type;
        $this->role = $role;
    }

    /**
     * Find ledger type with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : LedgerType
    {
        $ledger_type = $this->ledger_type->find($id);

        if (! $ledger_type) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger_type.ledger_type')])]);
        }

        return $ledger_type;
    }

    /**
     * Find ledger type with given uuid or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : LedgerType
    {
        $ledger_type = $this->ledger_type->filterByUuid($uuid)->first();

        if (! $ledger_type) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('accounting.ledger_type.ledger_type')])]);
        }

        return $ledger_type;
    }

    /**
     * Get all filtered data
     */
    public function getData($params = array())
    {
        $sort_by      = $this->ledger_type->getSortBy();
        $order        = $this->ledger_type->getOrder();

        $name     = request()->query('name');
        $keywords = request()->query('keywords');

        return $this->ledger_type->search($name)->search($keywords)->orderBy($sort_by, $order);
    }

    /**
     * List all ledger types using given params.
     */

    public function list($params = array()) : LedgerTypeCollection
    {
        return new LedgerTypeCollection($this->getData($params)->get());
    }

    /**
     * List all ledger types using given params.
     */

    public function listOrderedByName($params = array()) : LedgerTypeCollection
    {
        $params['sort_by'] = 'name';
        $params['order'] = 'asc';

        return new LedgerTypeCollection($this->getData($params)->get());
    }

    /**
     * Paginate all ledger types
     */
    public function paginate() : LedgerTypeCollection
    {
        $per_page     = $this->ledger_type->getPerPage();
        $current_page = $this->ledger_type->getCurrentPage();

        return new LedgerTypeCollection($this->getData()->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get pre requisite.
     *
     * @return Array
     */
    public function getPreRequisite()
    {
        // $ledger_types = $this->list();
        $roles = $this->role->getAll(['exclude_admin' => true]);

        return compact('roles');
    }

    /**
     * Search ledger type
     */
    public function search() : LedgerTypeCollection
    {
        $query = $this->ledger_type->filterByNameOrAlias(request()->query('q'));

        return new LedgerTypeCollection($query->orderBy('name', 'asc')->take(10)->get());
    }

    /**
     * Create a new ledger type
     */
    public function create() : LedgerType
    {
        return $this->ledger_type->forceCreate($this->formatParams());
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $formatted = [
            'name'        => request('name'),
            'description' => request('description'),
            'alias'       => request('alias'),
            'parent_id'   => null
        ];

        $parent = request('parent.uuid');

        if ($parent) {
            $parent = $this->findByUuidOrFail($parent);
            $formatted['parent_id'] = $parent->id != $uuid ? $parent->id : null;
        }

        if (! $uuid) {
            $formatted['uuid'] = Str::uuid();
        }

        return $formatted;
    }

    /**
     * Update given ledger type
     * @param LedgerType $ledger_type
     */
    public function update(LedgerType $ledger_type) : LedgerType
    {
        $ledger_type->forceFill($this->formatParams($ledger_type->uuid))->save();

        return $ledger_type;
    }

    /**
     * Update given ledger type role
     * @param LedgerType $ledger_type
     */
    public function updateRole(LedgerType $ledger_type) : LedgerType
    {
        if (! $ledger_type->getMeta('is_default')) {
            throw ValidationException::withMessages(['message' => trans('general.invalid_action')]);
        }

        request()->validate(['roles' => 'required|array']);

        $roles = Arr::pluck(request('roles', []), 'uuid');
        $role_ids = $this->role->getIdByNames($roles);

        $ledger_type->roles()->sync($role_ids);

        return $ledger_type;
    }

    /**
     * Delete ledger type
     * @param LedgerType $ledger_type
     */
    public function delete(LedgerType $ledger_type) : void
    {
        if ($ledger_type->getMeta('is_default')) {
            throw ValidationException::withMessages(['message' => trans('global.cannot_delete_default', ['attribute' => trans('accounting.ledger_type.ledger_type')])]);
        }

        if ($ledger_type->ledgers->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('accounting.ledger_type.ledger_type'), 'dependency' => trans('accounting.ledger.ledger')])]);
        }

        $ledger_type->delete();
    }
}
