<?php

namespace App\Repositories\Accounting;

use App\Enums\LedgerGroup;
use App\Enums\PaymentStatus;
use App\Enums\TransactionType;
use App\Helpers\ArrHelper;
use App\Http\Resources\Accounting\TransactionCollection;
use App\Models\Accounting\Ledger;
use App\Models\Accounting\Transaction;
use App\Models\Accounting\TransactionDetail;
use App\Models\Accounting\TransactionPaymentDetail;
use App\Models\Accounting\Voucher;
use App\Traits\TransactionConstraints;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class TransactionRepository
{
    use TransactionConstraints;

    protected $transaction;
    protected $transaction_detail;
    protected $transaction_payment_detail;
    protected $ledger;
    protected $voucher;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Transaction $transaction,
        TransactionDetail $transaction_detail,
        TransactionPaymentDetail $transaction_payment_detail,
        LedgerRepository $ledger,
        Voucher $voucher
    ) {
        $this->transaction = $transaction;
        $this->ledger = $ledger;
        $this->transaction_detail = $transaction_detail;
        $this->transaction_payment_detail = $transaction_payment_detail;
        $this->voucher = $voucher;
    }

    /**
     * Find transaction with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Transaction
    {
        $transaction = $this->transaction->find($id);

        if (! $transaction) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('transaction.transaction')])]);
        }

        return $transaction;
    }

    /**
     * Find transaction with given uuid or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Transaction
    {
        $transaction = $this->transaction->filterByUuid($uuid)->first();

        if (! $transaction) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('transaction.transaction')])]);
        }

        return $transaction;
    }

    /**
     * Get all filtered data
     */
    public function getData($params = array())
    {
        $sort_by = $this->transaction->getSortBy();
        $order   = $this->transaction->getOrder();

        $type        = request('type');
        $keywords    = request()->query('keywords');
        $code_number = request()->query('code_number');
        $start_date  = request()->query('start_date');
        $end_date    = request()->query('end_date');
        $ledger_uuid = request()->query('ledger');

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        return $this->transaction->when($type, function($q, $type) {
            $q->where('type', $type);
        })->when($code_number, function($q, $code_number) {
            $q->where('code_number', 'like', '%'.$code_number.'%');
        })->dateBetween([
            'start_date' => $start_date,
            'end_date'   => $end_date
        ])->when($ledger_uuid, function($q, $ledger_uuid) {
            $q->where(function($q1) use ($ledger_uuid) {
                $q1->whereHas('ledger', function($q2) use($ledger_uuid) {
                    $q2->whereUuid($ledger_uuid);
                })->orWhereHas('details', function($q3) use ($ledger_uuid) {
                    $q3->whereHas('ledger', function($q4) use ($ledger_uuid) {
                        $q4->whereUuid($ledger_uuid);
                    });
                });
            });
        })->when($keywords, function($q, $keywords) {
            $q->where(function($q1) use ($keywords) {
                $q1->where('description', 'like', '%'.$keywords.'%');
            });
        })->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        })->orderBy($sort_by, $order);
    }

    /**
     * Paginate all transactions
     */
    public function paginate() : TransactionCollection
    {
        $per_page     = $this->transaction->getPerPage();
        $current_page = $this->transaction->getCurrentPage();

        return new TransactionCollection($this->getData()->paginate((int) $per_page, ['*'], 'current_page'));
    }

    private function getCodeNumberType() : string
    {
        if (in_array(request('type'), [TransactionType::CHALLAN_RECEIPT, TransactionType::CHALLAN_PAYMENT])) {
            return 'challan';
        }

        return 'transaction';
    }

    private function getCodeNumber() : array
    {
        $type = $this->getCodeNumberType();

        $number_prefix = $type == 'challan' ? config('config.transaction.challan_number_prefix') : config('config.transaction.number_prefix');
        $number_suffix = $type == 'challan' ? config('config.transaction.challan_number_suffix') : config('config.transaction.number_suffix');
        $digit = $type == 'challan' ? config('config.transaction.challan_number_digit') : config('config.transaction.number_digit');

        $number_format = $number_prefix . '%NUMBER%' . $number_suffix;
        $code_number = (int) $this->transaction->when($type == 'challan', function($q) {
            $q->whereIn('type', [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT]);
        })->when($type != 'challan', function($q) {
            $q->whereNotIn('type', [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT]);
        })->whereNumberFormat($number_format)->max('number');
        $code_number += 1;

        $format = str_replace('%YEAR%', date('Y'), $number_format);
        $format = str_replace('%YEAR_SHORT%', date('y'), $format);
        $format = str_replace('%MONTH%', date('F'), $format);
        $format = str_replace('%MONTH_SHORT%', date('M'), $format);
        $format = str_replace('%MONTH_NUMBER%', date('m'), $format);
        $format = str_replace('%MONTH_NUMBER_SHORT%', date('n'), $format);
        $format = str_replace('%DAY%', date('d'), $format);
        $format = str_replace('%DAY_SHORT%', date('j'), $format);
        return array(
            'code_number'   => str_replace('%NUMBER%', str_pad($code_number, $digit, '0', STR_PAD_LEFT), $format),
            'number_format' => $number_format,
            'number'        => $code_number
        );
    }

    /**
     * Get pre requisite.
     *
     * @return Array
     */
    public function getPreRequisite()
    {
        $types = ArrHelper::getTransList('transaction_types', 'accounting');
        $code_number = Arr::get($this->getCodeNumber(), 'code_number');

        return compact('types', 'code_number');
    }

    /**
     * Create a new transaction
     */
    public function create() : Transaction
    {
        \DB::beginTransaction();

        $transaction = $this->transaction->forceCreate($this->formatParams());

        $transaction = $this->updateDetails($transaction);

        // $this->updateVoucherPaymentStatus($transaction);

        \DB::commit();

        return $transaction;
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        // $code_number = $this->validateCodeNumber($uuid);
        $code_number = $this->getCodeNumber();

        $type = request('type');

        if (! in_array($type, ArrHelper::getList('transaction_types', 'accounting'))) {
            throw ValidationException::withMessages(['type' => trans('global.could_not_find', ['attribute' => trans('transaction.props.type')])]);
        }

        $ledger = $this->ledger->findByUuidOrFailForTransaction(request('ledger.uuid'), 'ledger');

        $formatted = [
            'code_number'   => Arr::get($code_number, 'code_number'),
            'number_format' => Arr::get($code_number, 'number_format'),
            'number'        => Arr::get($code_number, 'number'),
            'type'          => $type,
            'date'          => request('date'),
            'ledger_id'     => $ledger->id,
            'is_non_taxed'  => request()->boolean('is_non_taxed'),
            'description'   => request('description')
        ];

        if (! $uuid) {
            $number = (int) $this->transaction->max('number');
            $formatted['number'] = $number + 1;
        }

        return $formatted;
    }

    /**
     * Validate code number
     *
     * @param string $uuid
     */
    private function validateCodeNumber($uuid = null) : array
    {
        $type = $this->getCodeNumberType();

        $duplicate_transaction_number = $this->transaction->when($type == 'challan', function($q) {
            $q->whereIn('type', [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT]);
        })->when($type != 'challan', function($q) {
            $q->whereNotIn('type', [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT]);
        })->whereCodeNumber(request('code_number'))->when($uuid, function($q, $uuid) {
            $q->where('uuid', '!=', $uuid);
        })->count();

        if ($duplicate_transaction_number) {
            throw ValidationException::withMessages(['code_number' => trans('transaction.duplicate_code_number')]);
        }

        $code_number = $this->getCodeNumber();

        return request('code_number') == Arr::get($code_number, 'code_number') ? $code_number : [];
    }

    /**
     * Validate ledger group
     *
     * @param Transaction $transaction
     * @param Ledger $ledger
     * @param integer $index
     */
    private function validateLedgerGroup(Transaction $transaction, Ledger $ledger, $index = 0) : void
    {
        $ledger_type = $ledger->type->code;

        if ($transaction->type == TransactionType::CONTRA && $ledger->id == $transaction->ledger_id) {
            throw ValidationException::withMessages(['rows.' . $index . '.ledger' => trans('validation.distinct', ['attribute' => trans('accounting.ledger.ledger')])]);
        }

        if (in_array($transaction->type, [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT]) && ! in_array($ledger_type, [LedgerGroup::SUNDRY_CREDITOR, LedgerGroup::SUNDRY_DEBTOR])) {
            throw ValidationException::withMessages(['rows.' . $index . '.ledger' => trans('transaction.invalid_ledger_for_challan_transaction')]);
        }

        if (in_array($transaction->type, [TransactionType::PAYMENT]) && in_array($ledger_type, [LedgerGroup::INDIRECT_INCOME])) {
            throw ValidationException::withMessages(['rows.' . $index . '.ledger' => trans('transaction.invalid_ledger_for_transaction')]);
        }

        if (in_array($transaction->type, [TransactionType::RECEIPT]) && in_array($ledger_type, [LedgerGroup::INDIRECT_EXPENSE])) {
            throw ValidationException::withMessages(['rows.' . $index . '.ledger' => trans('transaction.invalid_ledger_for_transaction')]);
        }
    }

    /**
     * Reverse ledger balance
     *
     * @param Transaction $transaction
     */
    private function reverseLedgerBalance(Transaction $transaction) : void
    {
        if (in_array($transaction->type, [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT])) {
            return;
        }

        foreach ($transaction->details as $transaction_detail) {
            $ledger = $transaction_detail->ledger;
            $current = $ledger->current;

            if ($transaction->is_non_taxed) {
                $current['balance_non_taxed'] = $ledger->current_balance_non_taxed - ($this->getSecondaryLedgerMultiplier($transaction->type, $ledger) * $transaction_detail->amount);
            } else {
                $current['balance_taxed'] = $ledger->current_balance_taxed - ($this->getSecondaryLedgerMultiplier($transaction->type, $ledger) * $transaction_detail->amount);
            }

            $ledger->current = $current;
            $ledger->save();
        }
    }

    /**
     * Reverse primary ledger balance
     *
     * @param Transaction $transaction
     */
    private function reversePrimaryLedgerBalance(Transaction $transaction) : void
    {
        if (in_array($transaction->type, [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT])) {
            return;
        }

        $ledger = $transaction->ledger;
        $current = $ledger->current;

        if ($transaction->is_non_taxed) {
            $current['balance_non_taxed'] = $ledger->current_balance_non_taxed - ($this->getPrimaryLedgerMultiplier($transaction->type) * $transaction->amount);
        } else {
            $current['balance_taxed'] = $ledger->current_balance_taxed - ($this->getPrimaryLedgerMultiplier($transaction->type) * $transaction->amount);
        }

        $ledger->current = $current;
        $ledger->save();
    }

    /**
     * Update primary ledger balance
     *
     * @param Transaction $transaction
     */
    private function updatePrimaryLedgerBalance(Transaction $transaction) : void
    {
        if (in_array($transaction->type, [TransactionType::CHALLAN_PAYMENT, TransactionType::CHALLAN_RECEIPT])) {
            return;
        }

        $ledger = $transaction->ledger;
        $current = $ledger->current;

        $key = $transaction->is_non_taxed ? 'balance_non_taxed' : 'balance_taxed';
        $ledger_key = $transaction->is_non_taxed ? 'current_balance_non_taxed' : 'current_balance_taxed';

        $current[$key] = $ledger->$ledger_key + ($this->getPrimaryLedgerMultiplier($transaction->type) * $transaction->amount);

        $ledger->current = $current;
        $ledger->save();
    }

    /**
     * Update transaction details
     *
     * @param Transaction $transaction
     */
    private function updateDetails(Transaction $transaction) : Transaction
    {
        $ledgerFunction = 'findByUuidOrFail';

        if ($transaction->type == TransactionType::PAYMENT) {
            $ledgerFunction = 'findByUuidOrFailForPayment';
        } else if ($transaction->type == TransactionType::RECEIPT) {
            $ledgerFunction = 'findByUuidOrFailForReceipt';
        }

        $transaction_detail_ledger_ids = $transaction->details->pluck('ledger_id')->all();

        $amount = 0;
        $ledger_ids = array();
        foreach (request('rows', []) as $index => $row) {
            $ledger = $this->ledger->$ledgerFunction(request('rows.' . $index . '.ledger.uuid'), 'rows.' . $index . '.ledger');

            $ledger_ids[] = $ledger->id;

            $this->validateLedgerGroup($transaction, $ledger, $index);

            $transaction_detail = $this->transaction_detail->firstOrCreate([
                'transaction_id' => $transaction->id,
                'ledger_id' => $ledger->id
            ]);

            $current = $ledger->current;
            $transaction_detail->uuid = $transaction_detail->uuid ? : (string) Str::uuid();
            $transaction_detail->amount = request('rows.' . $index . '.amount');
            $transaction_detail->description = request('rows.' . $index . '.description');
            $transaction_detail->save();

            $key = $transaction->is_non_taxed ? 'balance_non_taxed' : 'balance_taxed';
            $ledger_key = $transaction->is_non_taxed ? 'current_balance_non_taxed' : 'current_balance_taxed';

            $current[$key] = $ledger->$ledger_key + ($this->getSecondaryLedgerMultiplier($transaction->type, $ledger) * $transaction_detail->amount);

            $ledger->current = $current;
            $ledger->save();

            $amount += $transaction_detail->amount;
        }

        $removed_ledger_ids = array_diff($transaction_detail_ledger_ids, $ledger_ids);

        $this->transaction_detail->whereTransactionId($transaction->id)->whereIn('ledger_id', $removed_ledger_ids)->delete();

        $transaction->amount = $amount;
        $transaction->save();

        $this->updatePrimaryLedgerBalance($transaction);

        return $transaction;
    }

    /**
     * Update payment status
     *
     * @param Transaction $transaction
     */
    private function updateVoucherPaymentStatus(Transaction $transaction) : void
    {
    }

    /**
     * Reverse voucher payment status
     *
     * @param Transaction $transaction
     */
    private function reverseVoucherPaymentStatus(Transaction $transaction) : void
    {
    }

    /**
     * Get voucher payment status
     *
     * @param Voucher $voucher
     */
    private function getVoucherPaymentStatus(Voucher $voucher) : string
    {
        $balance = $voucher->total - $voucher->paid;

        if ($balance <= 0) {
            return PaymentStatus::PAID;
        } else if ($balance > 0 && $voucher->paid > 0) {
            return PaymentStatus::PARTIALLY_PAID;
        }

        return PaymentStatus::UNPAID;
    }

    /**
     * Update given transaction
     * @param Transaction $transaction
     */
    public function update(Transaction $transaction) : Transaction
    {
        $type = request('type');

        if ($type != $transaction->type) {
            throw ValidationException::withMessages(['message' => trans('global.could_not_change', ['attribute' => trans('transaction.props.type')])]);
        }

        \DB::beginTransaction();

        // Reverse all the ledger balance
        $this->reverseLedgerBalance($transaction);

        // Reverse the primary ledger balance
        $this->reversePrimaryLedgerBalance($transaction);

        $transaction->forceFill($this->formatParams($transaction->uuid))->save();

        $transaction = $this->updateDetails($transaction);

        \DB::commit();

        return $transaction;
    }

    /**
     * Delete transaction
     * @param Transaction $transaction
     */
    public function delete(Transaction $transaction) : void
    {
        \DB::beginTransaction();

        // Reverse all the ledger balance
        $this->reverseLedgerBalance($transaction);

        // Reverse the primary ledger balance
        $this->reversePrimaryLedgerBalance($transaction);

        // $this->reverseVoucherPaymentStatus($transaction);

        $transaction->delete();

        \DB::commit();
    }
}
