<?php
namespace App\Repositories\Accounting;

use App\Enums\LedgerGroup;
use App\Enums\PaymentStatus;
use App\Enums\VoucherStatus;
use App\Enums\VoucherType;
use App\Helpers\ArrHelper;
use App\Helpers\CalHelper;
use App\Helpers\SysHelper;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use App\Models\Accounting\Voucher;
use App\Repositories\OptionRepository;
use App\Repositories\ContactRepository;
use App\Models\Accounting\VoucherDetail;
use App\Repositories\Setting\TaxRepository;
use App\Repositories\Inventory\ItemRepository;
use Illuminate\Validation\ValidationException;
use App\Http\Resources\Accounting\VoucherSummaryCollection;
use App\Models\Accounting\VoucherTax;
use App\Models\Inventory\Item;
use App\Traits\VoucherConstraints;

class VoucherRepository
{
    use VoucherConstraints;

    protected $voucher;
    protected $voucher_detail;
    protected $ledger;
    protected $contact;
    protected $tax;
    protected $option;
    protected $item;
    protected $voucher_tax;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Voucher $voucher,
        VoucherDetail $voucher_detail,
        LedgerRepository $ledger,
        ContactRepository $contact,
        TaxRepository $tax,
        OptionRepository $option,
        ItemRepository $item,
        VoucherTax $voucher_tax
    ) {
        $this->voucher = $voucher;
        $this->voucher_detail = $voucher_detail;
        $this->ledger = $ledger;
        $this->contact = $contact;
        $this->tax = $tax;
        $this->option = $option;
        $this->item = $item;
        $this->voucher_tax = $voucher_tax;
    }

    /**
     * Find voucher with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Voucher
    {
        $voucher = $this->voucher->find($id);

        if (! $voucher) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('voucher.voucher')])]);
        }

        return $voucher;
    }

    /**
     * Find voucher with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Voucher
    {
        $voucher = $this->voucher->filterByUuid($uuid)->first();

        if (! $voucher) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('voucher.voucher')])]);
        }

        return $voucher;
    }

    /**
     * Get all filtered data
     */
    public function getData($params = array())
    {
        $sort_by = $this->voucher->getSortBy();
        $order   = $this->voucher->getOrder();

        $keywords            = request()->query('keywords');
        $code_number         = request()->query('code_number');
        $start_date          = request()->query('start_date');
        $end_date            = request()->query('end_date');
        $start_due_date      = request()->query('start_due_date');
        $end_due_date        = request()->query('end_due_date');
        $ledger_uuid         = request()->query('ledger');
        $referral_uuid       = request()->query('referral');
        $contact_uuid        = request()->query('contact');
        $enquiry_type_uuid   = request()->query('enquiry_type');
        $enquiry_source_uuid = request()->query('enquiry_source');

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        return $this->voucher->whereType(request('type'))->dateBetween('date', [
            'start_date' => $start_date,
            'end_date'   => $end_date
        ])->when($code_number, function($q, $code_number) {
            $q->where('code_number', 'like', '%'.$code_number.'%');
        })->dateBetween('due_date', [
            'start_date' => $start_due_date,
            'end_date'   => $end_due_date
        ])->when($referral_uuid, function($q, $referral_uuid) {
            $q->whereHas('referral', function($q1) use($referral_uuid) {
                $q1->whereUuid($referral_uuid);
            });
        })->when($ledger_uuid, function($q, $ledger_uuid) {
            $q->whereHas('ledger', function($q1) use($ledger_uuid) {
                $q1->whereUuid($ledger_uuid);
            });
        })->when($contact_uuid, function($q, $contact_uuid) {
            $q->whereHas('contact', function($q1) use($contact_uuid) {
                $q1->whereUuid($contact_uuid);
            });
        })->when($enquiry_type_uuid, function($q, $enquiry_type_uuid) {
            $q->whereHas('enquiryType', function($q1) use($enquiry_type_uuid) {
                $q1->whereUuid($enquiry_type_uuid);
            });
        })->when($enquiry_source_uuid, function($q, $enquiry_source_uuid) {
            $q->whereHas('enquirySource', function($q1) use($enquiry_source_uuid) {
                $q1->whereUuid($enquiry_source_uuid);
            });
        })->when($keywords, function($q, $keywords) {
            $q->where(function($q1) use ($keywords) {
                $q1->where('note', 'like', '%'.$keywords.'%')
                ->orWhere('terms', 'like', '%'.$keywords.'%')
                ->orWhere('subject', 'like', '%'.$keywords.'%')
                ->orWhere('description', 'like', '%'.$keywords.'%')
                ->orWhere('memo', 'like', '%'.$keywords.'%');
            });
        })->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        })->orderBy($sort_by, $order);
    }

    /**
     * Paginate all vouchers
     */
    public function paginate() : VoucherSummaryCollection
    {
        $per_page     = $this->voucher->getPerPage();
        $current_page = $this->voucher->getCurrentPage();

        return new VoucherSummaryCollection($this->getData()->paginate((int) $per_page, ['*'], 'current_page'));
    }

    private function getCodeNumber() : array
    {
        $type = str_replace('-', '_', request('type'));

        $number_prefix = config('config.'.$type.'.number_prefix');
        $number_suffix = config('config.'.$type.'.number_suffix');
        $digit = config('config.'.$type.'.number_digit');

        $number_format = $number_prefix . '%NUMBER%' . $number_suffix;
        $code_number = (int) $this->voucher->whereType(request('type'))->whereNumberFormat($number_format)->max('number');
        $code_number += 1;

        $format = str_replace('%YEAR%', date('Y'), $number_format);
        $format = str_replace('%YEAR_SHORT%', date('y'), $format);
        $format = str_replace('%MONTH%', date('F'), $format);
        $format = str_replace('%MONTH_SHORT%', date('M'), $format);
        $format = str_replace('%MONTH_NUMBER%', date('m'), $format);
        $format = str_replace('%MONTH_NUMBER_SHORT%', date('n'), $format);
        $format = str_replace('%DAY%', date('d'), $format);
        $format = str_replace('%DAY_SHORT%', date('j'), $format);
        return array(
            'code_number'   => str_replace('%NUMBER%', str_pad($code_number, $digit, '0', STR_PAD_LEFT), $format),
            'number_format' => $number_format,
            'number'        => $code_number
        );
    }

    /**
     * Get pre requisite.
     */
    public function getPreRequisite() : array
    {
        $taxes = $this->tax->getAll();
        $transports = $this->option->listOrderedByName([ 'type' => 'transport' ]);
        $code_number = Arr::get($this->getCodeNumber(), 'code_number');

        $enquiry_types = array();
        $enquiry_sources = array();
        $statuses = array();

        if ($this->requiresEnquiryAttributes()) {
            $enquiry_types = $this->option->listOrderedByName([ 'type' => 'enquiry_type' ]);
            $enquiry_sources = $this->option->listOrderedByName([ 'type' => 'enquiry_source' ]);
            $statuses = ArrHelper::getTransList('statuses', 'enquiry');
        }

        return compact('taxes', 'transports', 'code_number', 'enquiry_types', 'enquiry_sources', 'statuses');
    }

    /**
     * Create a new voucher
     */
    public function create() : Voucher
    {
        \DB::beginTransaction();

        $voucher = $this->voucher->forceCreate($this->formatParams());

        $voucher = $this->validateVoucherDetail($voucher);

        $this->updateLedger($voucher);

        $this->updateItemRate($voucher->details->pluck('item_id')->all());

        \DB::commit();

        return $voucher;
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $code_number = $this->validateCodeNumber($uuid);

        $ledger         = request('ledger') ? $this->ledger->findByUuidOrFailForVoucher(request('ledger.uuid'), 'ledger') : null;
        $contact        = request('contact') ? $this->contact->findByUuidOrFail(request('contact.uuid'), 'contact') : null;
        $referral       = request('referral') ? $this->contact->findByUuidOrFail(request('referral.uuid'), 'referral') : null;
        $transport      = request('transport') ? $this->option->findByTypeWithUuidOrFail('transport', request('transport.uuid'), 'transport') : null;
        $enquiry_type   = request('enquiry_type') ? $this->option->findByTypeWithUuidOrFail('enquiry_type', request('enquiry_type.uuid'), 'enquiry_type') : null;
        $enquiry_source = request('enquiry_source') ? $this->option->findByTypeWithUuidOrFail('enquiry_source', request('enquiry_source.uuid'), 'enquiry_source') : null;

        if ($ledger) {
            $this->validateBillingInfo($ledger->type->code);
        }

        if (request('type') == VoucherType::PURCHASE) {
            $existing_voucher = $this->voucher->whereType(VoucherType::PURCHASE)->whereLedgerId($ledger->id)->whereCodeNumber(request('code_number'))->when($uuid, function($q, $uuid) {
                $q->where('uuid', '!=', $uuid);
            })->count();

            if ($existing_voucher) {
                throw ValidationException::withMessages(['message' => trans('voucher.duplicate_code_number')]);
            }
        }

        $formatted = [
            'code_number'            => request('code_number'),
            'number_format'          => Arr::get($code_number, 'number_format'),
            'number'                 => Arr::get($code_number, 'number'),
            'reference_number'       => request('reference_number'),
            'date'                   => CalHelper::toDate(request('date')),
            'due_date'               => CalHelper::toDate(request('due_date')),
            'delivery_date'          => CalHelper::toDate(request('delivery_date')),
            'expected_delivery_date' => CalHelper::toDate(request('expected_delivery_date')),
            'is_non_taxed'           => request()->boolean('is_non_taxed'),
            'ledger_id'              => optional($ledger)->id,
            'contact_id'             => optional($contact)->id,
            'referral_id'            => optional($referral)->id,
            'transport_id'           => optional($transport)->id,
            'enquiry_type_id'        => optional($enquiry_type)->id,
            'enquiry_source_id'      => optional($enquiry_source)->id,
            'tracking_number'        => request('tracking_number'),
            'subject'                => request('subject'),
            'description'            => request('description'),
            'memo'                   => request('memo'),
            'note'                   => request('note'),
        ];

        if (! $uuid) {
            $formatted['payment_status'] = $this->getPaymentStatus($ledger);
            $formatted['type'] = request('type');
            $formatted['user_id'] = \Auth::id();
            $formatted['status'] = $this->getStatus();
        }

        return $formatted;
    }

    /**
     * Validate billing information
     *
     * @param string $ledger_group
     */
    private function validateBillingInfo($ledger_group = null) : void
    {
        if ($ledger_group != LedgerGroup::CASH) {
            return;
        }

        request()->validate([
            'billing_info.mobile' => 'required_without:billing_info.email',
            'billing_info.email' => 'email|required_without:billing_info.mobile'
        ], [], [
            'billing_info.mobile' => trans('contact.props.mobile'),
            'billing_info.email' => trans('contact.props.email'),
        ]);

        $this->contact->addIfNotExists([
            'name' => request('billing_info.billing_name'),
            'mobile' => request('billing_info.mobile'),
            'email' => request('billing_info.email'),
            'address' => array(
                'address_line1' => request('billing_info.address_line1'),
                'address_line2' => request('billing_info.address_line2'),
                'city'          => request('billing_info.city'),
                'state'         => request('billing_info.state'),
                'zipcode'       => request('billing_info.zipcode'),
                'country'       => request('billing_info.country')
            )
        ]);
    }

    private function getStatus()
    {
        if ($this->requiresActionableStatus()) {
            return VoucherStatus::OPEN;
        }

        return VoucherStatus::PUBLISHED;
    }

    private function getPaymentstatus($ledger)
    {
        if (! $this->canHavePaymentStatus()) {
            return null;
        }

        if ($ledger && $ledger->type->code == LedgerGroup::CASH) {
            return PaymentStatus::PAID;
        }

        return PaymentStatus::UNPAID;
    }

    /**
     * Validate code number
     *
     * @param string $uuid
     */
    private function validateCodeNumber($uuid = null) : array
    {
        if (! $this->shouldHaveValidCodeNumber()) {
            return [];
        }

        $duplicate_voucher_number = $this->voucher->whereType(request('type'))->whereCodeNumber(request('code_number'))->when($uuid, function($q, $uuid) {
            $q->where('uuid', '!=', $uuid);
        })->count();

        if ($duplicate_voucher_number) {
            throw ValidationException::withMessages(['code_number' => trans('voucher.duplicate_code_number')]);
        }

        $code_number = $this->getCodeNumber();

        return request('code_number') == Arr::get($code_number, 'code_number') ? $code_number : [];
    }

    /**
     * Update voucher detail
     *
     * @param Voucher $voucher
     */
    private function validateVoucherDetail(Voucher $voucher, $action = 'create') : Voucher
    {
        $discounted_item_sub_total = 0;
        $discounted_sub_total = 0;
        $taxed_sub_total = 0;
        $items = array();
        $line_item_taxes = array();
        $group_taxable_total = array();

        $taxes = $this->tax->get();
        foreach (request('items', []) as $index => $item_list) {
            $item_discounts = array();
            $item_taxes     = array();

            if (request()->boolean('items.'.$index.'.custom') && ! $this->canHaveCustomItem()) {
                throw ValidationException::withMessages(['message' => trans('voucher.cannot_contain_custom_item')]);
            }

            $item           = ! request()->boolean('items.'.$index.'.custom') ? $this->item->findByUuidOrFail(request('items.'.$index.'.item.uuid'), 'items.' . $index . '.item') : null;
            $rate           = request('items.'.$index.'.rate', 0);
            $quantity       = request('items.'.$index.'.quantity', 1);
            $discount_value = request('items.'.$index.'.discount', 0);

            $is_non_taxed = false;
            if ($voucher->is_non_taxed) {
                $is_non_taxed = true;
            } else {
                $is_non_taxed   = $this->canHaveNonTaxedItem() ? request()->boolean('items.'.$index.'.is_non_taxed') : false;
            }

            $discounted_rate = $rate;
            if ($discount_value) {
                $discount_amount = SysHelper::formatCurrency($rate * $discount_value / 100);
                $discounted_rate = SysHelper::formatCurrency($rate - $discount_amount);
                $item_discounts[] = array(
                    'value' => (float) $discount_value,
                    'amount' => (float) $discount_amount,
                    'amount_with_discount' => (float) $discounted_rate
                );
            }

            $amount = SysHelper::formatCurrency($discounted_rate * $quantity);
            $discounted_item_sub_total += $amount;

            $amount = $this->calculateVoucherLevelDiscounts($amount);

            $discounted_item_amount = $amount;
            $discounted_sub_total += $amount;

            $item_tax = $this->getItemLevelTax($index, $amount, $taxes);

            if ($item_tax) {
                $line_item_taxes[] = Arr::get($item_tax, 'uuid');
                $group_taxable_total[] = array(
                    'uuid' => Arr::get($item_tax, 'uuid'),
                    'name' => Arr::get($item_tax, 'name'),
                    'value' => Arr::get($item_tax, 'value'),
                    'amount' => Arr::get($item_tax, 'amount'),
                );

                $taxed_amount = SysHelper::formatCurrency($amount * Arr::get($item_tax, 'value', 0) / 100);
                $amount += $taxed_amount;
            }

            $taxed_sub_total += $amount;

            if (! empty($item_tax)) {
                $item_taxes[] =  $item_tax;
            }

            $meta = array(
                'discounted_item_amount' => $discounted_item_amount,
                'taxed_item_amount'      => $amount,
                'show_description'       => request()->boolean('items.'.$index.'.show_description')
            );

            if ($this->hasFulfillment()) {
                $meta['is_fulfilled'] = request()->boolean('items.'.$index.'.is_fulfilled');
            }

            $item_name = request()->boolean('items.'.$index.'.custom') ? request('items.'.$index.'.item.name') : null;
            $unit_name = request()->boolean('items.'.$index.'.custom') ? request('items.'.$index.'.item.unit.code') : null;

            $items[] = array(
                'uuid'         => (string) Str::uuid(),
                'voucher_id'   => $voucher->id,
                'is_non_taxed' => $is_non_taxed,
                'item_id'      => optional($item)->id,
                'item_name'    => $item_name,
                'unit_name'    => $unit_name,
                'description'  => request('items.'.$index.'.description'),
                'quantity'     => $quantity,
                'rate'         => $rate,
                'discount'     => json_encode($item_discounts),
                'tax'          => json_encode($item_taxes),
                'amount'       => $amount,
                'meta'         => json_encode($meta)
            );
        }

        $voucher_discounts = $this->getVoucherLevelDiscounts($discounted_item_sub_total);

        $voucher_taxes = $this->getVoucherLevelTax($discounted_sub_total, $taxed_sub_total, $taxes, $line_item_taxes);

        $this->updateVoucherTax($group_taxable_total, $voucher_taxes, $taxes, $voucher->id);

        $sub_total = $taxed_sub_total + collect($voucher_taxes)->sum('amount');

        $shipping  = (float) request('shipping', 0) ? SysHelper::formatCurrency(request('shipping')) : 0;
        $round_off = (float) request('round_off', 0);

        $total_without_roundoff = SysHelper::formatCurrency($sub_total + $shipping);
        $total = SysHelper::formatCurrency($sub_total + $shipping + $round_off);

        $this->validateTotal($total, $total_without_roundoff);

        $voucher->amount    = $sub_total;
        $voucher->shipping  = $shipping;
        $voucher->round_off = $round_off;
        $voucher->total     = $total;
        $voucher->discount  = $voucher_discounts;
        $voucher->tax       = $voucher_taxes;

        $meta = $voucher->meta;
        $meta['config']['round_off_forced'] = (bool) request('round_off_forced');
        $voucher->meta = $meta;

        $info = $voucher->info;
        $info['billing_info'] = array(
            'billing_name'  => request('billing_info.billing_name'),
            'address_line1' => request('billing_info.address_line1'),
            'address_line2' => request('billing_info.address_line2'),
            'city'          => request('billing_info.city'),
            'state'         => request('billing_info.state'),
            'zipcode'       => request('billing_info.zipcode'),
            'country'       => request('billing_info.country'),
            'mobile'        => request('billing_info.mobile'),
            'email'         => request('billing_info.email'),
        );
        $voucher->info = $info;

        $voucher->save();

        if ($this->doesAffectQuantity()) {
            $this->updateVoucherDetail($voucher, $items);
        } else {
            $this->updateNonQuantityVoucherDetail($voucher, $items);
        }

        return $voucher;
    }

    /**
     * Validate total
     *
     * @param float $total
     * @param float $total_without_roundoff
     */
    private function validateTotal($total, $total_without_roundoff) : void
    {
        if (! $this->shouldHaveValidTotal()) {
            return;
        }

        if (request('round_off') && round($total, 0) != $total) {
            $calculated_round_off = SysHelper::formatCurrency($total_without_roundoff - round($total_without_roundoff, 0));
            throw ValidationException::withMessages(['round_off' => trans('voucher.invalid_round_off', ['given' => request('round_off'), 'calculated' => $calculated_round_off])]);
        }

        if (request('items', []) && request('total.total_amount') != $total) {
            throw ValidationException::withMessages(['message' => trans('voucher.total_mismatch', ['given' => request('total.total_amount'), 'calculated' => $total])]);
        }
    }

    /**
     * Update voucher tax
     *
     * @param array $group_taxable_total
     * @param array $voucher_level_taxes
     * @param Collection $taxes
     * @param int $voucher_id
     */
    private function updateVoucherTax($group_taxable_total, $voucher_level_taxes, $taxes, $voucher_id) : void
    {
        if (! $this->canHaveTaxSummary()) {
            return;
        }

        foreach ($voucher_level_taxes as $voucher_tax) {
            $group_taxable_total[] = array(
                'uuid' => Arr::get($voucher_tax, 'uuid'),
                'value' => Arr::get($voucher_tax, 'value'),
                'amount' => Arr::get($voucher_tax, 'amount'),
            );
        }

        $voucher_taxes = collect($group_taxable_total)->groupBy('uuid')->map(function ($group) {
            return [
                'uuid' => $group->first()['uuid'],
                'value' => $group->first()['value'],
                'amount' => $group->sum('amount')
            ];
        })->toArray();

        $existing_voucher_tax_ids = $this->voucher_tax->whereVoucherId($voucher_id)->pluck('tax_id')->all();
        $new_voucher_tax_ids = array();
        foreach ($voucher_taxes as $voucher_tax_list) {
            $tax = $taxes->where('uuid', Arr::get($voucher_tax_list, 'uuid'))->first();

            if (! $tax) {
                throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('setting.tax.tax')])]);
            }

            $value = Arr::get($voucher_tax_list, 'value');
            $amount = Arr::get($voucher_tax_list, 'amount');

            $new_voucher_tax_ids[] = $tax->id;

            $voucher_tax = $this->voucher_tax->firstOrCreate([
                'voucher_id' => $voucher_id,
                'tax_id' => $tax->id
            ]);

            $tax_details = is_array($tax->details) ? $tax->details : [];

            $details = array();
            foreach ($tax_details as $tax_detail) {
                $details[] = array(
                    'name' => Arr::get($tax_detail, 'label'),
                    'value' => Arr::get($tax_detail, 'value', 0),
                    'amount' => SysHelper::formatCurrency(((Arr::get($tax_detail, 'value', 0) / $value) * $amount))
                );
            }

            $voucher_tax->value = $value;
            $voucher_tax->amount = $amount;
            $voucher_tax->meta = array('details' => $details);
            $voucher_tax->save();
        }

        $remove_taxes_ids = array_diff($existing_voucher_tax_ids, $new_voucher_tax_ids);
        $this->voucher_tax->whereVoucherId($voucher_id)->whereIn('tax_id', $remove_taxes_ids)->delete();
    }

    /**
     * Calculate item level tax
     *
     * @param integer $item_index
     * @param float $amount
     * @param collection $taxes
     */
    private function getItemLevelTax($item_index, $amount, $taxes) : array
    {
        if (! $this->canHaveTax()) {
            return [];
        }

        $tax_amount = 0;
        $tax_value = request('items.'.$item_index.'.tax.value', 0);

        if ($tax_value == 0 || empty($tax_value)) {
            return [];
        }

        if (request('is_non_taxed')) {
            return [];
        }

        $is_non_taxed = $this->canHaveNonTaxedItem() ? request()->boolean('items.'.$item_index.'.is_non_taxed') : false;

        if ($is_non_taxed) {
            return [];
        }

        $custom_item = request()->boolean('items.'.$item_index.'.custom', false);

        $tax = null;
        if (! $custom_item) {
            $tax = $taxes->firstWhere('uuid', request('items.'.$item_index.'.tax.uuid'));

            if (! $tax) {
                throw ValidationException::withMessages(['items.' . $item_index . '.tax' => trans('global.could_not_find', ['attribute' => trans('setting.tax.tax')])]);
            }

            if ($tax->getMeta('voucher_level_tax')) {
                throw ValidationException::withMessages(['items.' . $item_index . '.tax' => trans('global.could_not_find', ['attribute' => trans('setting.tax.tax')])]);
            }
        }

        $tax_amount = SysHelper::formatCurrency($amount * $tax_value / 100);

        $item_tax = array(
            'uuid'               => $tax ? $tax->uuid : request('items.'.$item_index.'.tax.uuid'),
            'name'               => $tax ? $tax->name : 'custom',
            'value'              => (float) $tax_value,
            'amount'             => (float) $tax_amount,
            'amount_with_tax'    => (float) SysHelper::formatCurrency($amount + $tax_amount),
            'calculate_on_total' => (bool) optional($tax)->getMeta('calculate_on_total'),
            'voucher_level_tax'  => (bool) optional($tax)->getMeta('voucher_level_tax')
        );

        return $item_tax;
    }

    /**
     * Get voucher level taxes
     *
     * @param float $sub_total
     * @param float $taxable_total
     * @param collection $taxes
     * @param array $line_item_taxes
     */
    private function getVoucherLevelTax($sub_total, $taxable_total, $taxes, $line_item_taxes) : array
    {
        if (! $this->canHaveTax()) {
            return [];
        }

        if (! $this->canHaveVoucherLevelTax()) {
            return [];
        }

        if (request('is_non_taxed')) {
            return [];
        }

        $voucher_taxes = array();

        foreach (request('taxes', []) as $index => $list) {

            $tax_uuid = request('taxes.'.$index.'.tax.uuid');
            $voucher_tax = request('taxes.'.$index.'.tax.value', 0);

            if (! in_array($tax_uuid, $line_item_taxes) && $voucher_tax)
            {
                $tax = $taxes->firstWhere('uuid', $tax_uuid);

                if (! $tax) {
                    throw ValidationException::withMessages(['taxes.' . $index . '.tax' => trans('global.could_not_find', ['attribute' => trans('setting.tax.tax')])]);
                }

                if (! $tax->getMeta('voucher_level_tax')) {
                    throw ValidationException::withMessages(['taxes.' . $index . '.tax' => trans('voucher.could_not_apply_non_voucher_level_tax_in_voucher_level')]);
                }

                $on_amount    = $tax->getMeta('calculate_on_total') ? $taxable_total : $sub_total;
                $tax_amount   = SysHelper::formatCurrency($on_amount * $voucher_tax / 100);
                $voucher_taxes[] = array(
                    'uuid'               => $tax->uuid,
                    'name'               => $tax->name,
                    'value'              => (float) $voucher_tax,
                    'amount'             => $tax_amount,
                    'on_amount'          => SysHelper::formatCurrency($on_amount),
                    'amount_with_tax'    => SysHelper::formatCurrency($on_amount + $tax_amount),
                    'on_total'           => (bool) $tax->getMeta('calculate_on_total'),
                    'calculate_on_total' => (bool) $tax->getMeta('calculate_on_total'),
                    'voucher_level_tax'  => (bool) $tax->getMeta('voucher_level_tax')
                );
            }
        }

        return $voucher_taxes;
    }

    /**
     * Calculate voucher level discounts
     *
     * @param float $amount
     */
    private function calculateVoucherLevelDiscounts($amount = 0) : float
    {
        if (! $this->canHaveVoucherLevelDiscount()) {
            return $amount;
        }

        foreach (request('discounts', []) as $discount_index => $discount_list) {
            $voucher_discount_value = request('discounts.'.$discount_index.'.value', 0);
            $voucher_discount_amount = SysHelper::formatCurrency($amount * $voucher_discount_value / 100);
            $amount -= $voucher_discount_amount;
        }

        return SysHelper::formatCurrency($amount);
    }

    /**
     * Get voucher level discounts
     *
     * @param float $amount
     */
    private function getVoucherLevelDiscounts($amount) : array
    {
        if ($this->doesNotHaveDiscount()) {
            return [];
        }

        if (! $this->canHaveVoucherLevelDiscount()) {
            return [];
        }

        $voucher_discounts = array();

        foreach (request('discounts', []) as $index => $discount_list) {
            $voucher_discount_value = request('discounts.'.$index.'.value', 0);
            $voucher_discount_amount = SysHelper::formatCurrency($amount * $voucher_discount_value / 100);
            $amount -= $voucher_discount_amount;
            $voucher_discounts[] = array(
                'uuid' => (string) Str::uuid(),
                'name' => request('discounts.'.$index.'.name'),
                'value' => $voucher_discount_value,
                'amount' => $voucher_discount_amount,
                'amount_with_discount' => $amount
            );
        }

        return $voucher_discounts;
    }

    /**
     * Update voucher details when quantity is not affected
     *
     * @param Voucher $voucher
     * @param array $items
     */
    private function updateNonQuantityVoucherDetail(Voucher $voucher, $items = array()) : void
    {
        $voucher_detail_item_ids = $voucher->details->pluck('item_id')->all();
        $voucher_detail_item_names = $voucher->details->pluck('item_name')->all();

        $item_ids = array();
        $item_names = array();
        foreach ($items as $item) {
            if (Arr::get($item, 'item_id')) {
                $item_ids[] = Arr::get($item, 'item_id');

                $voucher_detail = $this->voucher_detail->firstOrCreate([
                    'voucher_id' => $voucher->id,
                    'item_id' => Arr::get($item, 'item_id')
                ]);
            } else {
                $item_names[] = Arr::get($item, 'item_name');

                $voucher_detail = $this->voucher_detail->firstOrCreate([
                    'voucher_id' => $voucher->id,
                    'item_name' => Arr::get($item, 'item_name')
                ]);
            }

            if (! $voucher_detail) {
                throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('inventory.item.item')])]);
            }

            $voucher_detail->item_name    = Arr::get($item, 'item_name');
            $voucher_detail->unit_name    = Arr::get($item, 'unit_name');
            $voucher_detail->quantity     = Arr::get($item, 'quantity');
            $voucher_detail->is_non_taxed = Arr::get($item, 'is_non_taxed');
            $voucher_detail->description  = Arr::get($item, 'description');
            $voucher_detail->rate         = Arr::get($item, 'rate');
            $voucher_detail->discount     = json_decode(Arr::get($item, 'discount'), true);
            $voucher_detail->tax          = json_decode(Arr::get($item, 'tax'), true);
            $voucher_detail->amount       = Arr::get($item, 'amount');
            $voucher_detail->meta         = json_decode(Arr::get($item, 'meta'), true);
            $voucher_detail->save();
        }

        $removed_item_ids = array_diff($voucher_detail_item_ids, $item_ids);
        $removed_item_names = array_diff($voucher_detail_item_names, $item_names);

        $this->voucher_detail->whereVoucherId($voucher->id)->whereIn('item_id', $removed_item_ids)->delete();
        $this->voucher_detail->whereVoucherId($voucher->id)->whereIn('item_name', $removed_item_names)->delete();
    }

    /**
     * Update voucher details when quantity is affected
     *
     * @param Voucher $voucher
     * @param array $items
     */
    private function updateVoucherDetail(Voucher $voucher, $items = array()) : void
    {
        if (! count($items)) {
            return;
        }

        $items = collect($items);
        $item_ids = $items->pluck('item_id')->all();

        $voucher_details = $this->voucher_detail->whereVoucherId($voucher->id)->get();
        $voucher_detail_item_ids = $voucher_details->pluck('item_id')->all();

        $new_item_ids = array_diff($item_ids, $voucher_detail_item_ids);
        $this->updateNewItemQuantity($voucher, $items, $new_item_ids);

        $removed_item_ids = array_diff($voucher_detail_item_ids, $item_ids);
        $this->updateRemovedItemQuantity($voucher, $items, $removed_item_ids);

        $same_item_ids = array_intersect($item_ids, $voucher_detail_item_ids);
        $this->updateSameItemQuantity($voucher, $items, $same_item_ids);
    }

    /**
     * Update new item quantity
     *
     * @param Voucher $voucher
     * @param collection $items
     * @param array $new_item_ids
     */
    private function updateNewItemQuantity(Voucher $voucher, $items, $new_item_ids) : void
    {
        if (! $this->doesAffectQuantity()) {
            return;
        }

        $new_items = $items->whereIn('item_id', $new_item_ids)->all();
        $this->voucher_detail->insert($new_items);

        foreach ($new_item_ids as $item_id) {
            $item_data = $items->firstWhere('item_id', $item_id);
            $item_data_quantity = Arr::get($item_data, 'quantity', 0);
            $item_data_non_taxed = $this->canHaveNonTaxedItem() ? Arr::get($item_data, 'is_non_taxed', 0) : false;
            $item = $this->item->findOrFail($item_id);

            if ($item->is_service) {
                continue;
            }

            $current_quantity = $item->current;

            $item_data_quantity *= $this->getQuantityMultiplier();

            if (request('is_non_taxed') || $item_data_non_taxed) {
                $current_quantity['quantity_non_taxed'] = $item->current_quantity_non_taxed + $item_data_quantity;
            } else {
                $current_quantity['quantity_taxed'] = $item->current_quantity_taxed + $item_data_quantity;
            }

            $item->current = $current_quantity;

            $item->save();
        }
    }

    /**
     * Update removed item quantity
     *
     * @param Voucher $voucher
     * @param collection $items
     * @param array $removed_item_ids
     */
    private function updateRemovedItemQuantity(Voucher $voucher, $items, $removed_item_ids) : void
    {
        if (! $this->doesAffectQuantity()) {
            return;
        }

        foreach ($removed_item_ids as $removed_item_id) {
            $item_data = $items->firstWhere('item_id', $removed_item_id);
            $item = $this->item->findOrFail($removed_item_id);

            $current_quantity = $item->current;

            $voucher_detail = $this->voucher_detail->whereVoucherId($voucher->id)->where('item_id', $item->id)->first();

            $item_data_quantity = $voucher_detail->quantity * $this->getQuantityMultiplier();
            $voucher_detail_non_taxed = $this->canHaveNonTaxedItem() ? $voucher_detail->is_non_taxed : false;

            $voucher_detail->delete();

            if ($item->is_service) {
                continue;
            }

            if (request('previous_is_non_taxed') || $voucher_detail_non_taxed) {
                $current_quantity['quantity_non_taxed'] = $item->current_quantity_non_taxed - $item_data_quantity;
            } else {
                $current_quantity['quantity_taxed'] = $item->current_quantity_taxed - $item_data_quantity;
            }

            $item->current = $current_quantity;

            $item->save();
        }
    }

    /**
     * Update same item quantity
     *
     * @param Voucher $voucher
     * @param collection $items
     * @param array $same_item_ids
     */
    private function updateSameItemQuantity(Voucher $voucher, $items, $same_item_ids) : void
    {
        if (! $this->doesAffectQuantity()) {
            return;
        }

        foreach ($same_item_ids as $same_item_id) {
            $item_data = $items->firstWhere('item_id', $same_item_id);
            $item_data_quantity = Arr::get($item_data, 'quantity', 0);
            $item_data_non_taxed = $this->canHaveNonTaxedItem() ? Arr::get($item_data, 'is_non_taxed', 0) : false;
            $item = $this->item->findOrFail($same_item_id);

            $voucher_detail = $this->voucher_detail->whereVoucherId($voucher->id)->whereItemId($same_item_id)->first();

            $voucher_detail->update([
                'quantity' => $item_data_quantity,
                'is_non_taxed' => $item_data_non_taxed,
                'rate' => Arr::get($item_data, 'rate'),
                'discount' => json_decode(Arr::get($item_data, 'discount'), true),
                'tax' => request('is_non_taxed') ? null : json_decode(Arr::get($item_data, 'tax'), true),
                'amount' => Arr::get($item_data, 'amount'),
                'description' => json_decode(Arr::get($item_data, 'description'), true),
                'meta' => json_decode(Arr::get($item_data, 'meta'), true),
            ]);

            if ($item->is_service) {
                continue;
            }

            $this->updateItemQuantity($item, $voucher_detail->quantity * $this->getQuantityMultiplier(), $item_data_quantity * $this->getQuantityMultiplier(), $voucher_detail->is_non_taxed, $item_data_non_taxed);
        }
    }

    /**
     * Update item quantity
     *
     * @param Item $item
     * @param float $quantity
     * @param float $new_quantity
     * @param bool $is_non_taxed
     * @param bool $new_non_taxed
     */
    private function updateItemQuantity(Item $item, $quantity, $new_quantity, $is_non_taxed = false, $new_non_taxed = false) : void
    {
        if (! $this->doesAffectQuantity()) {
            return;
        }

        if (request('previous_is_non_taxed') || $is_non_taxed) {
            $current['quantity_non_taxed'] = $item->current_quantity_non_taxed - $quantity;
        } else {
            $current['quantity_taxed'] = $item->current_quantity_taxed - $quantity;
        }

        $item->current = $current;
        $item->save();

        if (request('is_non_taxed') || $new_non_taxed) {
            $current['quantity_non_taxed'] = $item->current_quantity_non_taxed + $new_quantity;
        } else {
            $current['quantity_taxed'] = $item->current_quantity_taxed + $new_quantity;
        }

        $item->current = $current;
        $item->save();
    }

    /**
     * Reverse item quantity
     *
     * @param Voucher $voucher
     */
    private function reverseItemQuantity(Voucher $voucher) : void
    {
        if (! $this->doesAffectQuantity()) {
            return;
        }

        foreach ($voucher->details as $voucher_detail) {
            $item = $voucher_detail->item;

            if ($item->is_service) {
                continue;
            }

            $current = $item->current;
            if ($voucher_detail->is_non_taxed) {
                $current['quantity_non_taxed'] = $item->current_quantity_non_taxed - ($this->getQuantityMultiplier() * $voucher_detail->quantity);
            } else {
                $current['quantity_taxed'] = $item->current_quantity_taxed - ($this->getQuantityMultiplier() * $voucher_detail->quantity);
            }

            $item->current = $current;
            $item->save();
        }
    }

    /**
     * Update ledger
     *
     * @param Voucher $voucher
     */
    private function updateLedger(Voucher $voucher) : void
    {
        if (! $this->doesAffectLedger()) {
            return;
        }

        if (! $voucher->ledger_id) {
            return;
        }

        $ledger = $this->ledger->findOrFailForVoucher($voucher->ledger_id);
        $current = $ledger->current;

        if (request('is_non_taxed')) {
            $current['balance_non_taxed'] = $ledger->current_balance_non_taxed + ($this->getLedgerMultiplier($ledger) * $voucher->total);
        } else {
            $current['balance_taxed'] = $ledger->current_balance_taxed + ($this->getLedgerMultiplier($ledger) * $voucher->total);
        }

        $ledger->current = $current;
        $ledger->save();
    }

    /**
     * Reverse ledger balance
     *
     * @param Voucher $voucher
     * @param int $ledger_id
     */
    private function reverseLedgerBalance(Voucher $voucher, $ledger_id = null) : void
    {
        if (! $this->doesAffectLedger()) {
            return;
        }

        $ledger_id = $ledger_id ? : $voucher->ledger_id;

        $ledger = $this->ledger->findOrFailForVoucher($ledger_id);
        $current = $ledger->current;

        // if update then previous_is_non_taxed, if delete then is_non_taxed

        if (request('previous_is_non_taxed')) {
            $current['balance_non_taxed'] = $ledger->current_balance_non_taxed - ($this->getLedgerMultiplier($ledger) * request('previous_total'));
        } else {
            $current['balance_taxed'] = $ledger->current_balance_taxed - ($this->getLedgerMultiplier($ledger) * request('previous_total'));
        }

        $ledger->current = $current;
        $ledger->save();

        $this->updateLedger($voucher);
    }

    /**
     * Update given voucher
     */
    private function updateItemRate($item_ids = array()) : void
    {
        if (! $this->needsItemRateUpdate()) {
            return;
        }

        $items = $this->item->findByIds(compact('item_ids'));

        foreach ($items as $item) {
            $price_calculation_method = $item->price_calculation_method ?? $item->category->price_calculation_method;

            $rate_taxed = null;
            $rate_non_taxed = null;

            if ($price_calculation_method == 'last_purchase') {

                $rate_taxed = $this->getLastItemRate($item->id, VoucherType::PURCHASE, false);
                $rate_non_taxed = $this->getLastItemRate($item->id, VoucherType::PURCHASE);

            } else if ($price_calculation_method == 'average_purchase') {

                $rate_taxed = $this->getAverageItemRate($item->id, VoucherType::PURCHASE, false);
                $rate_non_taxed = $this->getAverageItemRate($item->id, VoucherType::PURCHASE);

            }

            $current = $item->current;
            $current['rate_taxed'] = ! is_null($rate_taxed) ? $rate_taxed : 0;
            $current['rate_non_taxed'] = ! is_null($rate_non_taxed) ? $rate_non_taxed : 0;
            $item->current = $current;
            $item->save();
        }
    }

    /**
     * Get last item rate
     *
     * @param int $item_id
     * @param string $type
     * @param bool $non_taxed
     */
    private function getLastItemRate($item_id, $type, $non_taxed = true) : ?float
    {
        $voucher_detail = $this->voucher_detail->with('voucher')->whereIsNonTaxed($non_taxed)->whereItemId($item_id)->whereHas('voucher', function($q) use ($type) {
            $q->whereType($type)->whereStatus(VoucherStatus::PUBLISHED);
        })->select('voucher_details.*', \DB::raw('(SELECT date FROM vouchers WHERE voucher_details.voucher_id = vouchers.id ) as sort_by'))->orderBy('sort_by', 'desc')->orderBy('id', 'desc')->first();

        return $voucher_detail ? $voucher_detail->rate : 0;
    }

    /**
     * Get average item rate
     *
     * @param int $item_id
     * @param string $type
     * @param bool $non_taxed
     */
    private function getAverageItemRate($item_id, $type, $non_taxed = true) : float
    {
        $rate = $this->voucher_detail->with('voucher')->whereIsNonTaxed($non_taxed)->whereItemId($item_id)->whereHas('voucher', function($q) use ($type) {
            $q->whereType($type)->whereStatus(VoucherStatus::PUBLISHED);
        })->avg('rate');

        return $rate ?? 0;
    }

    /**
     * Is voucher updatable
     *
     * @param Voucher $voucher
     */
    private function isUpdatable(Voucher $voucher) : void
    {
        if ($voucher->type != request('type')) {
            throw ValidationException::withMessages(['message' => trans('general.invalid_action')]);
        }

        if ($voucher->ledger_id && ! request('ledger')) {
            throw ValidationException::withMessages(['ledger' => trans('validation.required', ['attribute' => trans('accounting.ledger.ledger')])]);
        }
    }

    /**
     * Update given voucher
     * @param Voucher $voucher
     */
    public function update(Voucher $voucher) : Voucher
    {
        $this->isUpdatable($voucher);

        request()->merge([
            'previous_ledger_id'    => $voucher->ledger_id,
            'previous_is_non_taxed' => $voucher->is_non_taxed,
            'previous_total'        => $voucher->total
        ]);

        $old_item_ids = $voucher->details->pluck('item_id')->all();

        \DB::beginTransaction();

        $voucher = $this->validateVoucherDetail($voucher, 'update');

        $voucher->forceFill($this->formatParams($voucher->uuid))->save();

        if (request('previous_ledger_id') != $voucher->ledger_id) {
            // If ledger is changed
            $this->reverseLedgerBalance($voucher, request('previous_ledger_id'));
        } else if (request('previous_total') != $voucher->total) {
            // If total is changed
            $this->reverseLedgerBalance($voucher, $voucher->ledger_id);
        } else if (request('previous_is_non_taxed') != request('is_non_taxed')) {
            // If is non taxed is changed
            $this->reverseLedgerBalance($voucher, $voucher->ledger_id);
        }

        $voucher->refresh();

        $item_ids = array_unique(array_merge($voucher->details->pluck('item_id')->all(), $old_item_ids));

        $this->updateItemRate($item_ids);

        \DB::commit();

        return $voucher;
    }

    /**
     * Is voucher deletable
     *
     * @param Voucher $voucher
     */
    private function isDeletable(Voucher $voucher) : void
    {
        if ($this->requiresActionableStatus()) {
            return;
        }

        if ($voucher->status != VoucherStatus::PUBLISHED) {
            return;
        }

        if ($voucher->ledger->type->code == LedgerGroup::CASH) {
            return;
        }

        if ($voucher->payment_status != PaymentStatus::UNPAID) {
            throw ValidationException::withMessages(['message' => trans('voucher.could_not_delete_if_paid')]);
        }
    }

    /**
     * Delete voucher
     * @param Voucher $voucher
     */
    public function delete(Voucher $voucher) : void
    {
        $this->isDeletable($voucher);

        $old_item_ids = $voucher->details->pluck('item_id')->all();

        \DB::beginTransaction();

        $item_ids = $voucher->details->pluck('item_id')->all();

        $this->reverseLedgerBalance($voucher);

        $this->reverseItemQuantity($voucher);

        $voucher->delete();

        $this->updateItemRate($item_ids);

        \DB::commit();
    }
}
