<?php
namespace App\Repositories;

use App\Helpers\ArrHelper;
use App\Helpers\CalHelper;
use App\Http\Resources\ContactCollection;
use App\Models\Contact;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class ContactRepository
{
    protected $contact;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Contact $contact
    ) {
        $this->contact = $contact;
    }

    /**
     * Find contact with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Contact
    {
        $contact = $this->contact->find($id);

        if (! $contact) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('contact.contact')])]);
        }

        return $contact;
    }

    /**
     * Find contact with given uuid or throw an error
     * @param uuid $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Contact
    {
        $contact = $this->contact->filterByUuid($uuid)->first();

        if (! $contact) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('contact.contact')])]);
        }

        return $contact;
    }

    /**
     * Get all filtered data
     */
    public function getData($params = array())
    {
        $sort_by      = $this->contact->getSortBy();
        $order        = $this->contact->getOrder();

        $name = request()->query('name');
        $gender = request()->query('gender');
        $phone = request()->query('phone');
        $email = request()->query('email');
        $address = request()->query('address');

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        return $this->contact->when($name, function($q, $name) {
            $q->where(function($q1) use ($name) {
                $q1->where('name', 'like', '%'.$name.'%')
                ->orWhere('alias', 'like', '%'.$name.'%');
            });
        })->when($gender, function($q, $gender) {
            $q->where('gender', $gender);
        })->when($phone, function($q, $phone) {
            $q->where(function($q1) use ($phone) {
                $q1->where('phone', 'like', '%'.$phone.'%')
                ->orWhere('mobile', 'like', '%'.$phone.'%')
                ->orWhere('alt_mobile', 'like', '%'.$phone.'%');
            });
        })->when($email, function($q, $email) {
            $q->where(function($q1) use ($email) {
                $q1->where('email', 'like', '%'.$email.'%')
                ->orWhere('alt_email', 'like', '%'.$email.'%');
            });
        })->when($address, function($q, $address) {
            $q->where(function($q1) use ($address) {
                $q1->where(\DB::raw('lower(address->"$.address_line")'), "LIKE", "%".strtolower($address)."%")
                ->orWhere(\DB::raw('lower(address->"$.address_line2")'), "LIKE", "%".strtolower($address)."%")
                ->orWhere(\DB::raw('lower(address->"$.city")'), "LIKE", "%".strtolower($address)."%")
                ->orWhere(\DB::raw('lower(address->"$.state")'), "LIKE", "%".strtolower($address)."%")
                ->orWhere(\DB::raw('lower(address->"$.zipcode")'), "LIKE", "%".strtolower($address)."%")
                ->orWhere(\DB::raw('lower(address->"$.country")'), "LIKE", "%".strtolower($address)."%");
            });
        })->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        })->orderBy($sort_by, $order);
    }

    /**
     * Paginate all contacts
     */
    public function paginate() : ContactCollection
    {
        $per_page     = $this->contact->getPerPage();
        $current_page = $this->contact->getCurrentPage();

        return new ContactCollection($this->getData()->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get pre requisite.
     *
     * @return Array
     */
    public function getPreRequisite()
    {
        $genders = ArrHelper::getTransList('genders');

        return compact('genders');
    }

    /**
     * Search contact
     */
    public function search() : ContactCollection
    {
        $query = $this->contact->filterByNameOrAlias(request()->query('q'));

        return new ContactCollection($query->orderBy('name', 'asc')->take(10)->get());
    }

    /**
     * Create a new contact
     */
    public function create() : Contact
    {
        $contact = $this->contact->forceCreate($this->formatParams());

        $this->updateMeta($contact);

        return $contact;
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $existing_contact = $this->validateDuplicate(array(
            'mobile' => request('mobile'),
            'email' => request('email'),
        ));

        if ($existing_contact) {
            throw ValidationException::withMessages(['message' => trans('global.duplicate_found', ['attribute' => trans('contact.contact')])]);
        }

        $formatted = [
            'name'             => request('name'),
            'alias'            => request('alias'),
            'designation'      => request('designation'),
            'phone'            => request('phone'),
            'mobile'           => request('mobile'),
            'gender'           => request('gender.uuid'),
            'alt_mobile'       => request('alt_mobile'),
            'email'            => request('email'),
            'alt_email'        => request('alt_email'),
            'birth_date'       => CalHelper::toDate(request('birth_date')),
            'anniversary_date' => CalHelper::toDate(request('anniversary_date')),
            'address' => array(
                'address_line1'    => request('address_line1'),
                'address_line2'    => request('address_line2'),
                'city'             => request('city'),
                'state'            => request('state'),
                'zipcode'          => request('zipcode'),
                'country'          => request('country')
            )
        ];

        return $formatted;
    }

    private function validateDuplicate($params) : ?Contact
    {
        $query = $this->contact->whereMobile(Arr::get($params, 'mobile'));

        if (Arr::get($params, 'email')) {
            $query->orWhere('email', Arr::get($params, 'email'));
        }

        return $query->first();
    }

    /**
     * Add contact if not exists
     *
     * @param array $params
     */
    public function addIfNotExists($params = array()) : void
    {
        $contact = $this->validateDuplicate($params);

        if ($contact) {
            return;
        }

        $this->contact->forceCreate($params);
    }

    /**
     * Update contact meta
     *
     * @param Contact $contact
     */
    private function updateMeta(Contact $contact)
    {
        $is_business = (bool) request('is_business', false);

        $meta = $contact->meta;
        $meta['is_business'] = $is_business;

        $contact->meta = $meta;
        $contact->save();
    }

    /**
     * Update given contact
     * @param Contact $contact
     */
    public function update(Contact $contact) : Contact
    {
        if ($contact->is_primary && $contact->name != request('name')) {
            throw ValidationException::withMessages(['message' => trans('global.cannot_modify_default', ['attribute' => trans('contact.props.name')])]);
        }

        $contact->forceFill($this->formatParams($contact->uuid))->save();

        $this->updateMeta($contact);

        return $contact;
    }

    /**
     * Delete contact
     * @param Contact $contact
     */
    public function delete(Contact $contact) : void
    {
        if ($contact->is_primary) {
            throw ValidationException::withMessages(['message' => trans('global.cannot_delete_default', ['attribute' => trans('contact.contact')])]);
        }

        if ($contact->vouchers->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('contact.contact'), 'dependency' => trans('voucher.voucher')])]);
        }

        $contact->delete();
    }
}
