<?php

namespace App\Repositories;

use App\Enums\VoucherStatus;
use App\Enums\VoucherType;
use App\Helpers\CalHelper;
use App\Models\Accounting\Voucher;
use Carbon\Carbon;
use Illuminate\Validation\ValidationException;

class DashboardRepository
{
    protected $voucher;

    public function __construct(
        Voucher $voucher
    ) {
        $this->voucher = $voucher;
    }

    private function getDuration()
    {
        $start_date = CalHelper::getDate(request('start_date', today()->subDays(7)->toDateString()));
        $end_date = CalHelper::getDate(request('end_date', today()->toDateString()));

        return compact('start_date', 'end_date');
    }

    /**
     * Get stats for dashboard
     */
    public function getStats() : array
    {
        $vouchers = $this->voucher->whereIn('type', [VoucherType::SALE, VoucherType::PURCHASE])->whereStatus(VoucherStatus::PUBLISHED)->dateBetween('date', $this->getDuration())->groupBy('type')->select(['type', \DB::raw("SUM(total) as total"), \DB::raw("SUM(total-paid) as balance")])->get();

        $sales = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'sale'))->total : 0;
        $due_receipts = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'sale'))->balance : 0;
        $purchases = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'purchase'))->total : 0;
        $due_payments = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'purchase'))->balance : 0;

        $vouchers = $this->voucher->whereIn('type', [VoucherType::SALE, VoucherType::PURCHASE])->whereStatus(VoucherStatus::PUBLISHED)->where('date', '=', today())->groupBy('type')->select(['type', \DB::raw("SUM(total) as total"), \DB::raw("SUM(total-paid) as balance")])->get();

        $today_sales = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'sale'))->total : 0;
        $today_due_receipts = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'sale'))->balance : 0;
        $today_purchases = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'purchase'))->total : 0;
        $today_due_payments = (float) $vouchers->count() ? optional($vouchers->firstWhere('type', 'purchase'))->balance : 0;

        $today_sales_array = array(
            'value' => $today_sales,
            'label' => trans('dashboard.stats.today', ['attribute' => trans('sale.sales')]),
            'color' => $today_sales > 0 ? 'text-success' : ($today_sales < 0 ? 'text-danger' : 'text-dark'),
            'icon' => $today_sales > 0 ? 'fas fa-arrow-up' : 'fas fa-arrow-down'
        );

        $today_due_receipts_array = array(
            'value' => $today_due_receipts,
            'label' => trans('dashboard.stats.today', ['attribute' => trans('sale.invoice.due_receipts')]),
            'color' => $today_due_receipts > 0 ? 'text-success' : ($today_due_receipts < 0 ? 'text-danger' : 'text-dark'),
            'icon' => $today_due_receipts > 0 ? 'fas fa-arrow-up' : 'fas fa-arrow-down'
        );

        $today_purchases_array = array(
            'value' => $today_purchases,
            'label' => trans('dashboard.stats.today', ['attribute' => trans('purchase.purchases')]),
            'color' => $today_purchases > 0 ? 'text-success' : ($today_purchases < 0 ? 'text-danger' : 'text-dark'),
            'icon' => $today_purchases > 0 ? 'fas fa-arrow-up' : 'fas fa-arrow-down'
        );

        $today_due_payments_array = array(
            'value' => $today_due_payments,
            'label' => trans('dashboard.stats.today', ['attribute' => trans('purchase.bill.due_payments')]),
            'color' => $today_due_payments > 0 ? 'text-success' : ($today_due_payments < 0 ? 'text-danger' : 'text-dark'),
            'icon' => $today_due_payments > 0 ? 'fas fa-arrow-up' : 'fas fa-arrow-down'
        );

        $stats[] = array(
            'value' => $sales,
            'label' => trans('dashboard.stats.total', ['attribute' => trans('sale.sales')]),
            'type' => 'currency',
            'color' => 'bg-success',
            'icon' => 'fas fa-shopping-cart',
            'extra' => array(
                'today' => $today_sales_array
            )
        );

        $stats[] = array(
            'value' => $due_receipts,
            'label' => trans('dashboard.stats.total', ['attribute' => trans('sale.invoice.due_receipts')]),
            'type' => 'currency',
            'color' => 'bg-danger',
            'icon' => 'fas fa-arrow-circle-down',
            'extra' => array(
                'today' => $today_due_receipts_array
            )
        );

        $stats[] = array(
            'value' => $purchases,
            'label' => trans('dashboard.stats.total', ['attribute' => trans('purchase.purchases')]),
            'type' => 'currency',
            'color' => 'bg-info',
            'icon' => 'fas fa-boxes',
            'extra' => array(
                'today' => $today_purchases_array
            )
        );

        $stats[] = array(
            'value' => $due_payments,
            'label' => trans('dashboard.stats.total', ['attribute' => trans('purchase.bill.due_payments')]),
            'type' => 'currency',
            'color' => 'bg-warning',
            'icon' => 'fas fa-arrow-circle-up',
            'extra' => array(
                'today' => $today_due_payments_array
            )
        );

        return $stats;
    }

    /**
     * Get chart for dashboard
     */
    public function getCharts()
    {
        $type = request()->query('type', 'bar');

        $frequency = request()->query('frequency', 'monthly');

        if (! in_array($frequency, ['monthly', 'weekly', 'daily'])) {
            throw ValidationException::withMessages(['message' => trans('general.invalid_input')]);
        }

        $labels = array();

        if ($frequency == 'monthly') {
            $last_date = today()->lastOfMonth();
            $date = Carbon::parse($last_date)->subMonth(11);
        } else if ($frequency == 'weekly') {
            $last_date = today()->startOfWeek()->addWeek(1);
            $date = Carbon::parse($last_date)->subWeeks(12);
        } else {
            $last_date = today();
            $date = Carbon::parse($last_date)->subDays(12);
            $type = 'line';
        }

        $sale_dataset = array();
        $purchase_dataset = array();

        while($date < $last_date) {

            if ($frequency == 'monthly') {

                $labels[] = Carbon::parse($date)->format('F');

                $start_date = Carbon::parse($date)->firstOfMonth()->toDateString();
                $end_date = Carbon::parse($date)->lastOfMonth()->toDateString();

                $date = Carbon::parse($date)->addMonth(1);

            } else if ($frequency == 'weekly') {

                $start_date = Carbon::parse($date)->startOfWeek()->toDateString();
                $end_date = Carbon::parse($date)->endOfWeek()->toDateString();

                $labels[] = Carbon::parse($start_date)->format('d M') . ' - ' . Carbon::parse($end_date)->format('d M');

                $date = Carbon::parse($date)->addWeek(1);

            } else {

                $labels[] = Carbon::parse($date)->format('d M');

                $start_date = Carbon::parse($date)->toDateString();
                $end_date = Carbon::parse($date)->toDateString();

                $date = Carbon::parse($date)->addDay(1);

            }


            $sale_dataset[] = $this->voucher->whereIn('type', [VoucherType::SALE])->whereStatus(VoucherStatus::PUBLISHED)->dateBetween('date', compact('start_date', 'end_date'))->sum('total');

            $purchase_dataset[] = $this->voucher->whereIn('type', [VoucherType::PURCHASE])->whereStatus(VoucherStatus::PUBLISHED)->dateBetween('date', compact('start_date', 'end_date'))->sum('total');
        }

        return array(
            'type' => $type,
            'title' => trans(
            'dashboard.stats.' . $frequency , ['attribute' => trans('purchase.purchases') . ' & ' . trans('sale.sales') . ' ' . trans('dashboard.stats.report')]),

            'chart_data' => array(
                'labels' => $labels,
                'datasets' => array(
                    array(
                        'label' => trans('purchase.purchases'),
                        'backgroundColor' => 'rgba(23,205,239,0.5)',
                        'borderColor' => '#17cdef',
                        'borderWidth' => 1,
                        'data' => $purchase_dataset,
                    ),
                    array(
                        'label' => trans('sale.sales'),
                        'backgroundColor' => 'rgba(46,206,137,0.5)',
                        'borderColor' => '#2ece89',
                        'borderWidth' => 1,
                        'data' => $sale_dataset,
                    )
                )
            ),
            'actions' => array(
                'multiple' => false,
                'options' => [
                    array(
                        'label' => trans('list.general.frequencies.monthly'),
                        'param' => 'frequency',
                        'value' => 'monthly',
                        'active' => $frequency === 'monthly',
                    ),
                    array(
                        'label' => trans('list.general.frequencies.weekly'),
                        'param' => 'frequency',
                        'value' => 'weekly',
                        'active' => $frequency === 'weekly',
                    ),
                    array(
                        'label' => trans('list.general.frequencies.daily'),
                        'param' => 'frequency',
                        'value' => 'daily',
                        'active' => $frequency === 'daily',
                    ),
                ],
            ),
        );
    }
}
