<?php

namespace App\Repositories\Inventory;

use App\Http\Resources\Inventory\BrandCollection;
use App\Models\Inventory\Brand;
use Illuminate\Validation\ValidationException;

class BrandRepository
{
    protected $brand;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Brand $brand
    ) {
        $this->brand = $brand;
    }

    /**
     * Get all brands
     */
    public function getAll() : BrandCollection
    {
        return new BrandCollection($this->brand->all());
    }

    /**
     * Find brand with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Brand
    {
        $brand = $this->brand->find($id);

        if (! $brand) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.brand.brand')])]);
        }

        return $brand;
    }

    /**
     * Find brand with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Brand
    {
        $brand = $this->brand->filterByUuid($uuid)->first();

        if (! $brand) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.brand.brand')])]);
        }

        return $brand;
    }

    /**
     * Paginate all brands
     */
    public function paginate() : BrandCollection
    {
        $sort_by      = $this->brand->getSortBy();
        $order        = $this->brand->getOrder('asc');
        $per_page     = $this->brand->getPerPage();
        $current_page = $this->brand->getCurrentPage();

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        $name = request()->query('name');

        $query = $this->brand->search($name)->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        });

        return new BrandCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get brand pre requisite
     */
    public function getPreRequisite() : array
    {
        return [];
    }

    /**
     * Create a new brand
     */
    public function create() : Brand
    {
        return $this->brand->forceCreate($this->formatParams());
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $formatted = [
            'name'        => request('name'),
            'description' => request('description'),
        ];

        return $formatted;
    }

    /**
     * Update given brand
     * @param Brand $brand
     */
    public function update(Brand $brand) : Brand
    {
        $brand->forceFill($this->formatParams($brand->uuid))->save();

        return $brand;
    }

    /**
     * Delete brand
     * @param Brand $brand
     */
    public function delete(Brand $brand) : void
    {
        if ($brand->items->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('inventory.brand.brand'), 'dependency' => trans('inventory.item.item')])]);
        }

        $brand->delete();
    }
}
