<?php

namespace App\Repositories\Inventory;

use App\Enums\ItemCategoryType;
use App\Helpers\ArrHelper;
use App\Http\Resources\Inventory\ItemCategoryCollection;
use App\Models\Accounting\VoucherDetail;
use App\Models\Inventory\ItemCategory;
use App\Repositories\Setting\MarginRepository;
use App\Repositories\Setting\TaxRepository;
use App\Repositories\Setting\UnitRepository;
use Illuminate\Support\Arr;
use Illuminate\Validation\ValidationException;

class ItemCategoryRepository
{
    protected $item_category;
    protected $unit;
    protected $tax;
    protected $margin;
    protected $voucher_detail;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        ItemCategory $item_category,
        UnitRepository $unit,
        TaxRepository $tax,
        MarginRepository $margin,
        VoucherDetail $voucher_detail
    ) {
        $this->item_category = $item_category;
        $this->unit = $unit;
        $this->tax = $tax;
        $this->margin = $margin;
        $this->voucher_detail = $voucher_detail;
    }

    /**
     * Get all item categories
     */
    public function getAll() : ItemCategoryCollection
    {
        return new ItemCategoryCollection($this->item_category->all());
    }

    /**
     * Find item category with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : ItemCategory
    {
        $item_category = $this->item_category->find($id);

        if (! $item_category) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.item_category.item_category')])]);
        }

        return $item_category;
    }

    /**
     * Find item category with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : ItemCategory
    {
        $item_category = $this->item_category->whereUuid($uuid)->first();

        if (! $item_category) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.item_category.item_category')])]);
        }

        return $item_category;
    }

    /**
     * Paginate all item categories
     */
    public function paginate() : ItemCategoryCollection
    {
        $sort_by      = $this->item_category->getSortBy();
        $order        = $this->item_category->getOrder('asc');
        $per_page     = $this->item_category->getPerPage();
        $current_page = $this->item_category->getCurrentPage();

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        $name               = request()->query('name');
        $keywords           = request()->query('keywords');
        $type               = request()->query('type');
        $item_category_uuid = request()->query('parent');
        $unit_uuid          = request()->query('unit');
        $tax_uuid           = request()->query('tax');
        $margin_uuid        = request()->query('margin');

        $query = $this->item_category->when($name, function($q, $name) {
            $q->where('name', 'like', '%'.$name.'%');
        })->when($keywords, function($q, $keywords) {
            $q->search($keywords);
        })->when($type, function($q, $type) {
            $q->where('type', $type);
        })->when($item_category_uuid, function($q, $item_category_uuid) {
            $q->whereHas('parent', function($q1) use($item_category_uuid) {
                $q1->whereUuid($item_category_uuid);
            });
        })->when($unit_uuid, function($q, $unit_uuid) {
            $q->whereHas('unit', function($q1) use($unit_uuid) {
                $q1->whereUuid($unit_uuid);
            });
        })->when($margin_uuid, function($q, $margin_uuid) {
            $q->whereHas('margin', function($q1) use($margin_uuid) {
                $q1->whereUuid($margin_uuid);
            });
        })->when($tax_uuid, function($q, $tax_uuid) {
            $q->whereHas('tax', function($q1) use($tax_uuid) {
                $q1->whereUuid($tax_uuid);
            });
        })->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        });

        if ($sort_by == 'stock_match_date') {
            $sort_by = 'last_stock_match_date';
        }

        return new ItemCategoryCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get item category pre requisite
     */
    public function getPreRequisite() : array
    {
        $units = $this->unit->getAll();
        $taxes = $this->tax->getAll();
        $margins = $this->margin->getAll();
        $types = ArrHelper::getTransList('types', 'inventory');
        $variant_types = ArrHelper::getTransList('variant_types', 'inventory');
        $price_calculation_methods = ArrHelper::getTransList('price_calculation_methods', 'inventory');

        return compact('units', 'taxes', 'margins', 'types', 'price_calculation_methods', 'variant_types');
    }

    /**
     * Search item category
     */
    public function search() : ItemCategoryCollection
    {
        $query = $this->item_category->filterByNameOrAlias(request()->query('q'));

        return new ItemCategoryCollection($query->orderBy('name', 'asc')->take(10)->get());
    }

    /**
     * Create a new item category
     */
    public function create() : ItemCategory
    {
        return $this->item_category->forceCreate($this->formatParams());
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $this->validateInput();

        $unit   = request('unit.uuid') ? $this->unit->findByUuidOrFail(request('unit.uuid'), 'unit') : null;
        $tax    = request('tax.uuid') ? $this->tax->findByUuidOrFail(request('tax.uuid'), 'tax') : null;
        $margin = request('margin.uuid') ? $this->margin->findByUuidOrFail(request('margin.uuid'), 'margin') : null;
        $parent = request('parent.uuid') ? $this->findByUuidOrFail(request('parent.uuid'), 'parent') : null;

        $variants = is_array(request('variants')) ? request('variants') : [];
        $variants = collect($variants)->transform(function($variant) {
            $options = Arr::get($variant, 'options', []);
            $type = Arr::get($variant, 'type.uuid', 'text');
            $options = collect($options)->transform(function($option) {
                return array('name' => Arr::get($option, 'name'));
            })->all();
            return array('name' => Arr::get($variant, 'name'), 'options' => $options, 'type' => $type);
        });

        $formatted = [
            'name'                     => request('name'),
            'alias'                    => request('alias'),
            'description'              => request('description'),
            'type'                     => request('type.uuid'),
            'reorder_level'            => request('reorder_level', 0),
            'price_calculation_method' => request('price_calculation_method.uuid'),
            'unit_id'                  => optional($unit)->id,
            'tax_id'                   => optional($tax)->id,
            'margin_id'                => optional($margin)->id,
            'parent_id'                => optional($parent)->id,
            'variants'                 => $variants
        ];

        if (! $uuid) {
            $formatted['meta'] = array('prefix' => request('prefix'));
        }

        return $formatted;
    }

    /**
     * Validate input
     */
    private function validateInput() : void
    {
        $type = request('type.uuid');

        if (! in_array($type, ArrHelper::getList('types', 'inventory'))) {
            throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('inventory.item_category.props.type')])]);
        }

        $price_calculation_method = request('price_calculation_method.uuid');

        if (! in_array($price_calculation_method, ArrHelper::getList('price_calculation_methods', 'inventory'))) {
            throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('inventory.item_category.props.price_calculation_method')])]);
        }

        $variants = is_array(request('variants')) ? request('variants') : [];
        foreach ($variants as $variant) {
            $type = Arr::get($variant, 'type.uuid', 'text');

            if (! in_array($type, ArrHelper::getList('variant_types', 'inventory'))) {
                throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('inventory.item_category.props.variant_type')])]);
            }

            $options = Arr::get($variant, 'options', []);

            if (in_array($type, ['single_select', 'multi_select'])) {
                if (! $options || ! count($options)) {
                    throw ValidationException::withMessages(['message' => trans('validation.required', ['attribute' => trans('inventory.item_category.props.variant_option')])]);
                }

                $names = array();
                foreach ($options as $option) {
                    $name = Arr::get($option, 'name');

                    if (! $name) {
                        throw ValidationException::withMessages(['message' => trans('validation.required', ['attribute' => trans('inventory.item_category.props.variant_option')])]);
                    }

                    $names[] = $name;
                }

                if (count($names) > count(array_unique($names))) {
                    throw ValidationException::withMessages(['message' => trans('validation.distinct', ['attribute' => trans('inventory.item_category.props.variant_option')])]);
                }
            }
        }
    }

    /**
     * Update given item category
     * @param ItemCategory $item_category
     */
    public function update(ItemCategory $item_category) : ItemCategory
    {
        $item_ids = $item_category->items()->pluck('id')->all();
        $voucher_exists = $this->voucher_detail->whereIn('item_id', $item_ids)->count();

        if ($voucher_exists && $item_category->type != request('type.uuid')) {
            throw ValidationException::withMessages(['message' => trans('inventory.could_not_change_type_if_voucher_exists')]);
        }

        $item_category->forceFill($this->formatParams($item_category->uuid))->save();

        $meta = $item_category->meta;
        $meta['prefix'] = request('prefix');
        $item_category->meta = $meta;
        $item_category->save();

        return $item_category;
    }

    /**
     * Delete item category
     * @param ItemCategory $item_category
     */
    public function delete(ItemCategory $item_category) : void
    {
        if ($item_category->items->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('inventory.item_category.item_categories'), 'dependency' => trans('inventory.item.item')])]);
        }

        $item_category->delete();
    }
}
