<?php
namespace App\Repositories\Inventory;

use App\Enums\VoucherStatus;
use App\Enums\VoucherType;
use App\Helpers\CalHelper;
use App\Http\Resources\Accounting\LedgerSummary;
use App\Models\Accounting\VoucherDetail;
use App\Models\Inventory\Item;
use App\Traits\CollectionPaginator;
use Illuminate\Support\Arr;

class ItemReportRepository
{
    use CollectionPaginator;

    protected $item;
    protected $voucher_detail;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Item $item,
        VoucherDetail $voucher_detail
    ) {
        $this->item = $item;
        $this->voucher_detail = $voucher_detail;
    }

    private function getDuration()
    {
        $start_date = CalHelper::getDate(request('start_date'), today()->subDays(7)->toDateString());
        $end_date = CalHelper::getDate(request('end_date'), today()->toDateString());

        return compact('start_date', 'end_date');
    }

    /**
     * Get report
     *
     * @return array
     */
    public function getReport(Item $item)
    {
        $per_page     = $this->item->getPerPage();
        $current_page = $this->item->getCurrentPage();
        // $order        = $this->item->getOrder('asc');
        $order        = 'asc';

        $desc = $order == 'asc' ? false : true;

        $duration = $this->getDuration();

        $is_non_taxed = request()->boolean('is_non_taxed');

        $opening_quantity = $is_non_taxed ? $item->opening_quantity_non_taxed : $item->opening_quantity_taxed;

        $in_quantity = $this->voucher_detail->where('item_id', $item->id)->whereHas('voucher', function($q) use ($duration, $is_non_taxed, $desc) {
            $q->where('status', VoucherStatus::PUBLISHED)
            ->whereIn('type', [VoucherType::PURCHASE, VoucherType::SALE_RETURN])
            ->when($desc, function($q1) use ($duration) {
                $q1->where('date', '<=', Arr::get($duration, 'end_date'));
            })->when(! $desc, function($q1) use ($duration) {
                $q1->where('date', '<', Arr::get($duration, 'start_date'));
            })->when(! $is_non_taxed, function($q1) {
                $q1->where('is_non_taxed', 0);
            });
        })->where('is_non_taxed', $is_non_taxed)->sum('quantity');

        $out_quantity = $this->voucher_detail->where('item_id', $item->id)->whereHas('voucher', function($q) use ($duration, $is_non_taxed, $desc) {
            $q->where('status', VoucherStatus::PUBLISHED)
            ->whereIn('type', [VoucherType::SALE, VoucherType::PURCHASE_RETURN])
            ->when($desc, function($q1) use ($duration) {
                $q1->where('date', '<=', Arr::get($duration, 'end_date'));
            })->when(! $desc, function($q1) use ($duration) {
                $q1->where('date', '<', Arr::get($duration, 'start_date'));
            })->when(! $is_non_taxed, function($q1) {
                $q1->where('is_non_taxed', 0);
            });
        })->where('is_non_taxed', $is_non_taxed)->sum('quantity');

        $opening_quantity += ($in_quantity - $out_quantity);
        $quantity = $opening_quantity;

        $voucher_details = $this->voucher_detail->where('item_id', $item->id)->with('voucher', 'voucher.ledger')->whereHas('voucher', function($q) use ($duration, $is_non_taxed) {
            $q->where('status', VoucherStatus::PUBLISHED)
            ->whereIn('type', [VoucherType::PURCHASE, VoucherType::SALE, VoucherType::SALE_RETURN, VoucherType::PURCHASE_RETURN])
            ->dateBetween('date', $duration)
            ->where('is_non_taxed', $is_non_taxed);
        })->select('voucher_details.*', \DB::raw('(SELECT date FROM vouchers WHERE voucher_details.voucher_id = vouchers.id ) as sort_by'))
            ->orderBy('sort_by', $order)
            ->cursor();

        $data = array();
        foreach ($voucher_details as $index => $voucher_detail) {

            if (in_array($voucher_detail->voucher->type, [VoucherType::PURCHASE, VoucherType::SALE_RETURN])) {
                $quantity += $voucher_detail->quantity;
            } else {
                $quantity -= $voucher_detail->quantity;
            }

            $data[] = array(
                'uuid' => $voucher_detail->uuid,
                'balance' => (float) $quantity,
                'code_number' => $voucher_detail->voucher->code_number,
                'date' => CalHelper::toDate($voucher_detail->voucher->date),
                'ledger' => new LedgerSummary($voucher_detail->voucher->ledger),
                'quantity' => (float) $voucher_detail->quantity,
                'type' => array(
                    'uuid' => $voucher_detail->voucher->type,
                    'name' => trans($voucher_detail->voucher->type.'.'.$voucher_detail->voucher->type)
                ),
                'voucher_uuid' => $voucher_detail->voucher->uuid
            );
        }

        return $this->collectionPaginate($data, $per_page, $current_page);
    }
}
