<?php

namespace App\Repositories\Inventory;

use App\Enums\ItemCategoryType;
use App\Helpers\ArrHelper;
use App\Http\Resources\Inventory\ItemCollection;
use App\Models\Accounting\VoucherDetail;
use App\Models\Inventory\Item;
use App\Repositories\Setting\MarginRepository;
use App\Repositories\Setting\TaxRepository;
use App\Repositories\Setting\UnitRepository;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Arr;
use Illuminate\Validation\ValidationException;

class ItemRepository
{
    protected $item;
    protected $unit;
    protected $tax;
    protected $margin;
    protected $brand;
    protected $item_category;
    protected $voucher_detail;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Item $item,
        UnitRepository $unit,
        TaxRepository $tax,
        BrandRepository $brand,
        MarginRepository $margin,
        ItemCategoryRepository $item_category,
        VoucherDetail $voucher_detail
    ) {
        $this->item = $item;
        $this->unit = $unit;
        $this->tax = $tax;
        $this->brand = $brand;
        $this->margin = $margin;
        $this->item_category = $item_category;
        $this->voucher_detail = $voucher_detail;
    }

    /**
     * Get all items
     */
    public function getAll($params = array()) : ItemCollection
    {
        $item_category_uuid = request()->query('item_category');

        return new ItemCollection($this->item->when($item_category_uuid, function($q, $item_category_uuid) {
            $q->whereHas('category', function($q1) use($item_category_uuid) {
                $q1->whereUuid($item_category_uuid);
            })->orWhereHas('category', function($q2) use($item_category_uuid) {
                $q2->whereHas('parent', function($q3) use($item_category_uuid) {
                    $q3->whereUuid($item_category_uuid);
                });
            });
        })->get());
    }

    /**
     * Find by item ids
     */
    public function findByIds($params = array()) : Collection
    {
        $item_ids = Arr::get($params, 'item_ids', []);

        return $this->item->when($item_ids, function($q, $item_ids) {
            $q->whereIn('id', $item_ids);
        })->get();
    }

    /**
     * Find item with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Item
    {
        $item = $this->item->find($id);

        if (! $item) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.item.item')])]);
        }

        return $item;
    }

    /**
     * Find item with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Item
    {
        $item = $this->item->filterByUuid($uuid)->first();

        if (! $item) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.item.item')])]);
        }

        return $item;
    }

    /**
     * Paginate all items
     */
    public function paginate() : ItemCollection
    {
        $sort_by      = $this->item->getSortBy();
        $order        = $this->item->getOrder();
        $per_page     = $this->item->getPerPage();
        $current_page = $this->item->getCurrentPage();

        $description        = request()->query('description');
        $name               = request()->query('name');
        $model_number       = request()->query('model_number');
        $item_category_uuid = request()->query('item_category');
        $brand_uuid         = request()->query('brand');
        $unit_uuid          = request()->query('unit');
        $tax_uuid           = request()->query('tax');
        $margin_uuid        = request()->query('margin');
        $requires_reorder   = request()->boolean('requires_reorder');

        $tags = request()->query('tags');
        $tags = is_array($tags) ? $tags : ($tags ? explode(',', $tags) : []);

        $query = $this->item->when($name, function($q, $name) {
            $q->where(function($q1) use ($name) {
                $q1->where('name', 'like', '%'.$name.'%')
                ->orWhere('alias', 'like', '%'.$name.'%');
            });
        })->when($model_number, function($q, $model_number) {
            $q->where(\DB::raw('lower(identifier->"$.model_number")'), "LIKE", "%".strtolower($model_number)."%");
        })->when($description, function($q, $description) {
            $q->where('description', 'like', '%'.$description.'%');
        })->when($brand_uuid, function($q, $brand_uuid) {
            $q->whereHas('brand', function($q1) use($brand_uuid) {
                $q1->whereUuid($brand_uuid);
            });
        })->when($item_category_uuid, function($q, $item_category_uuid) {
            $q->whereHas('category', function($q1) use($item_category_uuid) {
                $q1->whereUuid($item_category_uuid);
            })->orWhereHas('category', function($q2) use($item_category_uuid) {
                $q2->whereHas('parent', function($q3) use($item_category_uuid) {
                    $q3->whereUuid($item_category_uuid);
                });
            });
        })->when($unit_uuid, function($q, $unit_uuid) {
            $q->whereHas('unit', function($q1) use($unit_uuid) {
                $q1->whereUuid($unit_uuid);
            })->orWhereHas('category', function($q1) use($unit_uuid) {
                $q1->whereHas('unit', function($q2) use($unit_uuid) {
                    $q2->whereUuid($unit_uuid);
                });
            });
        })->when($margin_uuid, function($q, $margin_uuid) {
            $q->whereHas('margin', function($q1) use($margin_uuid) {
                $q1->whereUuid($margin_uuid);
            })->orWhereHas('category', function($q1) use($margin_uuid) {
                $q1->whereHas('margin', function($q2) use($margin_uuid) {
                    $q2->whereUuid($margin_uuid);
                });
            });
        })->when($tax_uuid, function($q, $tax_uuid) {
            $q->whereHas('tax', function($q1) use($tax_uuid) {
                $q1->whereUuid($tax_uuid);
            })->orWhereHas('category', function($q1) use($tax_uuid) {
                $q1->whereHas('tax', function($q2) use($tax_uuid) {
                    $q2->whereUuid($tax_uuid);
                });
            });
        })->when($tags, function($q, $tags) {
            $q->whereHas('tags', function($q1) use ($tags) {
                $q1->whereIn('slug', $tags);
            });
        });

        $query->select('items.*', \DB::raw('json_extract(opening, "$.quantity_taxed") + json_extract(current, "$.quantity_taxed") as quantity_taxed'), \DB::raw('json_extract(opening, "$.quantity_non_taxed") + json_extract(current, "$.quantity_non_taxed") as quantity_non_taxed'));

        if (request()->has('paginated') && ! request()->boolean('paginated')) {
            return new ItemCollection($query->get());
        }

        return new ItemCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get item pre requisite
     */
    public function getPreRequisite() : array
    {
        $units = $this->unit->getAll();
        $taxes = $this->tax->getAll();
        $margins = $this->margin->getAll();
        $brands = $this->brand->getAll();
        $price_calculation_methods = ArrHelper::getTransList('price_calculation_methods', 'inventory');

        return compact('units', 'taxes', 'margins', 'brands', 'price_calculation_methods');
    }

    /**
     * Search item
     */
    public function search() : ItemCollection
    {
        $query = $this->item->filterByNameOrAlias(request()->query('q'));

        return new ItemCollection($query->orderBy('name', 'asc')->take(10)->get());
    }

    /**
     * Create a new item
     */
    public function create() : Item
    {
        return $this->item->forceCreate($this->formatParams());
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $this->validateInput();

        $unit          = request('unit.uuid') ? $this->unit->findByUuidOrFail(request('unit.uuid'), 'unit') : null;
        $tax           = request('tax.uuid') ? $this->tax->findByUuidOrFail(request('tax.uuid'), 'tax') : null;
        $margin        = request('margin.uuid') ? $this->margin->findByUuidOrFail(request('margin.uuid'), 'margin') : null;
        $item_category = $this->item_category->findByUuidOrFail(request('item_category.uuid'), 'item_category');

        $brand = null;
        if ($item_category->type == ItemCategoryType::PRODUCT) {
            request()->validate([
                'brand' => 'required'
            ]);

            $brand = $this->brand->findByUuidOrFail(request('brand.uuid'), 'brand');
        }

        $variants = is_array(request('variants')) ? request('variants') : [];
        $variants = collect($variants)->transform(function($variant) {
            $type = Arr::get($variant, 'type', 'text');
            $value = Arr::get($variant, 'value');
            $options = Arr::get($variant, 'options', []);
            $options = collect($options)->transform(function($option) {
                return array('name' => Arr::get($option, 'name'));
            })->all();
            return array('name' => Arr::get($variant, 'name'), 'options' => $options, 'type' => $type, 'value' => $value);
        });

        $formatted = [
            'name'                     => request('name'),
            'alias'                    => request('alias'),
            'description'              => request('description'),
            'unit_id'                  => ($unit && $unit->id != $item_category->unit_id) ? $unit->id : null,
            'tax_id'                   => ($tax && $tax->id != $item_category->tax_id) ? $tax->id : null,
            'margin_id'                => ($margin && $margin->id != $item_category->margin_id) ? $margin->id : null,
            'brand_id'                 => optional($brand)->id,
            'item_category_id'         => optional($item_category)->id,
            'reorder_level'            => request('reorder_level') ? : 0,
            'price_calculation_method' => request('price_calculation_method.uuid'),
            'opening'          => array(
                'quantity_taxed'     => request('opening_quantity_taxed', 0),
                'rate_taxed'         => request('opening_rate_taxed', 0),
                'quantity_non_taxed' => request('opening_quantity_non_taxed', 0),
                'rate_non_taxed'     => request('opening_rate_non_taxed', 0),
            ),
            'identifier' => array(
                'model_number' => request('identifier.model_number'),
                'product_number' => request('identifier.product_number'),
            ),
            'mrp' => request('mrp') ? : null,
            'warranty' => request('warranty'),
            'variants' => $variants,
        ];

        if (! $uuid) {
            $formatted['current'] = array(
                'quantity_taxed'     => 0,
                'rate_taxed'         => 0,
                'quantity_non_taxed' => 0,
                'rate_non_taxed'     => 0,
            );
        }

        return $formatted;
    }

    /**
     * Validate input
     */
    private function validateInput() : void
    {
        $price_calculation_method = request('price_calculation_method.uuid');

        if ($price_calculation_method && ! in_array($price_calculation_method, ArrHelper::getList('price_calculation_methods', 'inventory'))) {
            throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('inventory.item.props.price_calculation_method')])]);
        }
    }

    /**
     * Update given item
     * @param Item $item
     */
    public function update(Item $item) : Item
    {
        $voucher_exists = $this->voucher_detail->whereItemId($item->id)->count();

        if ($voucher_exists && $item->category->uuid != request('item_category.uuid')) {
            throw ValidationException::withMessages(['message' => trans('inventory.could_not_change_category_if_voucher_exists')]);
        }

        if ($voucher_exists && $item->unit_id && $item->unit->uuid != request('unit.uuid')) {
            throw ValidationException::withMessages(['message' => trans('inventory.could_not_change_unit_if_voucher_exists')]);
        }

        $item->forceFill($this->formatParams($item->uuid))->save();

        return $item;
    }

    /**
     * Delete item
     * @param Item $item
     */
    public function delete(Item $item) : void
    {
        if ($item->voucherDetails->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('inventory.item.item'), 'dependency' => trans('voucher.voucher')])]);
        }

        $item->delete();
    }
}
