<?php

namespace App\Repositories\Inventory;

use App\Helpers\CalHelper;
use App\Http\Resources\Inventory\StockMatchCollection;
use App\Models\Inventory\StockMatchDetail;
use App\Models\Inventory\StockMatch;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class StockMatchRepository
{
    protected $stock_match;
    protected $stock_match_detail;
    protected $item_category;
    protected $item;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        StockMatch $stock_match,
        StockMatchDetail $stock_match_detail,
        ItemCategoryRepository $item_category,
        ItemRepository $item
    ) {
        $this->stock_match = $stock_match;
        $this->stock_match_detail = $stock_match_detail;
        $this->item_category = $item_category;
        $this->item = $item;
    }

    /**
     * Get all stock matches
     */
    public function getAll() : StockMatchCollection
    {
        return new StockMatchCollection($this->stock_match->all());
    }

    /**
     * Find stock match with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : StockMatch
    {
        $stock_match = $this->stock_match->info()->find($id);

        if (! $stock_match) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.stock_match.stock_match')])]);
        }

        return $stock_match;
    }

    /**
     * Find stock match with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : StockMatch
    {
        $stock_match = $this->stock_match->info()->filterByUuid($uuid)->first();

        if (! $stock_match) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('inventory.stock_match.stock_match')])]);
        }

        return $stock_match;
    }

    /**
     * Paginate all stock matches
     */
    public function paginate() : StockMatchCollection
    {
        $sort_by = $this->stock_match->getSortBy();
        $order   = $this->stock_match->getOrder();

        $keywords           = request()->query('keywords');
        $item_category_uuid = request()->query('item_category');
        $start_date         = request()->query('start_date');
        $end_date           = request()->query('end_date');

        $query = $this->stock_match->info()->when($keywords, function($q, $keywords) {
            $q->where('description', 'like', '%'.$keywords.'%');
        })->when($item_category_uuid, function($q, $item_category_uuid) {
            $q->whereHas('itemCategory', function($q1) use($item_category_uuid) {
                $q1->whereUuid($item_category_uuid);
            });
        })->dateBetween([
            'start_date' => $start_date,
            'end_date'   => $end_date
        ]);

        $per_page     = $this->stock_match->getPerPage();
        $current_page = $this->stock_match->getCurrentPage();

        return new StockMatchCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Create a new stock match
     */
    public function create() : StockMatch
    {
        \DB::beginTransaction();

        $stock_match = $this->stock_match->forceCreate($this->formatParams());

        $this->updateDetails($stock_match);

        \DB::commit();

        return $stock_match;
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $item_category = $this->item_category->findByUuidOrFail(request('item_category.uuid'), 'item_category');

        $stock_match_exists = $this->stock_match->whereItemCategoryId($item_category->id)->when($uuid, function($q, $uuid) {
            $q->where('uuid', '!=', $uuid);
        })->where('date', '>=', request('date'))->first();

        if ($stock_match_exists) {
            throw ValidationException::withMessages(['message' => trans('inventory.stock_match.could_not_match_stock_before', ['attribute' => CalHelper::toDate($stock_match_exists->date)])]);
        }

        $formatted = [
            'date'             => request('date'),
            'item_category_id' => $item_category->id,
            'description'      => request('description'),
        ];

        if (! $uuid) {
            $formatted['user_id'] = \Auth::id();
            $formatted['meta'] = array();
        }

        return $formatted;
    }

    /**
     * Update stock match details
     *
     * @param StockMatch $stock_match
     * @param string $action
     */
    private function updateDetails(StockMatch $stock_match, $action = 'create') : void
    {
        $items = $this->item->getAll(array(
            'item_category' => $stock_match->itemCategory->uuid
        ));

        foreach(request('items', []) as $index => $data) {
            $item = Arr::get($data, 'item');

            $item = $items->firstWhere('uuid', Arr::get($item, 'uuid'));

            if (! $item) {
                throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('inventory.item.item')])]);
            }

            $stock_match_detail = $this->stock_match_detail->firstOrCreate([
                'stock_match_id' => $stock_match->id,
                'item_id' => $item->id
            ]);

            $stock_match_detail->uuid = $stock_match_detail->uuid ? : Str::uuid();

            if ($action == 'create') {
                $stock_match_detail->billed = array(
                    'quantity_taxed'     => $item->opening_quantity_taxed + $item->current_quantity_taxed,
                    'quantity_non_taxed' => $item->opening_quantity_non_taxed + $item->current_quantity_non_taxed,
                );
            }
            $stock_match_detail->actual = array(
                'quantity_taxed'     => request('items.'.$index.'.actual_quantity_taxed', 0),
                'quantity_non_taxed' => request('items.'.$index.'.actual_quantity_non_taxed', 0)
            );

            $stock_match_detail->description = request('items.'.$index.'.description');
            $stock_match_detail->save();
        }
    }

    /**
     * Update given stock match
     * @param StockMatch $stock_match
     */
    public function update(StockMatch $stock_match) : StockMatch
    {
        if ($stock_match->itemCategory->uuid != request('item_category.uuid')) {
            throw ValidationException::withMessages(['message' => trans('inventory.stock_match.could_not_edit_item_category')]);
        }

        \DB::beginTransaction();

        $stock_match->forceFill($this->formatParams($stock_match->uuid))->save();

        $this->updateDetails($stock_match, 'update');

        \DB::commit();

        return $stock_match;
    }

    /**
     * Delete stock match
     * @param StockMatch $stock_match
     */
    public function delete(StockMatch $stock_match) : void
    {
        $stock_match_exists = $this->stock_match->where('uuid', '!=', $stock_match->uuid)->where('date', '>', $stock_match->date)->first();

        if ($stock_match_exists) {
            throw ValidationException::withMessages(['message' => trans('inventory.stock_match.could_not_delete_stock_match_before', ['attribute' => CalHelper::toDate($stock_match_exists->date)])]);
        }

        $stock_match->delete();
    }
}
