<?php

namespace App\Repositories;

use App\Enums\TransactionType;
use App\Enums\VoucherStatus;
use App\Enums\VoucherType;
use App\Helpers\CalHelper;
use App\Helpers\SysHelper;
use App\Http\Resources\Accounting\LedgerSummary;
use App\Models\Accounting\Transaction;
use App\Models\Accounting\TransactionDetail;
use App\Models\Accounting\Voucher;
use App\Models\Accounting\VoucherTax;
use App\Repositories\Accounting\LedgerRepository;
use App\Traits\CollectionPaginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;

class ReportRepository
{
    use CollectionPaginator;

    protected $ledger;
    protected $voucher;
    protected $transaction;
    protected $transaction_detail;
    protected $voucher_tax;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        LedgerRepository $ledger,
        Voucher $voucher,
        Transaction $transaction,
        TransactionDetail $transaction_detail,
        VoucherTax $voucher_tax
    ) {
        $this->ledger = $ledger;
        $this->voucher = $voucher;
        $this->transaction = $transaction;
        $this->transaction_detail = $transaction_detail;
        $this->voucher_tax = $voucher_tax;
    }

    private function getDuration()
    {
        $start_date = CalHelper::getDate(request('start_date', today()->subDays(7)->toDateString()));
        $end_date = CalHelper::getDate(request('end_date', today()->toDateString()));

        return compact('start_date', 'end_date');
    }

    /**
     * Get daybook report
     *
     * @return array
     */
    public function daybook() : array
    {
        $per_page     = $this->voucher->getPerPage();
        $current_page = $this->voucher->getCurrentPage();

        $duration = $this->getDuration();
        $is_non_taxed = (bool) request()->boolean('is_non_taxed');

        $data = $this->getVouchers($duration, $is_non_taxed);

        $data = $this->getTransactions($duration, $is_non_taxed, $data);

        $data = $this->reorder($data);

        return $this->collectionPaginate($data, $per_page, $current_page);
    }

    /**
     * Get tax report
     *
     * @return array
     */
    public function tax() : array
    {
        $per_page     = $this->voucher->getPerPage();
        $current_page = $this->voucher->getCurrentPage();

        $duration = $this->getDuration();

        $data = array();

        // $voucher_taxes = $this->voucher_tax->whereHas('voucher', function($q) use ($duration) {
        //     $q->whereIn('type', [VoucherType::PURCHASE, VoucherType::SALE])
        //     ->whereStatus(VoucherStatus::PUBLISHED)
        //     ->whereIsCancelled(0)
        //     ->dateBetween($duration);
        // })->get();

        return $this->collectionPaginate($data, $per_page, $current_page);
    }


    /**
     * Get vouchers
     *
     * @param array $duration
     * @param bool $is_non_taxed
     * @param array $data
     */
    private function getVouchers($duration, $is_non_taxed, $data = array()) : array
    {
        $vouchers = $this->voucher->with('ledger')->whereIn('type', [VoucherType::SALE, VoucherType::PURCHASE])->whereStatus(VoucherStatus::PUBLISHED)->dateBetween('date', $duration)->whereIsNonTaxed($is_non_taxed)->orderBy('date', 'asc')->get();

        foreach($vouchers as $voucher) {
            $data[] = array(
                'uuid' => $voucher->uuid,
                'date' => CalHelper::toDate($voucher->date),
                'type' => array('uuid' => $voucher->type, 'name' => trans($voucher->type.'.'.$voucher->type)),
                'code_number' => $voucher->code_number,
                'amount' => SysHelper::formatCurrency($voucher->total),
                'sub_type' => null,
                'ledger' => null,
                'direction' => null,
                'second_ledger' => new LedgerSummary($voucher->ledger),
                'status' => array('uuid' => $voucher->payment_status, 'name' => trans($voucher->type.'.'.$voucher->payment_status)),
                'voucher_uuid' => $voucher->uuid
            );
        }

        return $data;
    }

    /**
     * Get transactions
     *
     * @param array $duration
     * @param bool $is_non_taxed
     * @param array $data
     */
    private function getTransactions($duration, $is_non_taxed, $data = array()) : array
    {
        $transactions = $this->transaction->with('ledger', 'details', 'details.ledger')->whereIsNonTaxed($is_non_taxed)->whereIsCancelled(0)->dateBetween($duration)->orderBy('date', 'asc')->get();

        foreach($transactions as $transaction) {
            foreach ($transaction->details as $transaction_detail) {
                $data[] = array(
                    'uuid' => $transaction_detail->uuid,
                    'date' => CalHelper::toDate($transaction->date),
                    'type' => array('uuid' => 'transaction', 'name' => trans('transaction.transaction')),
                    'code_number' => $transaction->code_number,
                    'amount' => SysHelper::formatCurrency($transaction_detail->amount),
                    'sub_type' => array('uuid' => $transaction->type, 'name' => trans('transaction.types.'.$transaction->type)),
                    'ledger' => new LedgerSummary($transaction->ledger),
                    'direction' => $this->getTransactionDirection($transaction),
                    'second_ledger' => new LedgerSummary($transaction_detail->ledger),
                    'status' => null,
                    'voucher_uuid' => $transaction->uuid
                );
            }
        }

        return $data;
    }

    /**
     * Reorder
     *
     * @param array $data
     */
    private function reorder($data = array()) : array
    {
        array_multisort(array_map(function($element) {
            return $element['date'];
        }, $data), SORT_ASC, $data);

        $rows = array();
        foreach ($data as $item)
        {
            $rows[] = array(
                'uuid'          => Arr::get($item, 'uuid'),
                'date'          => Arr::get($item, 'date'),
                'type'          => Arr::get($item, 'type'),
                'code_number'   => Arr::get($item, 'code_number'),
                'amount'        => Arr::get($item, 'amount'),
                'sub_type'      => Arr::get($item, 'sub_type'),
                'ledger'        => Arr::get($item, 'ledger'),
                'direction'     => Arr::get($item, 'direction'),
                'second_ledger' => Arr::get($item, 'second_ledger'),
                'status'        => Arr::get($item, 'status'),
                'voucher_uuid'  => Arr::get($item, 'voucher_uuid')
            );
        }

        return $rows;
    }

    /**
     * Get direction for voucher
     *
     * @param Voucher $voucher
     */
    private function getVoucherDirection(Voucher $voucher) : ?string
    {
        return null;
    }

    /**
     * Get direction for transaction
     *
     * @param Voucher $voucher
     */
    private function getTransactionDirection(Transaction $transaction) : string
    {
        if (in_array($transaction->type, [TransactionType::PAYMENT, TransactionType::CONTRA])) {
            return 'from';
        }

        return 'to';
    }
}
