<?php

namespace App\Repositories\Setting;

use App\Http\Resources\Setting\MarginCollection;
use App\Models\Setting\Margin;
use App\Models\Setting\MarginDetail;
use App\Repositories\Config\RoleRepository;
use Illuminate\Support\Arr;
use Illuminate\Validation\ValidationException;

class MarginRepository
{
    protected $margin;
    protected $detail;
    protected $role;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Margin $margin,
        MarginDetail $detail,
        RoleRepository $role
    ) {
        $this->margin = $margin;
        $this->detail = $detail;
        $this->role = $role;
    }

    /**
     * Get all margins
     */
    public function getAll() : MarginCollection
    {
        return new MarginCollection($this->margin->all());
    }

    /**
     * Find margin with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Margin
    {
        $margin = $this->margin->find($id);

        if (! $margin) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('setting.margin.margin')])]);
        }

        return $margin;
    }

    /**
     * Find margin with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Margin
    {
        $margin = $this->margin->filterByUuid($uuid)->first();

        if (! $margin) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('setting.margin.margin')])]);
        }

        return $margin;
    }

    /**
     * Paginate all margins
     */
    public function paginate() : MarginCollection
    {
        $sort_by      = $this->margin->getSortBy();
        $order        = $this->margin->getOrder();
        $per_page     = $this->margin->getPerPage();
        $current_page = $this->margin->getCurrentPage();

        $name     = request()->query('name');
        $keywords = request()->query('keywords');

        $query = $this->margin->filterByName($name)->search($keywords);

        return new MarginCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get margin pre requisite
     */
    public function getPreRequisite() : array
    {
        $roles = $this->role->getAll();

        return compact('roles');
    }

    /**
     * Create a new margin
     */
    public function create() : Margin
    {
        \DB::beginTransaction();

        $margin = $this->margin->forceCreate($this->formatParams());

        $this->updateDetails($margin);

        \DB::commit();

        return $margin;
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $formatted = [
            'name'        => request('name'),
            'description' => request('description'),
        ];

        return $formatted;
    }

    /**
     * Update margin details
     *
     * @param Margin $margin
     */
    private function updateDetails(Margin $margin) : void
    {
        $roles = Arr::pluck($this->role->getAll(), 'name');

        foreach (request('details', []) as $index => $detail) {
            $discounts = Arr::get($detail, 'discounts', []);

            foreach ($discounts as $discount) {
                if (! in_array(Arr::get($discount, 'uuid'), $roles)) {
                    throw ValidationException::withMessages(['details.' . $index . '.value' => trans('global.could_not_find', ['attribute' => trans('config.role.role')])]);
                }

                $discount_value = Arr::get($discount, 'value', 0);

                if (! is_numeric($discount_value)) {
                    throw ValidationException::withMessages(['details.' . $index . '.value' => trans('validation.numeric', ['attribute' => trans('setting.discount.discount')])]);
                }

                if ($discount_value < 0) {
                    throw ValidationException::withMessages(['details.' . $index . '.value' => trans('validation.min.numeric', ['attribute' => trans('setting.discount.discount'), 'min' => 0])]);
                }

                if ($discount_value > 100) {
                    throw ValidationException::withMessages(['details.' . $index . '.value' => trans('validation.max.numeric', ['attribute' => trans('setting.discount.discount'), 'max' => 100])]);
                }

                // if ($discount_value > Arr::get($detail, 'value')) {
                //     throw ValidationException::withMessages(['details.' . $index . '.min_amount' => trans('validation.max.numeric', ['attribute' => trans('setting.discount.discount'), 'max' => Arr::get($detail, 'value')])]);
                // }
            }

            $margin_detail = $this->detail->firstOrCreate([
                'margin_id' => $margin->id,
                'min_amount' => Arr::get($detail, 'min_amount'),
                'max_amount' => Arr::get($detail, 'max_amount')
            ]);

            $margin_detail->value = Arr::get($detail, 'value');
            $margin_detail->discounts = array_map(function($discount) {
                return array('name' => $discount['uuid'], 'value' => (float) $discount['value']);
            }, $discounts);
            $margin_detail->save();
        }
    }

    /**
     * Update given margin
     * @param Margin $margin
     */
    public function update(Margin $margin) : Margin
    {
        \DB::beginTransaction();

        $margin->forceFill($this->formatParams($margin->uuid))->save();

        $this->updateDetails($margin);

        \DB::commit();

        return $margin;
    }

    /**
     * Delete margin
     * @param Margin $margin
     */
    public function delete(Margin $margin) : void
    {
        if ($margin->itemCategories->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.margin.margin'), 'dependency' => trans('inventory.item_category.item_categories')])]);
        }

        if ($margin->items->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.margin.margin'), 'dependency' => trans('inventory.item.item')])]);
        }

        $margin->delete();
    }
}
