<?php

namespace App\Repositories\Setting;

use App\Http\Resources\Setting\TaxCollection;
use App\Models\Setting\Tax;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Arr;
use Illuminate\Validation\ValidationException;

class TaxRepository
{
    protected $tax;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Tax $tax
    ) {
        $this->tax = $tax;
    }

    /**
     * Get all taxes
     */
    public function getAll() : TaxCollection
    {
        return new TaxCollection($this->tax->orderBy('name', 'asc')->get());
    }

    /**
     * Get all taxes
     */
    public function get() : Collection
    {
        return $this->tax->orderBy('name', 'asc')->get();
    }

    /**
     * Find tax with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Tax
    {
        $tax = $this->tax->find($id);

        if (! $tax) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('setting.tax.tax')])]);
        }

        return $tax;
    }

    /**
     * Find tax with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Tax
    {
        $tax = $this->tax->filterByUuid($uuid)->first();

        if (! $tax) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('setting.tax.tax')])]);
        }

        return $tax;
    }

    /**
     * Paginate all taxes
     */
    public function paginate() : TaxCollection
    {
        $sort_by      = $this->tax->getSortBy();
        $order        = $this->tax->getOrder('asc');
        $per_page     = $this->tax->getPerPage();
        $current_page = $this->tax->getCurrentPage();

        $name     = request()->query('name');
        $code     = request()->query('code');
        $keywords = request()->query('keywords');

        $query = $this->tax->filterByName($name)->filterByCode($code)->search($keywords);

        return new TaxCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get tax pre requisite
     */
    public function getPreRequisite() : array
    {
        return [];
    }

    /**
     * Create a new tax
     */
    public function create() : Tax
    {
        return $this->tax->forceCreate($this->formatParams());
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $formatted = [
            'name'        => request('name'),
            'code'        => request('code'),
            'value'       => request('value'),
            'details'     => $this->validateDetails(),
            'description' => request('description'),
            'meta'        => array(
                'calculate_on_total' => (bool) request('calculate_on_total'),
                'voucher_level_tax'  => (bool) request('voucher_level_tax'),
            )
        ];

        return $formatted;
    }

    /**
     * Validate input
     */
    private function validateDetails() : array
    {
        $details = request('details', []);

        if (! $details) {
            return [];
        }

        $total = 0;

        $labels = Arr::pluck($details, 'label');

        if (count($labels) > count(array_unique($labels))) {
            throw ValidationException::withMessages(['message' => trans('setting.tax.duplicate_label')]);
        }

        foreach ($details as $detail) {
            $total += Arr::get($detail, 'value', 0);
        }

        if ($total != request('value')) {
            throw ValidationException::withMessages(['message' => trans('setting.tax.total_mismatch')]);
        }

        return array_map(function($item) {
            return array('label' => $item['label'], 'value' => (float) $item['value']);
        }, $details);
    }

    /**
     * Update given tax
     * @param Tax $tax
     */
    public function update(Tax $tax) : Tax
    {
        $tax->forceFill($this->formatParams($tax->uuid))->save();

        return $tax;
    }

    /**
     * Delete tax
     * @param Tax $tax
     */
    public function delete(Tax $tax) : void
    {
        if ($tax->itemCategories->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.tax.tax'), 'dependency' => trans('inventory.item_category.item_categories')])]);
        }

        if ($tax->items->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.tax.tax'), 'dependency' => trans('inventory.item.item')])]);
        }

        if ($tax->voucherTaxes->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.tax.tax'), 'dependency' => trans('voucher.voucher')])]);
        }

        $tax->delete();
    }
}
