<?php

namespace App\Repositories\Setting;

use App\Http\Resources\Setting\UnitCollection;
use App\Models\Setting\Unit;
use Illuminate\Validation\ValidationException;

class UnitRepository
{
    protected $unit;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Unit $unit
    ) {
        $this->unit = $unit;
    }

    /**
     * Get all units
     */
    public function getAll() : UnitCollection
    {
        return new UnitCollection($this->unit->orderBy('name', 'asc')->get());
    }

    /**
     * Find unit with given id or throw an error
     * @param integer $id
     */
    public function findOrFail($id, $field = 'message') : Unit
    {
        $unit = $this->unit->find($id);

        if (! $unit) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('setting.unit.unit')])]);
        }

        return $unit;
    }

    /**
     * Find unit with given uuid or throw an error
     * @param string $uuid
     */
    public function findByUuidOrFail($uuid, $field = 'message') : Unit
    {
        $unit = $this->unit->filterByUuid($uuid)->first();

        if (! $unit) {
            throw ValidationException::withMessages([$field => __('global.could_not_find', ['attribute' => __('setting.unit.unit')])]);
        }

        return $unit;
    }

    /**
     * Paginate all units
     */
    public function paginate() : UnitCollection
    {
        $sort_by      = $this->unit->getSortBy();
        $order        = $this->unit->getOrder('asc');
        $per_page     = $this->unit->getPerPage();
        $current_page = $this->unit->getCurrentPage();

        $name     = request()->query('name');
        $code     = request()->query('code');
        $keywords = request()->query('keywords');

        $query = $this->unit->filterByName($name)->filterByCode($code)->search($keywords);

        return new UnitCollection($query->orderBy($sort_by, $order)->paginate((int) $per_page, ['*'], 'current_page'));
    }

    /**
     * Get unit pre requisite
     */
    public function getPreRequisite() : array
    {
        return [];
    }

    /**
     * Create a new unit
     */
    public function create() : Unit
    {
        return $this->unit->forceCreate($this->formatParams());
    }

    /**
     * Prepare given params for inserting into database
     * @param uuid $uuid
     */
    private function formatParams($uuid = null) : array
    {
        $formatted = [
            'name'          => request('name'),
            'code'          => request('code'),
            'decimal_point' => request('decimal_point'),
            'description'   => request('description'),
        ];

        return $formatted;
    }

    /**
     * Update given unit
     * @param Unit $unit
     */
    public function update(Unit $unit) : Unit
    {
        $unit->forceFill($this->formatParams($unit->uuid))->save();

        return $unit;
    }

    /**
     * Delete unit
     * @param Unit $unit
     */
    public function delete(Unit $unit) : void
    {
        if ($unit->itemCategories->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.unit.unit'), 'dependency' => trans('inventory.item_category.item_categories')])]);
        }

        if ($unit->items->count()) {
            throw ValidationException::withMessages(['message' => trans('global.associated_with_dependency', ['attribute' => trans('setting.unit.unit'), 'dependency' => trans('inventory.item.item')])]);
        }

        $unit->delete();
    }
}
