<?php

namespace App\Repositories;

use App\Http\Resources\TagCollection;
use App\Models\Accounting\Ledger;
use App\Models\Accounting\Transaction;
use App\Models\Accounting\Voucher;
use App\Models\Contact;
use App\Models\Inventory\Brand;
use App\Models\Inventory\Item;
use App\Models\Inventory\ItemCategory;
use App\Models\Tag;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class TagRepository
{
    protected $tag;
    protected $item;
    protected $item_category;
    protected $voucher;
    protected $ledger;
    protected $transaction;
    protected $contact;
    protected $brand;

    /**
     * Instantiate a new instance
     * @return void
     */
    public function __construct(
        Tag $tag,
        Item $item,
        ItemCategory $item_category,
        Voucher $voucher,
        Ledger $ledger,
        Transaction $transaction,
        Contact $contact,
        Brand $brand
    ) {
        $this->tag = $tag;
        $this->item = $item;
        $this->item_category = $item_category;
        $this->voucher = $voucher;
        $this->ledger = $ledger;
        $this->transaction = $transaction;
        $this->contact = $contact;
        $this->brand = $brand;
    }

    /**
     * Search all tags
     */
    public function search() : TagCollection
    {
        $q = request()->query('q');

        $query = $this->tag->search($q);

        return new TagCollection($query->orderBy('name', 'asc')->limit(5)->get());
    }

    /**
     * Update tag
     */
    public function update() : void
    {
        $allowed_types = array(
            'procurement',
            'purchase-enquiry',
            'purchase-order',
            'purchase',
            'sale-enquiry',
            'quotation',
            'sale',
            'item',
            'item-category',
            'ledger',
            'transaction',
            'contact',
            'brand'
        );

        $type = request('type');

        if (! in_array($type, $allowed_types)) {
            throw ValidationException::withMessages(['message' => trans('global.could_not_find', ['attribute' => trans('general.tag_type')])]);
        }

        if (! \Auth::user()->can('edit-'.$type)) {
            throw ValidationException::withMessages(['message' => trans('user.permission_denied')]);
        }

        $models = $this->getModel();

        $tag_ids = array();
        foreach (request('tags', []) as $tag) {
            $tag = Tag::firstOrCreate([
                'name' => Arr::get($tag, 'name'),
                'slug' => Str::slug(Arr::get($tag, 'name')),
            ]);

            if (! in_array($tag->id, $tag_ids)) {
                $tag_ids[] = $tag->id;
            }
        }

        foreach ($models as $model) {
            if (request('action') == 'add') {
                $model->tags()->syncWithoutDetaching($tag_ids);
            } else {
                $model->tags()->detach($tag_ids);
            }
        }
    }

    private function getModel() : ?Collection
    {
        if (request('type') == 'item') {
            return $this->item->whereIn('uuid', request('uuids'))->get();
        } else if (request('type') == 'item-category') {
            return $this->item_category->whereIn('uuid', request('uuids'))->get();
        } else if (request('type') == 'ledger') {
            return $this->ledger->whereIn('uuid', request('uuids'))->get();
        } else if (in_array(request('type'), ['procurement', 'purchase-enquiry', 'purchase-order', 'purchase', 'sale-enquiry', 'quotation', 'sale', 'challan', 'purchase-return', 'sale-return'] )) {
            return $this->voucher->whereIn('uuid', request('uuids'))->get();
        } else if (request('type') == 'transaction') {
            return $this->transaction->whereIn('uuid', request('uuids'))->get();
        } else if (request('type') == 'contact') {
            return $this->contact->whereIn('uuid', request('uuids'))->get();
        } else if (request('type') == 'brand') {
            return $this->brand->whereIn('uuid', request('uuids'))->get();
        }

        return null;
    }
}
