<?php

namespace App\Traits;

use App\Enums\LedgerGroup;
use App\Enums\VoucherType;

trait VoucherConstraints
{
    /**
     * Get quantity multiplier (BASE = PURCHASE)
     */
    private function getQuantityMultiplier() : int
    {
        if (in_array(request('type'), [VoucherType::SALE, VoucherType::PURCHASE_RETURN])) {
            return -1;
        }

        return 1;
    }

    /**
     * Get ledger multiplier (BASE = SALE)
     */
    private function getLedgerMultiplier($ledger, $type = null) : int
    {
        if (! $type) {
            $type = request('type');
        }

        $value = 1;
        $ledger_type = $ledger->type->code;

        if ($type == VoucherType::PURCHASE && $ledger_type == LedgerGroup::SUNDRY_DEBTOR
            || $type == VoucherType::PURCHASE && $ledger_type == LedgerGroup::CASH
            || $type == VoucherType::PURCHASE_RETURN && $ledger_type == LedgerGroup::SUNDRY_DEBTOR
            || $type == VoucherType::PURCHASE_RETURN && $ledger_type == LedgerGroup::CASH
            || $type == VoucherType::SALE && $ledger_type == LedgerGroup::SUNDRY_CREDITOR
            || $type == VoucherType::SALE_RETURN && $ledger_type == LedgerGroup::SUNDRY_CREDITOR
        ) {
            $value = -1;
        }

        return $value;
    }

    /**
     * Voucher affects quantity
     */
    private function doesAffectQuantity() : bool
    {
        if (in_array(request('type'), [
            VoucherType::CHALLAN,
            VoucherType::PURCHASE_ORDER,
            VoucherType::QUOTATION,
            VoucherType::PURCHASE_ENQUIRY,
            VoucherType::SALE_ENQUIRY,
            VoucherType::PROCUREMENT
        ])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher affects ledger
     */
    private function doesAffectLedger() : bool
    {
        if (in_array(request('type'), [
            VoucherType::CHALLAN,
            VoucherType::PURCHASE_ORDER,
            VoucherType::QUOTATION,
            VoucherType::PURCHASE_ENQUIRY,
            VoucherType::SALE_ENQUIRY,
            VoucherType::PROCUREMENT
        ])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher should have valid code number
     */
    private function shouldHaveValidCodeNumber() : bool
    {
        if (in_array(request('type'), [VoucherType::PURCHASE])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher can have valid custom item
     */
    private function canHaveCustomItem() : bool
    {
        if (in_array(request('type'), [VoucherType::CHALLAN, VoucherType::QUOTATION, VoucherType::PURCHASE_ORDER, VoucherType::PURCHASE_ENQUIRY, VoucherType::SALE_ENQUIRY, VoucherType::PROCUREMENT])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher requires actionable status
     */
    private function requiresActionableStatus() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE_ENQUIRY, VoucherType::PURCHASE_ENQUIRY, VoucherType::PROCUREMENT])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher should have valid total
     */
    private function shouldHaveValidTotal() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE_ENQUIRY, VoucherType::PURCHASE_ENQUIRY, VoucherType::PROCUREMENT])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher requires enquiry related attributes
     */
    private function requiresEnquiryAttributes() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE_ENQUIRY])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher can have voucher level discount
     */
    private function canHaveVoucherLevelDiscount() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE_ENQUIRY, VoucherType::PURCHASE_ENQUIRY, VoucherType::PROCUREMENT])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher can have voucher level tax
     */
    private function canHaveVoucherLevelTax() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE_ENQUIRY, VoucherType::PURCHASE_ENQUIRY])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher can have tax
     */
    private function canHaveTax() : bool
    {
        if (in_array(request('type'), [VoucherType::PROCUREMENT])) {
            return false;
        }

        return true;
    }

    /**
     * Voucher doesn't have rate
     */
    private function doesNotHaveRate() : bool
    {
        if (in_array(request('type'), [VoucherType::PROCUREMENT])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher doesn't have discount
     */
    private function doesNotHaveDiscount() : bool
    {
        if (in_array(request('type'), [VoucherType::PROCUREMENT])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher has fulfillment
     */
    private function hasFulfillment() : bool
    {
        if (in_array(request('type'), [VoucherType::PROCUREMENT])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher can have non taxed item
     */
    private function canHaveNonTaxedItem() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE, VoucherType::CHALLAN])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher can have tax summary
     */
    private function canHaveTaxSummary() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE, VoucherType::PURCHASE, VoucherType::SALE_RETURN, VoucherType::PURCHASE_RETURN])) {
            return true;
        }

        return false;
    }

    /**
     * Voucher can have payment status
     */
    private function canHavePaymentStatus() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE, VoucherType::PURCHASE, VoucherType::SALE_RETURN, VoucherType::PURCHASE_RETURN])) {
            return true;
        }

        return false;
    }

    private function needsItemRateUpdate() : bool
    {
        if (in_array(request('type'), [VoucherType::SALE, VoucherType::PURCHASE, VoucherType::SALE_RETURN, VoucherType::PURCHASE_RETURN])) {
            return true;
        }

        return false;
    }
}
