import { formatKeysToCamelCase } from '@js/core/utils/formatter'
import { formatDecimal, getOriginalPrice, removePercentage } from '@core/utils'

export default {
    props: {
        index: {
            type: Number,
            required: true
        },
        editing: {
            type: Boolean,
            default: false
        },
        formData: {
            type: Object,
            required: true,
        },
        formLabels: {
            type: Object,
            required: true,
        },
        config: {
            type: Object,
            required: true,
        },
        formErrors: {
            type: [String, Object, Array],
            default: null,
        },
        preRequisite: {
            type: Object,
            default() {
                return {}
            }
        },
        currency: {
            type: Object,
            default: null
        },
        discounts: {
            type: Array,
            default: null,
        },
    },
    data() {
        return {
            lastFieldUpdated: 'rate',
            showDescription: false,
            lastDiscounts: [],
            lastFormData: null,
            rowErrors: {},
        }
    },
    computed: {
        computedTaxesArray() {
            return this.preRequisite.taxes ? this.preRequisite.taxes.filter(t => !t.voucherLevelTax) : []
        },
        computedTaxValue() {
            return this.formData.tax ? this.formData.tax.value : 0
        },
        computedUnit() {
            return this.formData.item && this.formData.item.unit ? this.formData.item.unit.code : ''
        },
        computedStep() {
            return this.formData.item && this.formData.item.unit ? (1/Math.pow(10, this.formData.item.unit.decimalPoint)) : 0.01
        },
        computedIsNonTaxed: {
            get() {
                return this.config.isNonTaxed || this.formData.isNonTaxed
            },
            set(newVal) {
                this.formData.isNonTaxed = newVal

                if(!newVal && this.formData.item && this.formData.item.tax) {
                    this.formData.tax = this.formData.item.tax
                } else {
                    this.formData.tax = null
                }

                this.calculate()
                if(!newVal && this.formData.custom) {
                    this.focusTax()
                }
            }
        },
        configIsNonTaxed() {
            return this.config.isNonTaxed
        }
    },
    watch: {
        formData: {
            deep: true,
            handler: _.debounce(function(v) {
                this.$emit('updated')
            }, 500),
        },
        formErrors: {
            deep: true,
            handler: _.debounce(function(v) {
                this.rowErrors = v && v.isArray ? formatKeysToCamelCase(JSON.parse(v)) : {}
            }, 500),
        },
        discounts: {
            deep: true,
            handler: _.debounce(function(newVal) {
                if(! _.isArrayEqual(newVal, this.lastDiscounts)) {
                    this.calculate()
                }
            }, 500),
        },
        configIsNonTaxed(val) {
            if(!val && this.formData.item && this.formData.item.tax) {
                this.formData.tax = this.formData.item.tax
            } else {
                this.formData.tax = null
            }

            this.calculate()
        }
    },
    methods: {
        focusTax() {
            this.$nextTick(() => {
                setTimeout(() => {
                    if (this.$refs.tax) {
                        this.$refs.tax.focus()
                    }
                }, 500)
            })
        },
        toggleDescription() {
            this.showDescription = this.showDescription ? false : true
        },
        formatCurrency(val) {
            return formatDecimal(val, this.currency.decimal)
        },
        saveLastFormData() {
            this.lastFormData = _.cloneDeep(this.formData)
            this.lastDiscounts = _.cloneDeep(this.discounts)
        },

        addPercentage(val, perc) {
            return Number(Number(val) * this.formatCurrency( 1 + (Number(perc) / 100)) )
        },
        getDiscountedPrice(val, perc) {
            return Number(Number(val) - this.formatCurrency(val * (Number(perc) / 100)) )
        },


        getOuterDiscountedAmount(amount, reverse = false) {
            if(this.discounts && this.discounts.length) {
                let discounts = _.cloneDeep(this.discounts)
                discounts = reverse ? discounts.reverse() : discounts
                discounts.forEach(discount => {
                    if(discount.value) {
                        if(reverse) {
                            amount = this.formatCurrency( getOriginalPrice(amount, discount.value) )
                        } else {
                            amount = this.formatCurrency( this.getDiscountedPrice(amount, discount.value) )
                        }
                    }
                })
            }
            return amount
        },

        calculate() {
            if(this.formData.quantity) {

                const applicableDiscount = this.formData.discount || 0
                let rate = this.formData.rate,
                    rateWithDiscount = this.formData.rateWithDiscount,
                    amount = this.formData.amount,
                    amountWithDiscount = this.formData.amountWithDiscount

                // Calculate rate first

                if(this.lastFieldUpdated === 'rateWithTax') {

                    rate = removePercentage(this.formData.rateWithTax, this.computedTaxValue)

                } else if(this.lastFieldUpdated === 'rateWithDiscount') {

                    rate = getOriginalPrice(this.formData.rateWithDiscount, applicableDiscount)

                } else if(this.lastFieldUpdated === 'rateWithTaxDiscount') {

                    rateWithDiscount = removePercentage(this.formData.rateWithTaxDiscount, this.computedTaxValue)

                    rate = getOriginalPrice(rateWithDiscount, applicableDiscount)

                } else if(this.lastFieldUpdated === 'amount') {

                    rate = this.formData.amount / this.formData.quantity

                } else if(this.lastFieldUpdated === 'amountWithTax') {

                    amount = removePercentage(this.formData.amountWithTax, this.computedTaxValue)

                    rate = amount / this.formData.quantity

                } else if(this.lastFieldUpdated === 'amountWithDiscount') {

                    amount = getOriginalPrice(amount, applicableDiscount)

                    rate = amount / this.formData.quantity

                } else if(this.lastFieldUpdated === 'amountWithTaxDiscount') {

                    amountWithDiscount = removePercentage(this.formData.amountWithTaxDiscount, this.computedTaxValue)

                    amount = getOriginalPrice(amountWithDiscount, applicableDiscount)

                    rate = amount / this.formData.quantity

                }

                this.formData.rate = rate = this.formatCurrency(rate)


                // Now, calculate other values from rate

                if(this.lastFieldUpdated !== 'rateWithTax') {

                    this.formData.rateWithTax = this.formatCurrency( this.addPercentage(rate, this.computedTaxValue) )

                }

                if(this.lastFieldUpdated !== 'rateWithDiscount') {

                    this.formData.rateWithDiscount = this.formatCurrency( this.getDiscountedPrice(rate, applicableDiscount) )

                }

                if(this.lastFieldUpdated !== 'rateWithTaxDiscount') {

                    this.formData.rateWithTaxDiscount = this.formatCurrency( this.addPercentage(this.formData.rateWithDiscount, this.computedTaxValue) )

                }

                if(this.lastFieldUpdated !== 'amount') {

                    this.formData.amount = this.formatCurrency( rate * this.formData.quantity )

                }

                if(this.lastFieldUpdated !== 'amountWithTax') {

                    this.formData.amountWithTax = this.formatCurrency( this.addPercentage(this.formData.amount, this.computedTaxValue) )

                }

                if(this.lastFieldUpdated !== 'amountWithDiscount') {

                    this.formData.amountWithDiscount = this.formatCurrency( this.getDiscountedPrice(this.formData.amount, applicableDiscount) )

                }

                if(this.lastFieldUpdated !== 'amountWithTaxDiscount') {

                    this.formData.amountWithTaxDiscount = this.formatCurrency( this.addPercentage(this.formData.amountWithDiscount, this.computedTaxValue) )

                }

                this.formData.amountWithAllDiscounts = this.formatCurrency( this.getOuterDiscountedAmount( this.formData.amountWithDiscount ) )

                this.formData.amountTotal = this.formatCurrency( this.addPercentage(this.formData.amountWithAllDiscounts, this.computedTaxValue) )

            } else {
                if(this.lastFieldUpdated !== 'rate') {
                    this.formData.rate = ''
                }
                if(this.lastFieldUpdated !== 'rateWithTax') {
                    this.formData.rateWithTax = ''
                }
                if(this.lastFieldUpdated !== 'rateWithDiscount') {
                    this.formData.rateWithDiscount = ''
                }
                if(this.lastFieldUpdated !== 'rateWithTaxDiscount') {
                    this.formData.rateWithTaxDiscount = ''
                }
                if(this.lastFieldUpdated !== 'amount') {
                    this.formData.amount = ''
                }
                if(this.lastFieldUpdated !== 'amountWithTax') {
                    this.formData.amountWithTax = ''
                }
                if(this.lastFieldUpdated !== 'amountWithDiscount') {
                    this.formData.amountWithDiscount = ''
                }
                if(this.lastFieldUpdated !== 'amountWithTaxDiscount') {
                    this.formData.amountWithTaxDiscount = ''
                }

                this.formData.discount = ''

                this.formData.amountWithAllDiscounts = ''
                this.formData.amountTotal = ''
            }

            if(! formUtil.isUnchanged(this.lastFormData, this.formData)) {
                this.$emit('updateTotal')
            }
            this.saveLastFormData()
        },



        itemChanged() {
            if(this.formData.item) {
                if(this.formData.item.tax && !this.computedIsNonTaxed) {
                    this.formData.tax = this.formData.item.tax
                } else {
                    this.formData.tax = null
                }

                const voucherType = this.config && this.config.rateType ? this.config.rateType : 'sale'

                if(this.formData.isNonTaxed) {
                    if(voucherType === 'purchase' && this.formData.item.currentRateNonTaxed) {
                        this.formData.rate = this.formData.item.currentRateNonTaxed
                    } else if(voucherType === 'sale' && this.formData.item.rateNonTaxed) {
                        this.formData.rate = this.formData.item.rateNonTaxed
                    }
                } else {
                    if(voucherType === 'purchase' && this.formData.item.currentRateTaxed) {
                        this.formData.rate = this.formData.item.currentRateTaxed
                    } else if(voucherType === 'sale' && this.formData.item.rateTaxed) {
                        this.formData.rate = this.formData.item.rateTaxed
                    }
                }

                this.calculate()
            } else {
                this.formData.quantity = this.config.hasQuantity ? '' : 1
                this.formData.rate = ''
                this.formData.tax = null
                this.formData.rateWithTax = ''
                this.formData.discount = ''
                this.formData.rateWithDiscount = ''
                this.formData.rateWithTaxDiscount = ''
                this.formData.amount = ''
                this.formData.amountWithTax = ''
                this.formData.amountWithDiscount = ''
                this.formData.amountWithTaxDiscount = ''
                this.formData.amountWithAllDiscounts = ''
                this.formData.amountTotal = ''
                this.saveLastFormData()
            }
        },
        rateChanged: _.debounce(function() {
            if(this.formData.rate == this.lastFormData.rate) {
                return
            }

            this.lastFieldUpdated = 'rate'
            this.calculate()

        }, 200),
        rateWithTaxChanged: _.debounce(function() {
            if(this.formData.rateWithTax == this.lastFormData.rateWithTax) {
                return
            }

            this.lastFieldUpdated = 'rateWithTax'
            this.calculate()

        }, 200),
        taxChanged: _.debounce(function() {
            if(this.formData.tax && this.lastFormData.tax && this.formData.tax.value == this.lastFormData.tax.value) {
                return
            }

            this.calculate()

        }, 200),
        discountChanged: _.debounce(function() {
            if(this.formData.discount == this.lastFormData.discount) {
                return
            }

            this.calculate()

        }, 200),
        quantityChanged: _.debounce(function() {
            if(this.formData.quantity == this.lastFormData.quantity) {
                return
            }

            this.calculate()

        }, 200),
        rateWithDiscountChanged: _.debounce(function() {
            if(this.formData.rateWithDiscount == this.lastFormData.rateWithDiscount) {
                return
            }

            this.lastFieldUpdated = 'rateWithDiscount'
            this.calculate()

        }, 200),
        rateWithTaxDiscountChanged: _.debounce(function() {
            if(this.formData.rateWithTaxDiscount == this.lastFormData.rateWithTaxDiscount) {
                return
            }

            this.lastFieldUpdated = 'rateWithTaxDiscount'
            this.calculate()

        }, 200),
        amountChanged: _.debounce(function() {
            if(this.formData.amount == this.lastFormData.amount) {
                return
            }

            this.lastFieldUpdated = 'amount'
            this.calculate()

        }, 200),
        amountWithTaxChanged: _.debounce(function() {
            if(this.formData.amountWithTax == this.lastFormData.amountWithTax) {
                return
            }

            this.lastFieldUpdated = 'amountWithTax'
            this.calculate()

        }, 200),
        amountWithDiscountChanged: _.debounce(function() {
            if(this.formData.amountWithDiscount == this.lastFormData.amountWithDiscount) {
                return
            }

            this.lastFieldUpdated = 'amountWithDiscount'
            this.calculate()

        }, 200),
        amountWithTaxDiscountChanged: _.debounce(function() {
            if(this.formData.amountWithTaxDiscount == this.lastFormData.amountWithTaxDiscount) {
                return
            }

            this.lastFieldUpdated = 'amountWithTaxDiscount'
            this.calculate()

        }, 200),


        nonTaxedCheckChanged() {
            if(!this.formData.isNonTaxed && this.formData.item && this.formData.item.tax) {
                this.formData.tax = this.formData.item.tax
            } else {
                this.formData.tax = null
            }

            if(!this.formData.rate) {
                const voucherType = this.config && this.config.rateType ? this.config.rateType : 'sale'

                if(this.formData.isNonTaxed) {
                    if(voucherType === 'purchase' && this.formData.item.currentRateNonTaxed) {
                        this.formData.rate = this.formData.item.currentRateNonTaxed
                    } else if(voucherType === 'sale' && this.formData.item.rateNonTaxed) {
                        this.formData.rate = this.formData.item.rateNonTaxed
                    }
                } else {
                    if(voucherType === 'purchase' && this.formData.item.currentRateTaxed) {
                        this.formData.rate = this.formData.item.currentRateTaxed
                    } else if(voucherType === 'sale' && this.formData.item.rateTaxed) {
                        this.formData.rate = this.formData.item.rateTaxed
                    }
                }
            }

            this.calculate()
        },
    },
    mounted() {
        if(!this.config.hasQuantity) {
            this.formData.quantity = 1
        }

        this.formData.rate = this.formData.rate ? this.formatCurrency( this.formData.rate ) : this.formData.rate

        this.formData.rateWithTax = this.formData.rateWithTax ? this.formatCurrency( this.formData.rateWithTax ) : this.formData.rateWithTax

        this.formData.rateWithDiscount = this.formData.rateWithDiscount ? this.formatCurrency( this.formData.rateWithDiscount ) : this.formData.rateWithDiscount

        this.formData.rateWithTaxDiscount = this.formData.rateWithTaxDiscount ? this.formatCurrency( this.formData.rateWithTaxDiscount ) : this.formData.rateWithTaxDiscount

        this.formData.amount = this.formData.amount ? this.formatCurrency( this.formData.amount ) : this.formData.amount

        this.formData.amountWithTax = this.formData.amountWithTax ? this.formatCurrency( this.formData.amountWithTax ) : this.formData.amountWithTax

        this.formData.amountWithDiscount = this.formData.amountWithDiscount ? this.formatCurrency( this.formData.amountWithDiscount ) : this.formData.amountWithDiscount

        this.formData.amountWithTaxDiscount = this.formData.amountWithTaxDiscount ? this.formatCurrency( this.formData.amountWithTaxDiscount ) : this.formData.amountWithTaxDiscount

        this.formData.amountWithAllDiscounts = this.formData.amountWithAllDiscounts ? this.formatCurrency( this.formData.amountWithAllDiscounts ) : this.formData.amountWithAllDiscounts

        this.formData.amountTotal = this.formData.amountTotal ? this.formatCurrency( this.formData.amountTotal ) : this.formData.amountTotal

        this.lastFieldUpdated = this.formData.lastFieldUpdated || this.lastFieldUpdated

        this.calculate()

        this.$nextTick(() => {
            if (this.$refs.item && this.index !== 0 && !(this.formData && this.formData.item && this.formData.item.name)) {
                this.$refs.item.focus()
            }
        })
    }
}
