<?php

use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

// Guest Routes
Route::namespace('Auth')->prefix('auth')->group(function () {
    Route::post('login/otp', 'LoginWithOtp');
    Route::post('login', 'LoginController@login');

    Route::post('register', 'RegisterController@register');
    Route::post('verify', 'RegisterController@verify');

    Route::post('password', 'ResetPasswordController@password');
    Route::post('validate-reset-password', 'ResetPasswordController@validateCode');
    Route::post('reset', 'ResetPasswordController@reset');
});

// Global Routes
Route::namespace('Config')->group(function () {
    Route::get('config/pre-requisite', 'ConfigController@preRequisite');
    Route::group(['middleware' => ['under_maintenance']], function () {
        Route::get('config', 'ConfigController@index');
    });
});

Route::group(['middleware' => ['auth:sanctum']], function () {

    // Auth Routes
    Route::namespace('Auth')->prefix('auth')->group(function () {
        Route::post('logout', 'LoginController@logout');
        Route::post('lock', 'LockScreen');
        Route::get('user', 'UserController@me');
        Route::post('security', 'TwoFactorSecurity');
    });
});

Route::group(['middleware' => ['auth:sanctum', '2fa', 'under_maintenance']], function () {

    // Auth Routes
    Route::namespace('Auth')->prefix('auth')->group(function () {
        Route::post('change-password', 'ChangePassword');
    });

    // User Routes
    Route::namespace('Auth')->prefix('user')->group(function () {
        Route::post('preference', 'UserController@preference');
    });

    Route::namespace('Auth')->group(function () {
        Route::get('users/pre-requisite', 'UserController@preRequisite');
        Route::post('users/{user}/status', 'UserController@updateStatus');
        Route::post('users/subscriptions', 'UserController@updateSubscription');
        Route::post('users/subscriptions/delete', 'UserController@deleteSubscription');
        Route::apiResource('users', 'UserController');

        Route::post('profile', 'ProfileController@update');
        Route::post('profile/avatar', 'ProfileController@uploadAvatar');
        Route::delete('profile/avatar', 'ProfileController@removeAvatar');
    });

    // Dashboard Routes
    Route::get('dashboard', 'DashboardController@index');
    Route::get('dashboard/stats', 'DashboardController@getStats');
    Route::get('dashboard/chart', 'DashboardController@getChart');

    // Any key search
    Route::get('search', 'Search');

    Route::get('tags', 'TagController@search');
    Route::post('tags', 'TagController@update');

    Route::post('uploads/image', 'UploadController@image');

    // Config Routes
    Route::namespace('Config')->prefix('config')->group(function () {
        Route::post('', 'ConfigController@store');
        Route::post('notification', 'ConfigController@notification');
        Route::get('notification', 'ConfigController@showDemoNotification');
        Route::post('assets', 'ConfigController@uploadAsset');
        Route::delete('assets', 'ConfigController@removeAsset');

        Route::delete('roles/{name}', 'RoleController@destroy');
        Route::apiResource('roles', 'RoleController')->except(['update', 'destroy']);

        Route::get('permissions/pre-requisite', 'PermissionController@preRequisite');
        Route::post('permissions/assign', 'PermissionController@assign');

        Route::get('locales/pre-requisite', 'LocaleController@preRequisite');
        Route::post('locales/{locale}/translate', 'LocaleController@translate');
        Route::apiResource('locales', 'LocaleController');
    });

    // Activity Routes
    Route::get('activities', 'ActivityController@index');

    Route::namespace('Setting')->prefix('setting')->group(function() {
        Route::apiResource('units', 'UnitController');
        Route::apiResource('taxes', 'TaxController');
        Route::get('margins/pre-requisite', 'MarginController@preRequisite')->name('margins.preRequisite');
        Route::apiResource('margins', 'MarginController');
    });

    // Option Routes
    Route::get('options/pre-requisite', 'OptionController@preRequisite');
    Route::apiResource('options', 'OptionController');

    Route::namespace('Inventory')->prefix('inventory')->group(function() {
        Route::post('brands/{brand}/contacts', 'BrandController@addContact')->name('brands.add-contact');
        Route::delete('brands/{brand}/contacts', 'BrandController@removeContact')->name('brands.remove-contact');
        Route::apiResource('brands', 'BrandController');

        Route::get('item-categories/pre-requisite', 'ItemCategoryController@preRequisite')->name('item-categories.preRequisite');
        Route::get('item-categories/search', 'ItemCategoryController@search')->name('item-categories.search');
        Route::apiResource('item-categories', 'ItemCategoryController');
        Route::get('items/pre-requisite', 'ItemController@preRequisite')->name('items.preRequisite');
        Route::get('items/search', 'ItemController@search')->name('items.search');
        Route::get('items/{item}/report', 'ItemReportController@getReport')->name('items.report');
        Route::apiResource('items', 'ItemController');

        Route::apiResource('stock-matches', 'StockMatchController');
    });

    Route::namespace('Accounting')->prefix('accounting')->group(function() {
        Route::get('ledger-types/pre-requisite', 'LedgerTypeController@preRequisite')->name('ledger-types.preRequisite');
        Route::get('ledger-types/search', 'LedgerTypeController@search')->name('ledger-types.search');
        Route::post('ledger-types/{ledger_type}/role', 'LedgerTypeController@updateRole')->name('ledger-types.role');
        Route::apiResource('ledger-types', 'LedgerTypeController');

        Route::get('ledgers/pre-requisite', 'LedgerController@preRequisite')->name('ledgers.preRequisite');
        Route::get('ledgers/search', 'LedgerController@search')->name('ledgers.search');
        Route::post('ledgers/{ledger}/contacts', 'LedgerController@addContact')->name('ledgers.add-contact');
        Route::delete('ledgers/{ledger}/contacts', 'LedgerController@removeContact')->name('ledgers.remove-contact');
        Route::post('ledgers/{ledger}/accounts', 'LedgerController@addAccount')->name('ledgers.add-account');
        Route::patch('ledgers/{ledger}/accounts/{account}', 'LedgerController@updateAccount')->name('ledgers.edit-account');
        Route::delete('ledgers/{ledger}/accounts', 'LedgerController@removeAccount')->name('ledgers.remove-account');
        Route::get('ledgers/{ledger}/report', 'LedgerReportController@getReport')->name('ledgers.report');
        Route::apiResource('ledgers', 'LedgerController');
        Route::apiResource('accounts', 'AccountController')->only(['index']);

        Route::prefix('voucher/{voucher}')->middleware(['voucher'])->group(function() {
            Route::get('pre-requisite', 'VoucherController@preRequisite')->name('vouchers.preRequisite');
            Route::get('{uuid}', 'VoucherController@show')->name('vouchers.show');
            Route::get('', 'VoucherController@index')->name('vouchers.index');
            Route::post('', 'VoucherController@store')->name('vouchers.store');
            Route::patch('{uuid}', 'VoucherController@update')->name('vouchers.update');
            Route::delete('{uuid}', 'VoucherController@destroy')->name('vouchers.destroy');
        });

        Route::prefix('voucher/{voucher}/{uuid}')->middleware(['voucher'])->group(function() {
            Route::post('follow-ups', 'FollowUpController@store')->name('follow-ups.store');
        });

        Route::get('pre-requisite', 'TransactionController@preRequisite')->name('transactions.preRequisite');
        Route::apiResource('transactions', 'TransactionController');
    });

    Route::get('reports/tax', 'ReportController@tax');
    Route::get('reports/daybook', 'ReportController@daybook');

    Route::get('contacts/search', 'ContactController@search')->name('contacts.search');
    Route::get('contacts/pre-requisite', 'ContactController@preRequisite')->name('contacts.preRequisite');
    Route::apiResource('contacts', 'ContactController');

    Route::prefix('chat')->group(function() {
        Route::get('pre-requisite', 'ChatController@preRequisite');
        Route::get('rooms', 'ChatController@getRooms');
        Route::get('rooms/search', 'ChatController@searchRoom');
        Route::get('rooms/{uuid}', 'ChatController@getRoom');
        Route::patch('rooms/{uuid}', 'ChatController@editRoom');
        Route::post('rooms', 'ChatController@createRoom')->name('chat.create-room');
        Route::post('rooms/{uuid}/sync', 'ChatController@syncMember')->name('chat.sync-member')->middleware(['role:admin']);
        Route::get('rooms/{uuid}/members', 'ChatController@listMember')->name('chat.list-member');
        Route::post('rooms/{uuid}/members', 'ChatController@addMember')->name('chat.add-member');
        Route::delete('rooms/{uuid}/members', 'ChatController@removeMember')->name('chat.remove-member');
        Route::get('messages', 'ChatController@getMessage');
        Route::post('messages', 'ChatController@storeMessage');
        Route::get('messages/search', 'ChatController@searchMessage');
    });

    // Utility Routes
    Route::namespace('Utility')->prefix('utility')->group(function () {
        Route::post('todos/{uuid}/status', 'TodoController@toggleStatus');
        Route::apiResource('todos', 'TodoController');
    });
});

Broadcast::routes(["middleware" => ['auth:sanctum', '2fa']]);

//Fallback route
Route::fallback(function () {
    return response()->json(['message' => trans('general.api_not_found')], 404);
});
